import numpy as np
import os
import h5py
from glob import glob
from sklearn.metrics import roc_auc_score
# import matplotlib
# matplotlib.use('cairo')
import matplotlib.pyplot as plt
from collections import namedtuple
from scipy import stats
# import itertools
import matplotlib as mpl
from matplotlib.patches import Ellipse
from matplotlib.legend_handler import HandlerPatch

mpl.rcParams['pdf.fonttype'] = 42
mpl.rcParams['ps.fonttype'] = 42

GOOD_ELECS_ONLY = False
# GOOD_ELECS_ONLY = True

if GOOD_ELECS_ONLY:
    class_path_suffix = '_goodchans'
else:
    class_path_suffix = ''
    


# pylint: disable=C0103

# plot_name = 'FR1toCatFR1'

run_on_rhino = False
# run_on_rhino = True
if run_on_rhino:
    rhino_mount = ''
else:
    rhino_mount = '/home/ctw/fusemounts/rhino'


# plot_params = {
#     'FR1toCatFR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_FR1toCatFR1_classifiers/' +
#             'RAM_FR1toCatFR1_classifiers_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'  # ,
#         # 'participants': 'participants',
#         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
#         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
#     },
#     'FR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_FR1lolo_classifiers_allfreqs/' +
#             'RAM_FR1lolo_classifiers_allfreqs_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'
#     },
#     'catFR1toFR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_catFR1toFR1_classifiers/' +
#             'RAM_catFR1toFR1_classifiers_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'  # ,
#         # 'participants': 'participants',
#         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
#         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
#     },
#     'catFR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_catFR1lolo_classifiers_allfreqs/' +
#             'RAM_catFR1lolo_classifiers_allfreqs_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'
#     }
# }


plot_params = {
    'retrieval': {
        'FR1toCatFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_FR1toCatFR1_classifiers/' +
                'RAM_FR1toCatFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'events': 'events2'  # ,
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'FR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs/' +
                'RAM_FR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target2',
            'events': 'events2'
        },
        'catFR1toFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_catFR1toFR1_classifiers/' +
                'RAM_catFR1toFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'events': 'events2'  # ,
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'catFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_catFR1lolo_classifiers_allfreqs/' +
                'RAM_catFR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target2',
            'events': 'events2'
        }
    },
    'encoding': {
        'FR1toCatFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_FR1toCatFR1_classifiers/' +
                'RAM_FR1toCatFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'events': 'events2'  # ,
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'FR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs/' +
                'RAM_FR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target2',
            'events': 'events2'
        },
        'catFR1toFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_catFR1toFR1_classifiers/' +
                'RAM_catFR1toFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'events': 'events2'  # ,
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'catFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_catFR1lolo_classifiers_allfreqs/' +
                'RAM_catFR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target2',
            'events': 'events2'
        },
    }   
}


# aucs = {}
# aucs_sorted = {}
# # Auc95 = namedtuple(
# #     'AUC', ['auc_score', 'percentile95', 'subj'])
# Auc95 = namedtuple(
#     'AUC', ['auc_score', 'auc_score_shuffles', 'percentile95', 'subj'])
# # for label in plot_params:  # ['FR1toCatFR1', 'catFR1toFR1']:
# for label in ['FR1toCatFR1', 'catFR1toFR1']:
#     classifier_path = plot_params[label]['classifier_path']
#     classifier_path_pickle = classifier_path + 'pickle_files/'
#     classifier_path_hdf5 = classifier_path + 'hdf5_files/'
#     c = 0.000719685673001
#     # classifier_c_paths = np.sort(glob(classifier_path_hdf5 + '*'))
#     classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
#                                     '/*_classifier_info.hdf5'))
#     aucs[label] = []
#     print label
#     for i, classifier_file in enumerate(classifier_files):
#         print i + 1, 'of', len(classifier_files)
#         subj = classifier_file.split('/')[-1].split('_classifier_info.hdf5')[0]
#         classdat = h5py.File(classifier_file, 'r')
#         targets = classdat['target2'].value
#         if (np.sum(targets) == 0) or (np.sum(targets) == len(targets)):
#             # only one target class
#             aucs[label].append(Auc95(np.nan, np.array([np.nan]),
#                                      np.nan, subj))
#             continue
#         auc_score_shuffles = np.array(
#             [roc_auc_score(targets, lp[:, 1])
#              for lp in classdat['log_probas_shuffles']])
#         auc_score = roc_auc_score(targets, classdat['log_probas'][:, 1])
#         aucs[label].append(Auc95(auc_score, auc_score_shuffles,
#                                  np.percentile(auc_score_shuffles, 95), subj))
# for label in ['FR1', 'catFR1']: 
#     classifier_path = plot_params[label]['classifier_path']
#     classifier_path_pickle = classifier_path + 'pickle_files/'
#     classifier_path_hdf5 = classifier_path + 'hdf5_files/'
#     c = 0.000719685673001
#     # classifier_c_paths = np.sort(glob(classifier_path_hdf5 + '*'))
#     classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
#                                     '/*_classifier_info.hdf5'))
#     aucs[label] = []
#     print label
#     for i, classifier_file in enumerate(classifier_files):
#         print i + 1, 'of', len(classifier_files)
#         subj = classifier_file.split('/')[-1].split('_classifier_info.hdf5')[0]
#         classdat = h5py.File(classifier_file, 'r')
#         if 'valid_evs_filter' in classdat:
#             targets = classdat['target'][classdat['valid_evs_filter'][:]]
#         else:
#             targets = classdat['target'].value
#         log_probas_shuffles = classdat['log_probas_shuffles'][0, ..., 1].T
#         auc_score_shuffles = np.array(
#             [roc_auc_score(targets, lp)
#              for lp in log_probas_shuffles])
#         log_probas = np.ravel(classdat['log_probas'][0, ..., 1])
#         auc_score = roc_auc_score(targets, log_probas)
#         # log_probas_shuffles = np.transpose(classdat['log_probas_shuffles'][
#         #     0, ..., 1], (1, 0, 2)).reshape(
#         #         classdat['log_probas_shuffles'].shape[2],
#         #         (classdat['log_probas_shuffles'].shape[1]*
#         #          classdat['log_probas_shuffles'].shape[3]))
#         # auc_score_shuffles = np.array(
#         #     [roc_auc_score(targets, lp)
#         #      for lp in log_probas_shuffles])
#         # log_probas = np.ravel(classdat['log_probas'][0, ..., 1])
#         # auc_score = roc_auc_score(targets, log_probas)
#         aucs[label].append(Auc95(auc_score, auc_score_shuffles,
#                                  np.percentile(auc_score_shuffles, 95), subj))


# auc_scores = {label: np.array([a.auc_score
#                                for a in aucs[label]]) for label in aucs
#               if len(aucs[label])>0}
# auc_scores95percentile = {label: np.array(
#     [a.percentile95 for a in aucs[label]]) for label in auc_scores}
# significance_filt = {label: auc_scores[label] > auc_scores95percentile[label]
#                      for label in auc_scores}








aucs = {}
# aucs_sorted = {}
# Auc95 = namedtuple(
#     'AUC', ['auc_score', 'percentile95', 'subj'])
Auc95 = namedtuple(
    'AUC', ['auc_score', 'auc_score_shuffles', 'percentile95', 'subj'])
# for label in plot_params:  # ['FR1toCatFR1', 'catFR1toFR1']:
for encret in ['encoding', 'retrieval']:
    aucs[encret] = {}
    for label in ['FR1toCatFR1', 'catFR1toFR1']:
        classifier_path = plot_params[encret][label]['classifier_path']
        classifier_path_pickle = classifier_path + 'pickle_files/'
        classifier_path_hdf5 = classifier_path + 'hdf5_files/'
        c = 0.000719685673001
        # classifier_c_paths = np.sort(glob(classifier_path_hdf5 + '*'))
        classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
                                        '/*_classifier_info.hdf5'))
        aucs[encret][label] = []
        print encret, label
        for i, classifier_file in enumerate(classifier_files):
            print i + 1, 'of', len(classifier_files)
            subj = classifier_file.split('/')[-1].split('_classifier_info.hdf5')[0]
            classdat = h5py.File(classifier_file, 'r')
            targets = classdat['target2'].value[classdat['valid_evs_filter2'].value]
            if (np.sum(targets) == 0) or (np.sum(targets) == len(targets)):
                # only one target class
                aucs[encret][label].append(Auc95(np.nan, np.array([np.nan]),
                                                 np.nan, subj))
                continue
            auc_score_shuffles = np.array(
                [roc_auc_score(targets, lp[:, 1])
                 for lp in classdat['log_probas_shuffles']])
            auc_score = roc_auc_score(targets, classdat['log_probas'][:, 1])
            aucs[encret][label].append(Auc95(auc_score, auc_score_shuffles,
                                             np.percentile(auc_score_shuffles, 95), subj))
    for label in ['FR1', 'catFR1']: 
        classifier_path = plot_params[encret][label]['classifier_path']
        classifier_path_pickle = classifier_path + 'pickle_files/'
        classifier_path_hdf5 = classifier_path + 'hdf5_files/'
        c = 0.000719685673001
        # classifier_c_paths = np.sort(glob(classifier_path_hdf5 + '*'))
        classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
                                        '/*_classifier_info.hdf5'))
        aucs[encret][label] = []
        print encret, label
        for i, classifier_file in enumerate(classifier_files):
            print i + 1, 'of', len(classifier_files)
            subj = classifier_file.split('/')[-1].split('_classifier_info.hdf5')[0]
            classdat = h5py.File(classifier_file, 'r')
            if 'valid_evs_filter' in classdat:
                targets = classdat['target'][classdat['valid_evs_filter'][:]]
            else:
                targets = classdat['target'].value
            if encret == 'encoding':
                log_probas_shuffles = np.transpose(classdat['log_probas_shuffles'][
                    0, ..., 1], (1, 0, 2)).reshape(
                        classdat['log_probas_shuffles'].shape[2],
                        (classdat['log_probas_shuffles'].shape[1]*
                         classdat['log_probas_shuffles'].shape[3]))
                auc_score_shuffles = np.array(
                    [roc_auc_score(targets, lp)
                     for lp in log_probas_shuffles])
            elif encret == 'retrieval':
                log_probas_shuffles = classdat['log_probas_shuffles'][0, ..., 1].T
                auc_score_shuffles = np.array(
                    [roc_auc_score(targets, lp)
                     for lp in log_probas_shuffles])
            else:
                raise ValueError('Invalid encret: '+str(encret))
            log_probas = np.ravel(classdat['log_probas'][0, ..., 1])
            auc_score = roc_auc_score(targets, log_probas)
            # log_probas_shuffles = np.transpose(classdat['log_probas_shuffles'][
            #     0, ..., 1], (1, 0, 2)).reshape(
            #         classdat['log_probas_shuffles'].shape[2],
            #         (classdat['log_probas_shuffles'].shape[1]*
            #          classdat['log_probas_shuffles'].shape[3]))
            # auc_score_shuffles = np.array(
            #     [roc_auc_score(targets, lp)
            #      for lp in log_probas_shuffles])
            # log_probas = np.ravel(classdat['log_probas'][0, ..., 1])
            # auc_score = roc_auc_score(targets, log_probas)
            aucs[encret][label].append(Auc95(
                auc_score, auc_score_shuffles,
                np.percentile(auc_score_shuffles, 95), subj))


auc_crit = 0.7
power = {}
for encret in aucs:
    power[encret] = {}
    for label in aucs[encret]:
        power[encret][label] = []
        for a in aucs[encret][label]:
            power[encret][label].append((a.subj, (np.float(len(a.auc_score_shuffles))-
                                                  np.sum(a.auc_score_shuffles>auc_crit))/
                                         len(a.auc_score_shuffles)))
        power[encret][label] = np.array(power[encret][label])

pow_crit = 0.8
valid_subj = {}
for encret in aucs:
    valid_subj[encret] = {}
    for label in aucs[encret]:
        valid_subj[encret][label] = []
        for a in aucs[encret][label]:
            powval = ((np.float(len(a.auc_score_shuffles))-
                    np.sum(a.auc_score_shuffles>auc_crit))/
                   len(a.auc_score_shuffles))
            if powval > pow_crit:
                valid_subj[encret][label].append(a.subj)
        # valid_subj[encret][label] = set(valid_subj[encret][label])

sub_pow = set.intersection(*[set(valid_subj[encret][label])
                             for encret in valid_subj
                             for label in valid_subj[encret]])


# sub_pow = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
#            'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
#            'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
#            'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
#            'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
#            'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
#            'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
#            'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
#            'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
#            'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
#            'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
#            'R1342M', 'R1347D', 'R1351M'}


sub_pow_allchans = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
           'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
           'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
           'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
           'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
           'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
           'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
           'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
           'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
           'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
           'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
           'R1342M', 'R1347D', 'R1351M'}

sub_pow_goodchans = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1042M',
           'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J',
           'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P', 'R1102P',
           'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C', 'R1131M',
           'R1135E', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
           'R1170J', 'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M',
           'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M', 'R1217T',
           'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T', 'R1251M',
           'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M',
           'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D',
           'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M', 'R1334T',
           'R1337E', 'R1338T', 'R1342M', 'R1351M', 'R1364C', 'R1367D',
           'R1368T', 'R1377M', 'R1378T', 'R1379E'}


# sub_pow = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1042M',
#            'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J',
#            'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P', 'R1102P',
#            'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C', 'R1131M',
#            'R1135E', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
#            'R1170J', 'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M',
#            'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M', 'R1217T',
#            'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T', 'R1251M',
#            'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M',
#            'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D',
#            'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M', 'R1334T',
#            'R1337E', 'R1338T', 'R1342M', 'R1351M', 'R1364C', 'R1367D',
#            'R1368T', 'R1377M', 'R1378T', 'R1379E'}



sub_pow = sub_pow_allchans







# OLD!!!
# In [46]: sub_pow
# Out[46]: 
# {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1039M', 'R1042M', 'R1045E',
#  'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M', 'R1074M',
#  'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C',
#  'R1131M', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1171M',
#  'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M', 'R1204T',
#  'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T',
#  'R1251M', 'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P',
#  'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M', 'R1328E',
#  'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T', 'R1342M', 'R1347D', 'R1351M',
#  'R1364C', 'R1367D', 'R1368T', 'R1375C', 'R1377M', 'R1378T', 'R1379E', 'R1381T'}






# auc_scores = {encret: {label: np.array(
#     [a.auc_score for a in aucs[encret][label] if a.subj in sub_pow])
#                        for label in aucs[encret]
#                        if len(aucs[encret][label])>0} for encret in aucs}



aucsubj_scores = {encret: {label: np.array(
    [(a.auc_score, a.subj) for a in aucs[encret][label] if a.subj in sub_pow])
                       for label in aucs[encret]
                       if len(aucs[encret][label])>0} for encret in aucs}


# from scipy.stats import norm
# auc_scores_z = {
#     encret:
#     {label: np.array(
#         [np.nan if ((a.auc_score == 0) or not np.isfinite(a.auc_score))
#          else norm.ppf((len(a.auc_score_shuffles)*2.-1) /
#                        (len(a.auc_score_shuffles)*2))
#          if a.auc_score_shuffles.max() < a.auc_score
#          else norm.ppf(1. / (len(a.auc_score_shuffles)*2))
#          if a.auc_score_shuffles.min() > a.auc_score
#          else norm.ppf(np.where(a.auc_score <= np.sort(a.auc_score_shuffles))[0][0] /
#                        np.float(len(a.auc_score_shuffles)))
#          for a in aucs[encret][label] if a.subj in sub_pow]) for label in aucs[encret]}
#     for encret in aucs}



from scipy.stats import norm
aucsubj_scores_z = {
    encret:
    {label: np.array(
        [(np.nan, a.subj) if ((a.auc_score == 0) or not np.isfinite(a.auc_score))
         else (norm.ppf((len(a.auc_score_shuffles)*2.-1) /
                        (len(a.auc_score_shuffles)*2)), a.subj)
         if a.auc_score_shuffles.max() < a.auc_score
         else (norm.ppf(1. / (len(a.auc_score_shuffles)*2)), a.subj)
         if a.auc_score_shuffles.min() > a.auc_score
         else (norm.ppf(np.where(a.auc_score <= np.sort(a.auc_score_shuffles))[0][0] /
                        np.float(len(a.auc_score_shuffles))), a.subj)
         for a in aucs[encret][label] if a.subj in sub_pow]) for label in aucs[encret]}
    for encret in aucs}






# auc_comps = {encret:
#              {'CU': auc_scores[encret]['catFR1toFR1'],
#               'UC': auc_scores[encret]['FR1toCatFR1'],
#               'CC': auc_scores[encret]['catFR1'],
#               'UU': auc_scores[encret]['FR1']}
#              for encret in auc_scores}
# auc_comps_z = {encret:
#              {'CU': auc_scores_z[encret]['catFR1toFR1'],
#               'UC': auc_scores_z[encret]['FR1toCatFR1'],
#               'CC': auc_scores_z[encret]['catFR1'],
#               'UU': auc_scores_z[encret]['FR1']}
#              for encret in auc_scores_z}


# import h5py
# auc_comps_h5py = h5py.File('./auc_comps.hdf5', 'w-')
# for encret in auc_comps.keys():
#     for cond in auc_comps[encret].keys():
#         auc_comps_h5py.create_dataset('aucs/'+encret+'/'+cond, data=auc_comps[encret][cond])
# auc_comps_h5py.close()




import h5py
aucsubj_scores_h5py = h5py.File('./aucsubj_comps' + class_path_suffix +
                                '.hdf5', 'w-')
for raw_z in [(aucsubj_scores, 'raw'), (aucsubj_scores_z, 'z')]:
    for encret in raw_z[0].keys():
        for cond in raw_z[0][encret].keys():
            aucsubj_scores_h5py.create_dataset(
                'aucs_scores/'+raw_z[1]+'/'+encret+
                '/'+cond, data=np.float32([rz[0] for rz in raw_z[0][encret][cond]]))
            aucsubj_scores_h5py.create_dataset(
                'subjects/'+raw_z[1]+'/'+encret+
                '/'+cond, data=[rz[1] for rz in raw_z[0][encret][cond]])
aucsubj_scores_h5py.close()





# import cPickle as pickle
# pickle.dump(auc_comps, open('aucs_comps_SMEretrieval.pickle', 'w'), -1)


import seaborn as sns
sns.set_style('white')
sns.set_style("ticks")
#plt.figure()
fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharex=True, sharey=True)
conds = ['CC', 'UU', 'CU', 'UC']
encret = 'encoding'
for er, encret in enumerate(['encoding', 'retrieval']):
    # sns.stripplot(data=[auc_comps[encret][comp] for comp in conds],
    #               jitter=True, alpha=0.3, ax=axs[0], color='black')
    sns.boxplot(data=[auc_comps[encret][comp] for comp in conds], ax=axs[er])
    # sns.violinplot(data=[auc_comps[encret][comp] for comp in conds], ax=axs[er])
    # sns.despine(offset=0, trim=True)
    axs[er].set_ylim((0.1, 1.0))
    axs[er].set_xlim((-0.75, 3.75))
    # axs[er].yaxis.set_ticks_position('both')
    # axs[er].yaxis.set_tick_params(direction='inout')
    # axs[er].yaxis.set_tick_params(direction='in')
    axs[er].set_xticklabels(conds)
    axs[er].set_ylabel('AUC')

axs[0].text(1.5, 0.2, 'encoding', horizontalalignment='center',
            verticalalignment='center', size='large')
axs[1].text(1.5, 0.2, 'retrieval', horizontalalignment='center',
            verticalalignment='center', size='large')
#axs[1].set_yticklabels([])
fig.subplots_adjust(left=0.15, bottom=0.15,
                    right=0.999, top=0.97, wspace=0.5, hspace=0.3)
sns.despine(offset=0, trim=True)
axs[0].text(-0.35, 0.97, 'A', horizontalalignment='center', fontdict={'weight':'bold', 'size':'large'},
            verticalalignment='center', size='large', transform=axs[0].transAxes)
axs[1].text(-0.35, 0.97, 'B', horizontalalignment='center', fontdict={'weight':'bold', 'size':'large'},
            verticalalignment='center', size='large', transform=axs[1].transAxes)
# fig.savefig('./figs/auc_boxplots_acrossexp_encret.pdf')




fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharex=True, sharey=True)
conds = ['CC', 'UU', 'CU', 'UC']
encret = 'encoding'
for er, encret in enumerate(['encoding', 'retrieval']):
    # sns.stripplot(data=[auc_comps[encret][comp] for comp in conds],
    #               jitter=True, alpha=0.3, ax=axs[0], color='black')
    sns.boxplot(data=[auc_comps_z[encret][comp] for comp in conds], ax=axs[er])
    # sns.violinplot(data=[auc_comps[encret][comp] for comp in conds], ax=axs[er])
    # sns.despine(offset=0, trim=True)
    #axs[er].set_ylim((0.1, 1.0))
    axs[er].set_xlim((-0.75, 3.75))
    # axs[er].yaxis.set_ticks_position('both')
    # axs[er].yaxis.set_tick_params(direction='inout')
    # axs[er].yaxis.set_tick_params(direction='in')
    axs[er].set_xticklabels(conds)
    axs[er].set_ylabel(r'$z($AUC$)$')

axs[0].text(1.5, -2, 'encoding', horizontalalignment='center',
            verticalalignment='center', size='large')
axs[1].text(1.5, -2, 'retrieval', horizontalalignment='center',
            verticalalignment='center', size='large')
#axs[1].set_yticklabels([])
fig.subplots_adjust(left=0.15, bottom=0.15,
                    right=0.999, top=0.97, wspace=0.5, hspace=0.3)
sns.despine(offset=0, trim=True)
axs[0].text(-0.35, 0.97, 'A', horizontalalignment='center', fontdict={'weight':'bold', 'size':'large'},
            verticalalignment='center', size='large', transform=axs[0].transAxes)
axs[1].text(-0.35, 0.97, 'B', horizontalalignment='center', fontdict={'weight':'bold', 'size':'large'},
            verticalalignment='center', size='large', transform=axs[1].transAxes)
# fig.savefig('./figs/auc_boxplots_acrossexp_encret.pdf')



fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharex=True, sharey=True)
conds = ['CC', 'UU', 'CU', 'UC']
encret = 'encoding'
for er, encret in enumerate(['encoding', 'retrieval']):
    # sns.stripplot(data=[auc_comps[encret][comp] for comp in conds],
    #               jitter=True, alpha=0.3, ax=axs[0], color='black')
    sns.swarmplot(data=[auc_comps[encret][comp] for comp in conds], ax=axs[er],
                  alpha=0.3, color='gray', zorder=1)
    sns.pointplot(data=[auc_comps[encret][comp] for comp in conds], ax=axs[er],
                  color='black', join=False, scale=0.5, zorder=3)
    # sns.violinplot(data=[auc_comps[encret][comp] for comp in conds], ax=axs[er])
    # sns.despine(offset=0, trim=True)
    axs[er].set_ylim((0.1, 1.0))
    axs[er].set_xlim((-0.75, 3.75))
    # axs[er].yaxis.set_ticks_position('both')
    # axs[er].yaxis.set_tick_params(direction='inout')
    # axs[er].yaxis.set_tick_params(direction='in')
    axs[er].set_xticklabels(conds)
    axs[er].set_ylabel('AUC')

axs[0].text(1.5, 0.2, 'encoding', horizontalalignment='center',
            verticalalignment='center', size='large')
axs[1].text(1.5, 0.2, 'retrieval', horizontalalignment='center',
            verticalalignment='center', size='large')
#axs[1].set_yticklabels([])
fig.subplots_adjust(left=0.15, bottom=0.15,
                    right=0.999, top=0.97, wspace=0.5, hspace=0.5)
sns.despine(offset=0, trim=True)
# axs[0].text(-0.35, 0.97, 'A', horizontalalignment='center', fontdict={'weight':'bold', 'size':'large'},
#             verticalalignment='center', size='large', transform=axs[0].transAxes)
# axs[1].text(-0.35, 0.97, 'B', horizontalalignment='center', fontdict={'weight':'bold', 'size':'large'},
#             verticalalignment='center', size='large', transform=axs[1].transAxes)
fig.savefig('./figs/auc_swarmplots_acrossexp_encret.pdf')




from scipy.stats import ttest_1samp
test_outputs = {}
for encret in ['encoding', 'retrieval']:
    test_outputs[encret] = {}
    for comp in conds:
        test_outputs[encret][comp] = dict(
            ttest_1samp(auc_comps_z[encret][comp], 0.0)._asdict())
        test_outputs[encret][comp]['SE'] = (
            np.std(
                auc_comps_z[encret][comp], ddof=1) /
            np.sqrt(len(auc_comps_z[encret][comp])))
        test_outputs[encret][comp]['df'] = (
            len(auc_comps_z[encret][comp])-1)


from scipy.stats import ttest_rel
test_outputs_comps = {}
for encret in ['encoding', 'retrieval']:
    test_outputs_comps[encret] = {}
    for c, comp1 in enumerate(conds[:-1]):
        for comp2 in conds[c+1:]:
            test_outputs_comps[encret][comp1+'-'+comp2] = dict(
                ttest_rel(auc_comps[encret][comp1],
                          auc_comps[encret][comp2])._asdict())
            test_outputs_comps[encret][comp1+'-'+comp2]['SE'] = np.std(
                auc_comps[encret][comp1]-auc_comps[encret][comp2],
                ddof=1)/np.sqrt(len(auc_comps[encret][comp1]))
            test_outputs_comps[encret][comp1+'-'+comp2]['df'] = len(
                auc_comps[encret][comp1])-1
encret1 = 'retrieval'
encret2 = 'encoding'
encret = encret1+'-'+encret2
test_outputs_comps[encret] = {}
for comp in conds:
    test_outputs_comps[encret][comp] = dict(
        ttest_rel(auc_comps[encret1][comp],
                  auc_comps[encret2][comp])._asdict())
    test_outputs_comps[encret][comp]['SE'] = np.std(
                auc_comps[encret1][comp]-auc_comps[encret2][comp],
                ddof=1)/np.sqrt(len(auc_comps[encret1][comp]))
    test_outputs_comps[encret][comp]['df'] = len(auc_comps[encret1][comp])-1

# In [29]: test_outputs
# Out[29]: 
# {'encoding': {'CC': {'SE': 0.15534372544633776,
#    'df': 50,
#    'pvalue': 1.8170599337798349e-19,
#    'statistic': 14.438172480341182},
#   'CU': {'SE': 0.14365328795205187,
#    'df': 50,
#    'pvalue': 3.9830074864710651e-15,
#    'statistic': 11.121855357595766},
#   'UC': {'SE': 0.13055492566361337,
#    'df': 50,
#    'pvalue': 3.4848059563556939e-16,
#    'statistic': 11.888704103021368},
#   'UU': {'SE': 0.19514996693370709,
#    'df': 50,
#    'pvalue': 6.0825285914478123e-17,
#    'statistic': 12.453411140924709}},
#  'retrieval': {'CC': {'SE': 0.18886628941613171,
#    'df': 50,
#    'pvalue': 2.2930889429995385e-18,
#    'statistic': 13.551760297593244},
#   'CU': {'SE': 0.13135852060286199,
#    'df': 50,
#    'pvalue': 1.4903079702688894e-22,
#    'statistic': 17.118810015203323},
#   'UC': {'SE': 0.13601986358473553,
#    'df': 50,
#    'pvalue': 8.0839271987852327e-23,
#    'statistic': 17.364426849116189},
#   'UU': {'SE': 0.22308548935900863,
#    'df': 50,
#    'pvalue': 2.0017756491486287e-12,
#    'statistic': 9.2598823243016639}}}

# In [30]: test_outputs_comps
# Out[30]: 
# {'encoding': {'CC-CU': {'SE': 0.0092407592193890012,
#    'df': 50,
#    'pvalue': 0.25622159070010719,
#    'statistic': 1.1485137116223221},
#   'CC-UC': {'SE': 0.0065760521898196814,
#    'df': 50,
#    'pvalue': 0.1263988942132063,
#    'statistic': 1.5543902509379179},
#   'CC-UU': {'SE': 0.010977130716225868,
#    'df': 50,
#    'pvalue': 0.6905565158162339,
#    'statistic': -0.40041519694434896},
#   'CU-UC': {'SE': 0.0069105723255071059,
#    'df': 50,
#    'pvalue': 0.95506087279462948,
#    'statistic': -0.056636011797369622},
#   'UU-CU': {'SE': 0.0093527137763574353,
#    'df': 50,
#    'pvalue': 0.11485254308363274,
#    'statistic': 1.6047266051089848},
#   'UU-UC': {'SE': 0.010049098463642766,
#    'df': 50,
#    'pvalue': 0.15203644577692513,
#    'statistic': 1.4545744002828118}},
#  'retrieval': {'CC-CU': {'SE': 0.018501678546099382,
#    'df': 50,
#    'pvalue': 0.72896748102866482,
#    'statistic': 0.34844695870525855},
#   'CC-UC': {'SE': 0.017050958927153109,
#    'df': 50,
#    'pvalue': 0.17515316713332585,
#    'statistic': -1.3753527786118735},
#   'CC-UU': {'SE': 0.022433243623063988,
#    'df': 50,
#    'pvalue': 0.044518896459683016,
#    'statistic': 2.0610423898874712},
#   'CU-UC': {'SE': 0.0092746653156939082,
#    'df': 50,
#    'pvalue': 0.0022311104838792306,
#    'statistic': -3.2236136120401593},
#   'UU-CU': {'SE': 0.013700771327085946,
#    'df': 50,
#    'pvalue': 0.005469542253659543,
#    'statistic': -2.9041439696767681},
#   'UU-UC': {'SE': 0.017560144509776571,
#    'df': 50,
#    'pvalue': 0.00023132765365631408,
#    'statistic': -3.9684724547378636}},
#  'retrieval-encoding': {'CC': {'SE': 0.018766077117937228,
#    'df': 50,
#    'pvalue': 1.9472630006153271e-06,
#    'statistic': 5.3842768424206948},
#   'CU': {'SE': 0.012671050793780739,
#    'df': 50,
#    'pvalue': 5.6738425255322206e-11,
#    'statistic': 8.3030240514675366},
#   'UC': {'SE': 0.015258269423020072,
#    'df': 50,
#    'pvalue': 8.9308188304573211e-12,
#    'statistic': 8.8289560150195499},
#   'UU': {'SE': 0.014379295779414844,
#    'df': 50,
#    'pvalue': 0.00097108670216041544,
#    'statistic': 3.5057682389375411}}}









# # http://stackoverflow.com/questions/12301071/multidimensional-confidence-intervals
# def plot_point_cov(points, nstd=2, ax=None, **kwargs):
#     """
#     Plots an `nstd` sigma ellipse based on the mean and covariance of a point
#     "cloud" (points, an Nx2 array).

#     Parameters
#     ----------
#         points : An Nx2 array of the data points.
#         nstd : The radius of the ellipse in numbers of standard deviations.
#             Defaults to 2 standard deviations.
#         ax : The axis that the ellipse will be plotted on. Defaults to the 
#             current axis.
#         Additional keyword arguments are pass on to the ellipse patch.

#     Returns
#     -------
#         A matplotlib ellipse artist
#     """
#     pos = points.mean(axis=0)
#     cov = np.cov(points, rowvar=False)
#     return plot_cov_ellipse(cov, pos, nstd, ax, **kwargs)

# def plot_cov_ellipse(cov, pos, nstd=2, ax=None, **kwargs):
#     """
#     Plots an `nstd` sigma error ellipse based on the specified covariance
#     matrix (`cov`). Additional keyword arguments are passed on to the 
#     ellipse patch artist.

#     Parameters
#     ----------
#         cov : The 2x2 covariance matrix to base the ellipse on
#         pos : The location of the center of the ellipse. Expects a 2-element
#             sequence of [x0, y0].
#         nstd : The radius of the ellipse in numbers of standard deviations.
#             Defaults to 2 standard deviations.
#         ax : The axis that the ellipse will be plotted on. Defaults to the 
#             current axis.
#         Additional keyword arguments are pass on to the ellipse patch.

#     Returns
#     -------
#         A matplotlib ellipse artist
#     """
#     def eigsorted(cov):
#         vals, vecs = np.linalg.eigh(cov)
#         order = vals.argsort()[::-1]
#         return vals[order], vecs[:,order]

#     if ax is None:
#         ax = plt.gca()

#     vals, vecs = eigsorted(cov)
#     theta = np.degrees(np.arctan2(*vecs[:,0][::-1]))

#     # Width and height are "full" widths, not radius
#     width, height = 2 * nstd * np.sqrt(vals)
#     ellip = Ellipse(xy=pos, width=width, height=height, angle=theta, **kwargs)

#     ax.add_artist(ellip)
#     return ellip

# # http://matplotlib.org/users/legend_guide.html
# class HandlerEllipse(HandlerPatch):
#     def create_artists(self, legend, orig_handle,
#                        xdescent, ydescent, width, height, fontsize, trans):
#         center = 0.5 * width - 0.5 * xdescent, 0.5 * height - 0.5 * ydescent
#         p = Ellipse(xy=center, width=width + xdescent,
#                              height=height + ydescent)
#         self.update_prop(p, orig_handle, legend)
#         p.set_transform(trans)
#         return [p]

# auc_comps = {encret:
#              {'cross_task': np.vstack(
#                  [auc_scores[encret]['catFR1toFR1'][
#                      subj_filts[encret]['catFR1toFR1']],
#                   auc_scores[encret]['FR1toCatFR1'][
#                       subj_filts[encret]['FR1toCatFR1']]]),
#               'from_catFR1': np.vstack(
#                   [auc_scores[encret]['catFR1'][subj_filts[encret]['catFR1']],
#                    auc_scores[encret]['catFR1toFR1'][subj_filts[encret][
#                        'catFR1toFR1']]]),
#               'from_FR1': np.vstack(
#                   [auc_scores[encret]['FR1'][subj_filts[encret]['FR1']],
#                    auc_scores[encret]['FR1toCatFR1'][subj_filts[encret][
#                        'FR1toCatFR1']]]),
#               'within_task': np.vstack(
#                   [auc_scores[encret]['catFR1'][subj_filts[encret]['catFR1']],
#                    auc_scores[encret]['FR1'][subj_filts[encret]['FR1']]])}
#              for encret in auc_scores}


# def make_ellipses_plot(axs, auc_comps, SE_instead_of_CI = False, legend='lower center'):
#     ellipse_kw = dict(fill=False, lw=2, alpha=0.8)
#     ellipses = []
#     if SE_instead_of_CI:
#         df = 1
#     else:
#         df = stats.t.ppf(0.975, len(auc_comps['cross_task'].T)-1)
#     ellipses.append(plot_point_cov(auc_comps['cross_task'].T, nstd=df, ax=axs,
#                                    color='gray', label='CU-UC', **ellipse_kw))
#     axs.plot(auc_comps['cross_task'].mean(1)[0],
#              auc_comps['cross_task'].mean(1)[1], color='gray', marker='o',
#              alpha=0.8, label='CU-UC', linestyle='None')
#     if SE_instead_of_CI:
#         df = 1
#     else:
#         df = stats.t.ppf(0.975, len(auc_comps['from_catFR1'].T)-1)
#     ellipses.append(plot_point_cov(auc_comps['from_catFR1'].T, nstd=df, ax=axs,
#                                    color='blue', label='CC-CU', **ellipse_kw))
#     axs.plot(auc_comps['from_catFR1'].mean(1)[0],
#              auc_comps['from_catFR1'].mean(1)[1], color='blue', marker='o',
#              alpha=0.8, label='CC-CU', linestyle='None')
#     if SE_instead_of_CI:
#         df = 1
#     else:
#         df = stats.t.ppf(0.975, len(auc_comps['from_FR1'].T)-1)
#     ellipses.append(plot_point_cov(auc_comps['from_FR1'].T, nstd=df, ax=axs,
#                                    color='red', label='UU-UC', **ellipse_kw))
#     axs.plot(auc_comps['from_FR1'].mean(1)[0],
#              auc_comps['from_FR1'].mean(1)[1], color='red', marker='o',
#              alpha=0.8, label='UU-UC', linestyle='None')
#     if SE_instead_of_CI:
#         df = 1
#     else:
#         df = stats.t.ppf(0.975, len(auc_comps['within_task'].T)-1)
#     ellipses.append(plot_point_cov(auc_comps['within_task'].T, nstd=df, ax=axs,
#                                    color='black', label='CC-UU', **ellipse_kw))
#     axs.plot(auc_comps['within_task'].mean(1)[0],
#              auc_comps['within_task'].mean(1)[1], color='black', marker='o',
#              alpha=0.8, label='CC-UU', linestyle='None')
#     # axs.set_xlabel(r"AUC 1", size='large')
#     # axs.set_ylabel(r"AUC 2", size='large')
#     axs.set_xlabel(r"AUC 1")
#     axs.set_ylabel(r"AUC 2")
#     axs.set_xlim((0.392, 1.0))
#     axs.set_ylim((0.392, 1.0))
#     # axs.set_xlim((0.4, 1.0))
#     # axs.set_ylim((0.4, 1.0))
#     axs.set_xticks(np.arange(0.4, 1.01, 0.1))
#     axs.set_xticklabels(np.arange(0.4, 1.01, 0.1), size='large')
#     axs.set_yticks(np.arange(0.4, 1.01, 0.1))
#     axs.set_yticklabels(np.arange(0.4, 1.01, 0.1), size='large')
#     axs.set_aspect('equal')
#     if legend:
#         legend1 = axs.legend(ellipses, [e.get_label() for e in ellipses], loc=legend,
#                              frameon=False, borderpad=0.0, ncol=1, labelspacing=0.25,
#                              handler_map={Ellipse: HandlerEllipse()})
#         axs.add_artist(legend1) # to keep it on after adding marker legend
#         axs.legend(loc=legend,
#                    frameon=False, borderpad=0.0, ncol=1, labelspacing=0.25, numpoints=1)
# fig, axs = plt.subplots(1, 2, figsize=(5.2, 2.4))
# make_ellipses_plot(axs[0], auc_comps['encoding'], SE_instead_of_CI = False,
#                    legend='upper right')
# make_ellipses_plot(axs[1], auc_comps['retrieval'], SE_instead_of_CI = False,
#                    legend=False)
# axs[0].spines['right'].set_color('none')
# axs[0].spines['top'].set_color('none')
# axs[0].spines['bottom'].set_position(('axes', -0.05))
# axs[0].spines['left'].set_position(('axes', -0.05))

# axs[1].spines['right'].set_color('none')
# axs[1].spines['top'].set_color('none')
# # axs[1].spines['left'].set_color('none')
# axs[1].spines['left'].set_position(('axes', -0.05))
# axs[1].spines['bottom'].set_position(('axes', -0.05))
# fig.subplots_adjust(left=0.11, bottom=0.21,
#                     right=0.99, top=0.97, wspace=0.3, hspace=0.3)
# axs[0].text(-0.3, 0.98, 'A', fontdict={'weight':'bold', 'size':'large'},
#             horizontalalignment='center', verticalalignment='center', transform=axs[0].transAxes)
# axs[1].text(-0.3, 0.98, 'B', fontdict={'weight':'bold', 'size':'large'},
#             horizontalalignment='center', verticalalignment='center', transform=axs[1].transAxes)
# fig.savefig('./figs/auc_errorellipses_withmean_encret.pdf')

