import numpy as np
import os
import h5py
from glob import glob
from sklearn.metrics import roc_auc_score
# import matplotlib
# matplotlib.use('cairo')
import matplotlib.pyplot as plt
from collections import namedtuple
from scipy import stats
# import itertools
import matplotlib as mpl
from matplotlib.patches import Ellipse
from matplotlib.legend_handler import HandlerPatch

mpl.rcParams['pdf.fonttype'] = 42
mpl.rcParams['ps.fonttype'] = 42

GOOD_ELECS_ONLY = False
# GOOD_ELECS_ONLY = True

if GOOD_ELECS_ONLY:
    class_path_suffix = '_goodchans'
else:
    class_path_suffix = ''
    

# pylint: disable=C0103

# plot_name = 'FR1toCatFR1'

run_on_rhino = False
# run_on_rhino = True
if run_on_rhino:
    rhino_mount = ''
else:
    rhino_mount = '/home/ctw/fusemounts/rhino'


# plot_params = {
#     'encoding': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/encoding/'+
#             'RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_encoding/'+
#             'RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_encoding_withshuffles/'),
#         'target': 'target',
#         'events': 'events'  # ,
#         # 'participants': 'participants',
#         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
#         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
#     },
#     'retrieval': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/'+
#             'RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_retrieval/'+
#             'RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_retrieval_withshuffles/'),
#         'target': 'target',
#         'events': 'events'  # ,
#         # 'participants': 'participants',
#         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
#         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
#     }
# }





plot_params = {
    'encoding':{
        'lag2cat': {
            'classifier_path': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_encoding/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_encoding_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'valevs': 'valid_evs_filter2'
        },
        'cat2lag': {
            'classifier_path': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_encoding/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_encoding_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'valevs': 'valid_evs_filter2'
        }
    },
    'retrieval':{
        'lag2cat': {
            'classifier_path': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_retrieval/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_retrieval_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'valevs': 'valid_evs_filter2'
        },
        'cat2lag': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' +
                class_path_suffix +
                '/RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_retrieval/'+
                'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_retrieval_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'valevs': 'valid_evs_filter2'
        }
    }
}



aucs = {}
Auc95 = namedtuple(
    'AUC', ['auc_score', 'auc_score_shuffles', 'percentile95', 'subj'])
for encret in ['encoding', 'retrieval']:
    aucs[encret] = {}
    for label in plot_params[encret]:
        classifier_path = plot_params[encret][label]['classifier_path']
        classifier_path_pickle = classifier_path + 'pickle_files/'
        classifier_path_hdf5 = classifier_path + 'hdf5_files/'
        c = 0.000719685673001
        # classifier_c_paths = np.sort(glob(classifier_path_hdf5 + '*'))
        classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
                                        '/*_classifier_info.hdf5'))
        aucs[encret][label] = []
        print encret, label
        for i, classifier_file in enumerate(classifier_files):
            print i + 1, 'of', len(classifier_files)
            subj = classifier_file.split('/')[-1].split(
                '_classifier_info.hdf5')[0]
            classdat = h5py.File(classifier_file, 'r')
            # filt = np.ones(classdat[
            #     plot_params[encret][label]['target']].shape, np.bool)
            # if label == 'withinvsbetweenCatFR1':
            #     filt = ((classdat['events']['within_cat'][:]==1) |
            #             (classdat['events']['between_cat'][:]==1))
            # if label == 'withinvsbetweenCatFR1':
            targets = classdat[plot_params[encret][label]['target']][
                classdat[plot_params[encret][label]['valevs']][:]]
            # else:
            #     targets = classdat[plot_params[encret][label]['target']][:]           
            if (np.sum(targets) == 0) or (np.sum(targets) == len(targets)):
                # only one target class
                aucs[encret][label].append(Auc95(np.nan, np.array([np.nan]),
                                                 np.nan, subj))
                continue
            # if label == 'withinvsbetweenCatFR1':
            log_probas_shuffles = classdat['log_probas_shuffles'][0, 0, ..., 1] #.T
            auc_score_shuffles = np.array(
                [roc_auc_score(targets, lp)
                 for lp in log_probas_shuffles])
            log_probas = np.ravel(classdat['log_probas'][0, ..., 1])
            auc_score = roc_auc_score(targets, log_probas)
            aucs[encret][label].append(Auc95(
                auc_score, auc_score_shuffles,
                np.percentile(auc_score_shuffles, 95), subj))


# # OLD:
# sub_pow_old = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1039M',
#            'R1042M', 'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T',
#            'R1065J', 'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P',
#            'R1102P', 'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C',
#            'R1131M', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
#            'R1170J', 'R1171M', 'R1174T', 'R1176M', 'R1178P', 'R1184M',
#            'R1189M', 'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M',
#            'R1217T', 'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T',
#            'R1251M', 'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J',
#            'R1291M', 'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T',
#            'R1317D', 'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M',
#            'R1334T', 'R1337E', 'R1338T', 'R1342M', 'R1347D', 'R1351M',
#            'R1364C', 'R1367D', 'R1368T', 'R1375C', 'R1377M', 'R1378T',
#            'R1379E', 'R1381T'}

# sub_pow = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
#            'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
#            'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
#            'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
#            'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
#            'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
#            'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
#            'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
#            'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
#            'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
#            'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
#            'R1342M', 'R1347D', 'R1351M'}


sub_pow_allchans = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
           'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
           'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
           'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
           'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
           'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
           'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
           'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
           'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
           'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
           'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
           'R1342M', 'R1347D', 'R1351M'}

sub_pow_goodchans = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1042M',
           'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J',
           'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P', 'R1102P',
           'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C', 'R1131M',
           'R1135E', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
           'R1170J', 'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M',
           'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M', 'R1217T',
           'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T', 'R1251M',
           'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M',
           'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D',
           'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M', 'R1334T',
           'R1337E', 'R1338T', 'R1342M', 'R1351M', 'R1364C', 'R1367D',
           'R1368T', 'R1377M', 'R1378T', 'R1379E'}


# sub_pow = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1042M',
#            'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J',
#            'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P', 'R1102P',
#            'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C', 'R1131M',
#            'R1135E', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
#            'R1170J', 'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M',
#            'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M', 'R1217T',
#            'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T', 'R1251M',
#            'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M',
#            'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D',
#            'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M', 'R1334T',
#            'R1337E', 'R1338T', 'R1342M', 'R1351M', 'R1364C', 'R1367D',
#            'R1368T', 'R1377M', 'R1378T', 'R1379E'}

if GOOD_ELECS_ONLY:
    sub_pow = sub_pow_goodchans
else:
    sub_pow = sub_pow_allchans





aucsubj_scores = {encret: {label: np.array(
    [(a.auc_score, a.subj) for a in aucs[encret][label] if a.subj in sub_pow])
                       for label in aucs[encret]
                       if len(aucs[encret][label])>0} for encret in aucs}

from scipy.stats import norm
aucsubj_scores_z = {
    encret:
    {label: np.array(
        [(np.nan, a.subj) if ((a.auc_score == 0) or not np.isfinite(a.auc_score))
         else (norm.ppf((len(a.auc_score_shuffles)*2.-1) /
                        (len(a.auc_score_shuffles)*2)), a.subj)
         if a.auc_score_shuffles.max() < a.auc_score
         else (norm.ppf(1. / (len(a.auc_score_shuffles)*2)), a.subj)
         if a.auc_score_shuffles.min() > a.auc_score
         else (norm.ppf(np.where(a.auc_score <= np.sort(a.auc_score_shuffles))[0][0] /
                        np.float(len(a.auc_score_shuffles))), a.subj)
         for a in aucs[encret][label] if a.subj in sub_pow]) for label in aucs[encret]}
    for encret in aucs}



import h5py
aucsubj_scores_h5py = h5py.File('./aucsubj_LagCat_scores' + class_path_suffix + 
                                '.hdf5', 'w-')
for raw_z in [(aucsubj_scores, 'raw'), (aucsubj_scores_z, 'z')]:
    for encret in raw_z[0].keys():
        for cond in raw_z[0][encret].keys():
            aucsubj_scores_h5py.create_dataset(
                'aucs_scores/'+raw_z[1]+'/'+encret+
                '/'+cond, data=np.float32([rz[0] for rz in raw_z[0][encret][cond]]))
            aucsubj_scores_h5py.create_dataset(
                'subjects/'+raw_z[1]+'/'+encret+
                '/'+cond, data=[rz[1] for rz in raw_z[0][encret][cond]])
aucsubj_scores_h5py.close()
