import numpy as np
import os
import h5py
from glob import glob
from sklearn.metrics import roc_auc_score
# import matplotlib
# matplotlib.use('cairo')
import matplotlib.pyplot as plt
from collections import namedtuple
from scipy import stats
# import itertools
import matplotlib as mpl
from matplotlib.patches import Ellipse
from matplotlib.legend_handler import HandlerPatch

mpl.rcParams['pdf.fonttype'] = 42
mpl.rcParams['ps.fonttype'] = 42

# GOOD_ELECS_ONLY = False
GOOD_ELECS_ONLY = True

if GOOD_ELECS_ONLY:
    class_path_suffix = '_goodchans'
else:
    class_path_suffix = ''
    


# pylint: disable=C0103

# plot_name = 'FR1toCatFR1'

run_on_rhino = False
# run_on_rhino = True
if run_on_rhino:
    rhino_mount = ''
else:
    rhino_mount = '/home/ctw/fusemounts/rhino'




plot_params = {
    'encoding':{
        'withinvsbetweenFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_encoding/'+
                'RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_encoding_withshuffles/'),
            'target': 'target',
            'events': 'events'
        }},
    #     'FR1toWithinCatFR1': {
    #         'classifier_path': (
    #             rhino_mount +
    #             '/scratch/cweidema/RAM/RAM_FRcatFR/encoding/'+
    #             'RAM_FR1toWithinCatFR1_classifiers/'+
    #             'RAM_FR1toWithinCatFR1_classifiers_withshuffles/'),
    #         'target': 'target2',
    #         'events': 'events2'  # ,
    #         # 'participants': 'participants',
    #         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
    #         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
    #     },
    #     'FR1toBetweenCatFR1': {
    #         'classifier_path': (
    #             rhino_mount +
    #             '/scratch/cweidema/RAM/RAM_FRcatFR/encoding/'+
    #             'RAM_FR1toBetweenCatFR1_classifiers/'+
    #             'RAM_FR1toBetweenCatFR1_classifiers_withshuffles/'),
    #         'target': 'target2',
    #         'events': 'events2'  # ,
    #         # 'participants': 'participants',
    #         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
    #         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
    #     }
    # },
    'retrieval':{
        'withinvsbetweenFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_retrieval/'+
                'RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_retrieval_withshuffles/'),
            'target': 'target',
            'events': 'events'
        }}#,
        # 'FR1toWithinCatFR1': {
        #     'classifier_path': (
        #         rhino_mount +
        #         '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/'+
        #         'RAM_FR1toWithinCatFR1_classifiers/'+
        #         'RAM_FR1toWithinCatFR1_classifiers_withshuffles/'),
        #     'target': 'target2',
        #     'events': 'events2'  # ,
        #     # 'participants': 'participants',
        #     # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
        #     # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        # },
        # 'FR1toBetweenCatFR1': {
        #     'classifier_path': (
        #         rhino_mount +
        #         '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/'+
        #         'RAM_FR1toBetweenCatFR1_classifiers/'+
        #         'RAM_FR1toBetweenCatFR1_classifiers_withshuffles/'),
        #     'target': 'target2',
        #     'events': 'events2'  # ,
        #     # 'participants': 'participants',
        #     # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
        #     # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        # }
    #}
}



aucs = {}
Auc95 = namedtuple(
    'AUC', ['auc_score', 'auc_score_shuffles', 'percentile95', 'subj'])
for encret in ['encoding', 'retrieval']:
    aucs[encret] = {}
    for label in plot_params[encret]:
        classifier_path = plot_params[encret][label]['classifier_path']
        classifier_path_pickle = classifier_path + 'pickle_files/'
        classifier_path_hdf5 = classifier_path + 'hdf5_files/'
        c = 0.000719685673001
        # classifier_c_paths = np.sort(glob(classifier_path_hdf5 + '*'))
        classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
                                        '/*_classifier_info.hdf5'))
        aucs[encret][label] = []
        print encret, label
        for i, classifier_file in enumerate(classifier_files):
            print i + 1, 'of', len(classifier_files)
            subj = classifier_file.split('/')[-1].split(
                '_classifier_info.hdf5')[0]
            classdat = h5py.File(classifier_file, 'r')
            # filt = np.ones(classdat[
            #     plot_params[encret][label]['target']].shape, np.bool)
            #if label == 'withinvsbetweenFR1':
            # filt = np.isfinite(classdat['target'])
            targets = classdat[plot_params[encret][label]['target']][
                classdat['valid_evs_filter'][:]]
            if (np.sum(targets) == 0) or (np.sum(targets) == len(targets)):
                # only one target class
                aucs[encret][label].append(Auc95(np.nan, np.array([np.nan]),
                                                 np.nan, subj))
                continue
            if label == 'withinvsbetweenFR1':
                log_probas_shuffles = classdat['log_probas_shuffles'][0, ..., 1].T
                auc_score_shuffles = np.array(
                    [roc_auc_score(targets, lp)
                     for lp in log_probas_shuffles])
                log_probas = np.ravel(classdat['log_probas'][0, ..., 1])
                auc_score = roc_auc_score(targets, log_probas)
            else:
                auc_score_shuffles = np.array(
                    [roc_auc_score(targets, lp[:, 1])
                     for lp in classdat['log_probas_shuffles']])
                auc_score = roc_auc_score(targets, classdat['log_probas'][:, 1])
            aucs[encret][label].append(Auc95(
                auc_score, auc_score_shuffles,
                np.percentile(auc_score_shuffles, 95), subj))

# # OLD:
# sub_pow_old = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1039M',
#            'R1042M', 'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T',
#            'R1065J', 'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P',
#            'R1102P', 'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C',
#            'R1131M', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
#            'R1170J', 'R1171M', 'R1174T', 'R1176M', 'R1178P', 'R1184M',
#            'R1189M', 'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M',
#            'R1217T', 'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T',
#            'R1251M', 'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J',
#            'R1291M', 'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T',
#            'R1317D', 'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M',
#            'R1334T', 'R1337E', 'R1338T', 'R1342M', 'R1347D', 'R1351M',
#            'R1364C', 'R1367D', 'R1368T', 'R1375C', 'R1377M', 'R1378T',
#            'R1379E', 'R1381T'}


# sub_pow = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
#            'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
#            'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
#            'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
#            'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
#            'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
#            'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
#            'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
#            'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
#            'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
#            'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
#            'R1342M', 'R1347D', 'R1351M'}


sub_pow_allchans = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
           'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
           'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
           'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
           'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
           'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
           'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
           'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
           'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
           'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
           'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
           'R1342M', 'R1347D', 'R1351M'}

sub_pow_goodchans = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1042M',
           'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J',
           'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P', 'R1102P',
           'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C', 'R1131M',
           'R1135E', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
           'R1170J', 'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M',
           'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M', 'R1217T',
           'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T', 'R1251M',
           'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M',
           'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D',
           'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M', 'R1334T',
           'R1337E', 'R1338T', 'R1342M', 'R1351M', 'R1364C', 'R1367D',
           'R1368T', 'R1377M', 'R1378T', 'R1379E'}


sub_pow = {'R1026D', 'R1031M', 'R1032D', 'R1035M', 'R1036M', 'R1042M',
           'R1045E', 'R1050M', 'R1056M', 'R1060M', 'R1061T', 'R1065J',
           'R1066P', 'R1069M', 'R1074M', 'R1075J', 'R1089P', 'R1102P',
           'R1105E', 'R1106M', 'R1111M', 'R1112M', 'R1114C', 'R1131M',
           'R1135E', 'R1147P', 'R1158T', 'R1163T', 'R1167M', 'R1169P',
           'R1170J', 'R1174T', 'R1176M', 'R1178P', 'R1184M', 'R1189M',
           'R1201P', 'R1202M', 'R1204T', 'R1207J', 'R1215M', 'R1217T',
           'R1222M', 'R1228M', 'R1230J', 'R1231M', 'R1243T', 'R1251M',
           'R1260D', 'R1264P', 'R1274T', 'R1275D', 'R1277J', 'R1291M',
           'R1293P', 'R1302M', 'R1310J', 'R1313J', 'R1315T', 'R1317D',
           'R1321M', 'R1324M', 'R1328E', 'R1330D', 'R1332M', 'R1334T',
           'R1337E', 'R1338T', 'R1342M', 'R1351M', 'R1364C', 'R1367D',
           'R1368T', 'R1377M', 'R1378T', 'R1379E'}




aucsubj_scores = {encret: {label: np.array(
    [(a.auc_score, a.subj) for a in aucs[encret][label] if a.subj in sub_pow])
                           for label in aucs[encret]
                           if len(aucs[encret][label])>0} for encret in aucs}

from scipy.stats import norm
aucsubj_scores_z = {
    encret:
    {label: np.array(
        [(np.nan, a.subj) if ((a.auc_score == 0) or not np.isfinite(a.auc_score))
         else (norm.ppf((len(a.auc_score_shuffles)*2.-1) /
                        (len(a.auc_score_shuffles)*2)), a.subj)
         if a.auc_score_shuffles.max() < a.auc_score
         else (norm.ppf(1. / (len(a.auc_score_shuffles)*2)), a.subj)
         if a.auc_score_shuffles.min() > a.auc_score
         else (norm.ppf(np.where(a.auc_score <= np.sort(a.auc_score_shuffles))[0][0] /
                        np.float(len(a.auc_score_shuffles))), a.subj)
         for a in aucs[encret][label] if a.subj in sub_pow]) for label in aucs[encret]}
    for encret in aucs}




import h5py
aucsubj_scores_h5py = h5py.File('./aucsubj_scores_temporal' + class_path_suffix +
                                '.hdf5', 'w-')
for raw_z in [(aucsubj_scores, 'raw'), (aucsubj_scores_z, 'z')]:
    for encret in raw_z[0].keys():
        for cond in raw_z[0][encret].keys():
            aucsubj_scores_h5py.create_dataset(
                'aucs_scores/'+raw_z[1]+'/'+encret+
                '/'+cond, data=np.float32([rz[0] for rz in  raw_z[0][encret][cond]]))
            aucsubj_scores_h5py.create_dataset(
                'subjects/'+raw_z[1]+'/'+encret+
                '/'+cond, data=[rz[1] for rz in  raw_z[0][encret][cond]])
aucsubj_scores_h5py.close()
































# # conds = ['catFR1', 'FRtoBetweenVsWithinCatFR1']
# from scipy.stats import ttest_1samp, ttest_rel
# test_outputs = {}
# for encret in ['encoding', 'retrieval']:
#     test_outputs[encret] = {}
#     test_outputs[encret]['catFR1'] = dict(
#         ttest_1samp(auc_scores_z[encret]['withinvsbetweenCatFR1'], 0.0)._asdict())
#     test_outputs[encret]['catFR1']['SE'] = (
#         np.std(
#             auc_scores_z[encret]['withinvsbetweenCatFR1'], ddof=1) /
#         np.sqrt(len(auc_scores_z[encret]['withinvsbetweenCatFR1'])))
#     test_outputs[encret]['catFR1']['df'] = (
#         len(auc_scores_z[encret]['withinvsbetweenCatFR1'])-1)
#     test_outputs[encret]['FRtoBetweenVsWithinCatFR1'] = dict(
#         ttest_rel(auc_scores_z[encret]['FR1toBetweenCatFR1'],
#                   auc_scores_z[encret]['FR1toWithinCatFR1'])._asdict())
#     test_outputs[encret]['FRtoBetweenVsWithinCatFR1']['SE'] = (
#         np.std(
#             auc_scores_z[encret]['FR1toBetweenCatFR1']-
#             auc_scores_z[encret]['FR1toWithinCatFR1'], ddof=1) /
#         np.sqrt(len(auc_scores_z[encret]['FR1toBetweenCatFR1'])))
#     test_outputs[encret]['FRtoBetweenVsWithinCatFR1']['df'] = (
#         len(auc_scores_z[encret]['FR1toBetweenCatFR1'])-1)

# test_outputs['retrieval-encoding'] = {}
# test_outputs['retrieval-encoding']['catFR1'] = dict(
#     ttest_rel(auc_scores_z['retrieval']['withinvsbetweenCatFR1'],
#               auc_scores_z['encoding']['withinvsbetweenCatFR1'])._asdict())
# test_outputs['retrieval-encoding']['catFR1']['SE'] = (
#     np.std(
#         auc_scores_z['retrieval']['withinvsbetweenCatFR1']-
#         auc_scores_z['encoding']['withinvsbetweenCatFR1'], ddof=1) /
#     np.sqrt(len(auc_scores_z['retrieval']['withinvsbetweenCatFR1'])))
# test_outputs['retrieval-encoding']['catFR1']['df'] = (
#     len(auc_scores_z['retrieval']['withinvsbetweenCatFR1'])-1)

# # test_outputs['retrieval-encoding']['FRtoBetweenVsWithinCatFR1'] = dict(
# #     ttest_rel(auc_scores_z['retrieval']['FR1toBetweenCatFR1'],
# #               auc_scores_z['encoding']['FR1toWithinCatFR1'])._asdict())
# test_outputs['retrieval-encoding']['FRtoBetweenVsWithinCatFR1'] = dict(
#     ttest_rel((auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
#                auc_scores_z['retrieval']['FR1toWithinCatFR1']),
#               (auc_scores_z['encoding']['FR1toBetweenCatFR1']-
#                auc_scores_z['encoding']['FR1toWithinCatFR1']))._asdict())
# test_outputs['retrieval-encoding']['FRtoBetweenVsWithinCatFR1']['SE'] = (
#     np.std(
#         (auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
#          auc_scores_z['retrieval']['FR1toWithinCatFR1'])-
#         (auc_scores_z['encoding']['FR1toBetweenCatFR1']-
#          auc_scores_z['encoding']['FR1toWithinCatFR1']), ddof=1) /
#     np.sqrt(len(auc_scores_z['retrieval']['FR1toBetweenCatFR1'])))
# test_outputs['retrieval-encoding']['FRtoBetweenVsWithinCatFR1']['df'] = (
#     len(auc_scores_z['retrieval']['FR1toBetweenCatFR1'])-1)



# import seaborn as sns
# sns.set_style('white')
# sns.set_style("ticks")
# #plt.figure()
# # fig, ax = plt.subplots(1, 1, figsize=((2,2)))
# # #ax.axis('off')
# # # ax.set_frame_on(False)
# # # ax.axes.xaxis.set_visible(False)
# # # ax.plot([-0.175,1.175], [0.5, 0.5], 'k') #color='gray', ls='-')
# # sns.stripplot(data=[auc_scores['encoding'], auc_scores['retrieval']], jitter=True, alpha=0.3, ax=ax, color='black')
# # # sns.stripplot(data=[auc_scores['encoding'], auc_scores['retrieval']], jitter=True, ax=ax, color='black')
# # sns.pointplot(data=[auc_scores['encoding'], auc_scores['retrieval']], ax=ax, color='black')
# # ax.set_xlim((-0.175,1.175))
# # sns.despine(offset=0, trim=True)
# # ax.set_xticklabels(['encoding', 'retrieval'])
# # ax.set_ylabel('AUC')
# # fig.subplots_adjust(left=0.3, bottom=0.16,
# #                     right=0.95, top=0.999, wspace=0.42, hspace=0.42)
# # fig.savefig('./figures/pub/withinvsbetween_encret.pdf')


# fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharey=True)#,
# #                       gridspec_kw = {'height_ratios': [5, 2]})
# #ax.axis('off')
# # ax.set_frame_on(False)
# # ax.axes.xaxis.set_visible(False)
# # ax.plot([-0.175,1.175], [0.5, 0.5], 'k') #color='gray', ls='-')
# sns.boxplot(data=[auc_scores['encoding']['withinvsbetweenCatFR1']-0.5,
#                   auc_scores['retrieval']['withinvsbetweenCatFR1']-0.5,
#                   (auc_scores['retrieval']['withinvsbetweenCatFR1']-
#                    auc_scores['encoding']['withinvsbetweenCatFR1'])], ax=axs[0])

# sns.boxplot(data=[(auc_scores['encoding']['FR1toBetweenCatFR1']-
#                    auc_scores['encoding']['FR1toWithinCatFR1']),
#                   (auc_scores['retrieval']['FR1toBetweenCatFR1']-
#                    auc_scores['retrieval']['FR1toWithinCatFR1']),
#                   ((auc_scores['retrieval']['FR1toBetweenCatFR1']-
#                     auc_scores['retrieval']['FR1toWithinCatFR1'])-
#                    (auc_scores['encoding']['FR1toBetweenCatFR1']-
#                     auc_scores['encoding']['FR1toWithinCatFR1']))], ax=axs[1])
# sns.despine(offset=0, trim=True, ax=axs[0])
# sns.despine(offset=0, trim=True, ax=axs[1])
# axs[0].set_ylabel(r'$\Delta$AUC')
# axs[1].set_ylabel(r'$\Delta$AUC')
# axs[0].set_xticklabels(['E','R', r'R$-$E'])
# axs[1].set_xticklabels(['E','R', r'R$-$E'])
# fig.subplots_adjust(left=0.17, bottom=0.16,
#                     right=0.9999, top=0.9999, wspace=0.6, hspace=0.6)
# # fig.savefig('./figs/withinvsbetween_boxplots_encret.pdf')





# # fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharey=True)#,
# # #                       gridspec_kw = {'height_ratios': [5, 2]})
# # #ax.axis('off')
# # # ax.set_frame_on(False)
# # # ax.axes.xaxis.set_visible(False)
# # # ax.plot([-0.175,1.175], [0.5, 0.5], 'k') #color='gray', ls='-')
# # sns.boxplot(data=[auc_scores_z['encoding']['withinvsbetweenCatFR1'],
# #                   auc_scores_z['retrieval']['withinvsbetweenCatFR1'],
# #                   (auc_scores_z['retrieval']['withinvsbetweenCatFR1']-
# #                    auc_scores_z['encoding']['withinvsbetweenCatFR1'])], ax=axs[0])

# # sns.boxplot(data=[(auc_scores_z['encoding']['FR1toBetweenCatFR1']-
# #                    auc_scores_z['encoding']['FR1toWithinCatFR1']),
# #                   (auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
# #                    auc_scores_z['retrieval']['FR1toWithinCatFR1']),
# #                   ((auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
# #                     auc_scores_z['retrieval']['FR1toWithinCatFR1'])-
# #                    (auc_scores_z['encoding']['FR1toBetweenCatFR1']-
# #                     auc_scores_z['encoding']['FR1toWithinCatFR1']))], ax=axs[1])
# # sns.despine(offset=0, trim=True, ax=axs[0])
# # sns.despine(offset=0, trim=True, ax=axs[1])
# # axs[0].set_ylabel(r'$z($AUC$)$')
# # axs[1].set_ylabel(r'$z($AUC$)$')
# # axs[0].set_xticklabels(['E','R', r'R$-$E'])
# # axs[1].set_xticklabels(['E','R', r'R$-$E'])
# # fig.subplots_adjust(left=0.17, bottom=0.16,
# #                     right=0.9999, top=0.9999, wspace=0.6, hspace=0.6)



# fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharey=True)#,
# #                       gridspec_kw = {'height_ratios': [5, 2]})
# sns.swarmplot(data=[auc_scores_z['encoding']['withinvsbetweenCatFR1'],
#                   auc_scores_z['retrieval']['withinvsbetweenCatFR1'],
#                   (auc_scores_z['retrieval']['withinvsbetweenCatFR1']-
#                    auc_scores_z['encoding']['withinvsbetweenCatFR1'])],
#               ax=axs[0], alpha=0.3, color='gray', zorder=1)
# sns.pointplot(data=[auc_scores_z['encoding']['withinvsbetweenCatFR1'],
#                   auc_scores_z['retrieval']['withinvsbetweenCatFR1'],
#                   (auc_scores_z['retrieval']['withinvsbetweenCatFR1']-
#                    auc_scores_z['encoding']['withinvsbetweenCatFR1'])],
#               join=False, ax=axs[0], scale=0.5, color='black', zorder=3)


# sns.swarmplot(data=[(auc_scores_z['encoding']['FR1toBetweenCatFR1']-
#                    auc_scores_z['encoding']['FR1toWithinCatFR1']),
#                   (auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
#                    auc_scores_z['retrieval']['FR1toWithinCatFR1']),
#                   ((auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
#                     auc_scores_z['retrieval']['FR1toWithinCatFR1'])-
#                    (auc_scores_z['encoding']['FR1toBetweenCatFR1']-
#                     auc_scores_z['encoding']['FR1toWithinCatFR1']))],
#               color='gray', alpha=0.3, ax=axs[1], zorder=1)
# sns.pointplot(data=[(auc_scores_z['encoding']['FR1toBetweenCatFR1']-
#                    auc_scores_z['encoding']['FR1toWithinCatFR1']),
#                   (auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
#                    auc_scores_z['retrieval']['FR1toWithinCatFR1']),
#                   ((auc_scores_z['retrieval']['FR1toBetweenCatFR1']-
#                     auc_scores_z['retrieval']['FR1toWithinCatFR1'])-
#                    (auc_scores_z['encoding']['FR1toBetweenCatFR1']-
#                     auc_scores_z['encoding']['FR1toWithinCatFR1']))],
#               join=False, ax=axs[1], scale=0.5, color='black', zorder=3)
# sns.despine(offset=0, trim=True, ax=axs[0])
# sns.despine(offset=0, trim=True, ax=axs[1])
# axs[0].set_ylabel(r'$z($AUC$)$')
# axs[1].set_ylabel(r'$z($AUC$)$')
# axs[0].set_xticklabels(['E','R', r'R$-$E'])
# axs[1].set_xticklabels(['E','R', r'R$-$E'])
# axs[0].plot([-1, 4], [0, 0], 'k-')
# axs[1].plot([-1, 4], [0, 0], 'k-')
# fig.subplots_adjust(left=0.14, bottom=0.16,
#                     right=0.9999, top=0.99999, wspace=0.5, hspace=0.5)
# fig.savefig('./figs/withinvsbetween_swarmplots_encret.pdf')






# # fig, axs = plt.subplots(1, 2, figsize=((4,2)), sharey=True)#,
# # #                       gridspec_kw = {'height_ratios': [5, 2]})
# # sns.swarmplot(data=[auc_scores['encoding']['withinvsbetweenCatFR1'],
# #                   auc_scores['retrieval']['withinvsbetweenCatFR1'],
# #                   (auc_scores['retrieval']['withinvsbetweenCatFR1']-
# #                    auc_scores['encoding']['withinvsbetweenCatFR1'])],
# #               ax=axs[0], alpha=0.3, color='gray', zorder=1)
# # sns.pointplot(data=[auc_scores['encoding']['withinvsbetweenCatFR1'],
# #                   auc_scores['retrieval']['withinvsbetweenCatFR1'],
# #                   (auc_scores['retrieval']['withinvsbetweenCatFR1']-
# #                    auc_scores['encoding']['withinvsbetweenCatFR1'])],
# #               join=False, ax=axs[0], scale=0.5, color='black', zorder=3)


# # sns.swarmplot(data=[(auc_scores['encoding']['FR1toBetweenCatFR1']-
# #                    auc_scores['encoding']['FR1toWithinCatFR1']),
# #                   (auc_scores['retrieval']['FR1toBetweenCatFR1']-
# #                    auc_scores['retrieval']['FR1toWithinCatFR1']),
# #                   ((auc_scores['retrieval']['FR1toBetweenCatFR1']-
# #                     auc_scores['retrieval']['FR1toWithinCatFR1'])-
# #                    (auc_scores['encoding']['FR1toBetweenCatFR1']-
# #                     auc_scores['encoding']['FR1toWithinCatFR1']))],
# #               color='gray', alpha=0.3, ax=axs[1], zorder=1)
# # sns.pointplot(data=[(auc_scores['encoding']['FR1toBetweenCatFR1']-
# #                    auc_scores['encoding']['FR1toWithinCatFR1']),
# #                   (auc_scores['retrieval']['FR1toBetweenCatFR1']-
# #                    auc_scores['retrieval']['FR1toWithinCatFR1']),
# #                   ((auc_scores['retrieval']['FR1toBetweenCatFR1']-
# #                     auc_scores['retrieval']['FR1toWithinCatFR1'])-
# #                    (auc_scores['encoding']['FR1toBetweenCatFR1']-
# #                     auc_scores['encoding']['FR1toWithinCatFR1']))],
# #               join=False, ax=axs[1], scale=0.5, color='black', zorder=3)
# # sns.despine(offset=0, trim=True, ax=axs[0])
# # sns.despine(offset=0, trim=True, ax=axs[1])
# # axs[0].set_ylabel(r'$z($AUC$)$')
# # axs[1].set_ylabel(r'$z($AUC$)$')
# # axs[0].set_xticklabels(['E','R', r'R$-$E'])
# # axs[1].set_xticklabels(['E','R', r'R$-$E'])
# # axs[0].plot([-1, 4], [0, 0], 'k-')
# # axs[1].plot([-1, 4], [0, 0], 'k-')
# # fig.subplots_adjust(left=0.14, bottom=0.16,
# #                     right=0.9999, top=0.99999, wspace=0.5, hspace=0.5)


# import h5py
# auc_scores_h5py = h5py.File('./auc_scores_temporal.hdf5', 'w-')
# for raw_z in [(auc_scores, 'raw'), (auc_scores_z, 'z')]:
#     for encret in raw_z[0].keys():
#         for cond in raw_z[0][encret].keys():
#             auc_scores_h5py.create_dataset('aucs_scores/'+raw_z[1]+'/'+encret+
#                                            '/'+cond, data=raw_z[0][encret][cond])
# auc_scores_h5py.close()






# # fig, axs = plt.subplots(1, 2, figsize=((3,1.5)), sharey=True)#,
# # #                       gridspec_kw = {'height_ratios': [5, 2]})
# # #ax.axis('off')
# # # ax.set_frame_on(False)
# # # ax.axes.xaxis.set_visible(False)
# # # ax.plot([-0.175,1.175], [0.5, 0.5], 'k') #color='gray', ls='-')
# # sns.boxplot(data=[auc_scores['encoding']['withinvsbetweenCatFR1']-0.5,
# #                   auc_scores['retrieval']['withinvsbetweenCatFR1']-0.5,
# #                   (auc_scores['retrieval']['withinvsbetweenCatFR1']-
# #                    auc_scores['encoding']['withinvsbetweenCatFR1'])], ax=axs[0])

# # sns.boxplot(data=[(auc_scores['encoding']['FR1toBetweenCatFR1']-
# #                    auc_scores['encoding']['FR1toWithinCatFR1']),
# #                   (auc_scores['retrieval']['FR1toBetweenCatFR1']-
# #                    auc_scores['retrieval']['FR1toWithinCatFR1']),
# #                   ((auc_scores['retrieval']['FR1toBetweenCatFR1']-
# #                     auc_scores['retrieval']['FR1toWithinCatFR1'])-
# #                    (auc_scores['encoding']['FR1toBetweenCatFR1']-
# #                     auc_scores['encoding']['FR1toWithinCatFR1']))], ax=axs[1])
# # sns.despine(offset=0, trim=True, ax=axs[0])
# # sns.despine(offset=0, trim=True, ax=axs[1])
# # axs[0].set_ylabel(r'$\Delta$AUC')
# # axs[1].set_ylabel(r'$\Delta$AUC')
# # axs[0].set_xticklabels(['E','R', r'R$-$E'])
# # axs[1].set_xticklabels(['E','R', r'R$-$E'])
# # fig.subplots_adjust(left=0.26, bottom=0.2,
# #                     right=0.9999, top=0.9999, wspace=1.2, hspace=1.2)
# # fig.savefig('./figs/withinvsbetween_boxplots_encret_small.pdf')






# # fig, axs = plt.subplots(1, 2, figsize=((3,2)))#,
# # #                       gridspec_kw = {'height_ratios': [5, 2]})
# # #ax.axis('off')
# # # ax.set_frame_on(False)
# # # ax.axes.xaxis.set_visible(False)
# # # ax.plot([-0.175,1.175], [0.5, 0.5], 'k') #color='gray', ls='-')
# # sns.boxplot(data=[auc_scores['encoding']-0.5, auc_scores['retrieval']-0.5,auc_scores['retrieval']-auc_scores['encoding']], ax=axs[0])

# # sns.boxplot(data=[auc_scores['encoding']-0.5, auc_scores['retrieval']-0.5,auc_scores['retrieval']-auc_scores['encoding']], ax=axs[0])



# # # sns.stripplot(data=[auc_scores['encoding'], auc_scores['retrieval']], jitter=True, ax=ax, color='black')
# # sns.pointplot(data=[auc_scores['encoding'], auc_scores['retrieval']], ax=ax, color='black')
# # ax.set_xlim((-0.175,1.175))
# # sns.despine(offset=0, trim=True)
# # ax.set_xticklabels(['encoding', 'retrieval'])
# # ax.set_ylabel('AUC')
# # fig.subplots_adjust(left=0.3, bottom=0.16,
# #                     right=0.95, top=0.999, wspace=0.42, hspace=0.42)
# # fig.savefig('./figures/pub/withinvsbetween_encret.pdf')









# # grid = sns.jointplot(x=auc_scores['encoding'], y=auc_scores['retrieval'],
# #                      stat_func=None, color='k', kind='reg', ci=95, size=3.5)
# # # grid = sns.jointplot(x=auc_scores['encoding'], y=auc_scores['retrieval'],
# # #                      color='k', kind='reg', ci=95, size=3.5)
# # grid.set_axis_labels('encoding', 'retrieval', labelpad=1, size='large')
# # #grid.fig.set_size_inches((4,4))
# # #grid.fig.set_size_inches((2,2))
# # grid.fig.axes[0].xaxis.set_tick_params(pad=3)
# # grid.fig.axes[0].yaxis.set_tick_params(pad=3)
# # grid.fig.axes[0].set_xticks(np.arange(0, 1.1, 0.1))
# # grid.fig.axes[0].set_xticklabels(np.arange(0, 1.1, 0.1), size='large')
# # grid.fig.axes[0].set_yticks(np.arange(0., 1.1, 0.1))
# # grid.fig.axes[0].set_yticklabels(np.arange(0., 1.1, 0.1), size='large')
# # grid.fig.axes[0].yaxis.set_tick_params(pad=3)
# # grid.fig.axes[0].set_xlim((0.15, 0.85))
# # grid.fig.axes[0].set_ylim((0.15, 0.85))
# # grid.fig.axes[0].plot([0, 1], [0, 1], 'r--')


# # #grid.fig.subplots_adjust(left=0.3, right=0.9, bottom=0.3, top=0.9,
# # #                         wspace=0, hspace=0)
# # #grid.savefig('./figures/pub/...')







# # tmp.xaxis.set_visible(False)
# # sns.despine(offset=-5, trim=True)

# # #fig, ax = plt.figure()
# # sns.set_style('white')
# # sns.set_style("ticks")
# # sns.pointplot(data=[auc_scores['encoding'], auc_scores['retrieval']], alpha=0.7, markers=['^', 'v'])
# # # sns.pointplot(data=auc_scores['retrieval'], alpha=0.7, markers=['v'])
# # #sns.stripplot(data=[auc_scores['encoding'], auc_scores['retrieval']], jitter=True, alpha=0.7)
# # sns.despine(offset=-5, trim=True)

# # # http://stackoverflow.com/questions/12301071/multidimensional-confidence-intervals
# # def plot_point_cov(points, nstd=2, ax=None, **kwargs):
# #     """
# #     Plots an `nstd` sigma ellipse based on the mean and covariance of a point
# #     "cloud" (points, an Nx2 array).

# #     Parameters
# #     ----------
# #         points : An Nx2 array of the data points.
# #         nstd : The radius of the ellipse in numbers of standard deviations.
# #             Defaults to 2 standard deviations.
# #         ax : The axis that the ellipse will be plotted on. Defaults to the 
# #             current axis.
# #         Additional keyword arguments are pass on to the ellipse patch.

# #     Returns
# #     -------
# #         A matplotlib ellipse artist
# #     """
# #     pos = points.mean(axis=0)
# #     cov = np.cov(points, rowvar=False)
# #     return plot_cov_ellipse(cov, pos, nstd, ax, **kwargs)

# # def plot_cov_ellipse(cov, pos, nstd=2, ax=None, **kwargs):
# #     """
# #     Plots an `nstd` sigma error ellipse based on the specified covariance
# #     matrix (`cov`). Additional keyword arguments are passed on to the 
# #     ellipse patch artist.

# #     Parameters
# #     ----------
# #         cov : The 2x2 covariance matrix to base the ellipse on
# #         pos : The location of the center of the ellipse. Expects a 2-element
# #             sequence of [x0, y0].
# #         nstd : The radius of the ellipse in numbers of standard deviations.
# #             Defaults to 2 standard deviations.
# #         ax : The axis that the ellipse will be plotted on. Defaults to the 
# #             current axis.
# #         Additional keyword arguments are pass on to the ellipse patch.

# #     Returns
# #     -------
# #         A matplotlib ellipse artist
# #     """
# #     def eigsorted(cov):
# #         vals, vecs = np.linalg.eigh(cov)
# #         order = vals.argsort()[::-1]
# #         return vals[order], vecs[:,order]

# #     if ax is None:
# #         ax = plt.gca()

# #     vals, vecs = eigsorted(cov)
# #     theta = np.degrees(np.arctan2(*vecs[:,0][::-1]))

# #     # Width and height are "full" widths, not radius
# #     width, height = 2 * nstd * np.sqrt(vals)
# #     ellip = Ellipse(xy=pos, width=width, height=height, angle=theta, **kwargs)

# #     ax.add_artist(ellip)
# #     return ellip

# # # http://matplotlib.org/users/legend_guide.html
# # class HandlerEllipse(HandlerPatch):
# #     def create_artists(self, legend, orig_handle,
# #                        xdescent, ydescent, width, height, fontsize, trans):
# #         center = 0.5 * width - 0.5 * xdescent, 0.5 * height - 0.5 * ydescent
# #         p = Ellipse(xy=center, width=width + xdescent,
# #                              height=height + ydescent)
# #         self.update_prop(p, orig_handle, legend)
# #         p.set_transform(trans)
# #         return [p]



# # auc_comps = {'cross_task': np.vstack([auc_scores['catFR1toFR1'],
# #                                       auc_scores['FR1toCatFR1']]),
# #              'from_catFR1': np.vstack([auc_scores['catFR1'],
# #                                        auc_scores['catFR1toFR1']]),
# #              'from_FR1': np.vstack([auc_scores['FR1'],
# #                                     auc_scores['FR1toCatFR1']]),
# #              'within_task': np.vstack([auc_scores['catFR1'],
# #                                        auc_scores['FR1']])}




# # SE_instead_of_CI = False
# # fig, axs = plt.subplots(1, 1, figsize=(3.2, 3.0))
# # ellipse_kw = dict(fill=False, lw=2, alpha=0.8)
# # ellipses = []
# # if SE_instead_of_CI:
# #     df = 1
# # else:
# #     df = stats.t.ppf(0.975, len(auc_comps['cross_task'].T)-1)
# # ellipses.append(plot_point_cov(auc_comps['cross_task'].T, nstd=df, ax=axs,
# #                color='gray', label='CU-UC', **ellipse_kw))
# # # axs.plot(auc_scores['FR1toCatFR1'],
# # #          auc_scores['catFR1toFR1'],
# # #          'ko', alpha=0.5, mew=2)
# # axs.plot(auc_comps['cross_task'].mean(1)[0],
# #          auc_comps['cross_task'].mean(1)[1], color='gray', marker='o',
# #          alpha=0.8, label='CU-UC', linestyle='None')



# # if SE_instead_of_CI:
# #     df = 1
# # else:
# #     df = stats.t.ppf(0.975, len(auc_comps['from_catFR1'].T)-1)
# # ellipses.append(plot_point_cov(auc_comps['from_catFR1'].T, nstd=df, ax=axs,
# #                color='blue', label='CC-CU', **ellipse_kw))
# # # axs.plot(auc_scores['catFR1'],
# # #          auc_scores['catFR1toFR1'],
# # #          'ko', alpha=0.5, mew=2)
# # axs.plot(auc_comps['from_catFR1'].mean(1)[0],
# #          auc_comps['from_catFR1'].mean(1)[1], color='blue', marker='o',
# #          alpha=0.8, label='CC-CU', linestyle='None')


# # if SE_instead_of_CI:
# #     df = 1
# # else:
# #     df = stats.t.ppf(0.975, len(auc_comps['from_FR1'].T)-1)
# # ellipses.append(plot_point_cov(auc_comps['from_FR1'].T, nstd=df, ax=axs,
# #                color='red', label='UU-UC', **ellipse_kw))
# # # axs.plot(auc_scores['FR1'],
# # #          auc_scores['FR1toCatFR1'],
# # #          'ko', alpha=0.5, mew=2)
# # axs.plot(auc_comps['from_FR1'].mean(1)[0],
# #          auc_comps['from_FR1'].mean(1)[1], color='red', marker='o',
# #          alpha=0.8, label='UU-UC', linestyle='None')


# # if SE_instead_of_CI:
# #     df = 1
# # else:
# #     df = stats.t.ppf(0.975, len(auc_comps['within_task'].T)-1)
# # ellipses.append(plot_point_cov(auc_comps['within_task'].T, nstd=df, ax=axs,
# #                color='black', label='CC-UU', **ellipse_kw))
# # # axs.plot(auc_scores['catFR1'],
# # #          auc_scores['FR1'],
# # #          'ko', alpha=0.5, mew=2)
# # axs.plot(auc_comps['within_task'].mean(1)[0],
# #          auc_comps['within_task'].mean(1)[1], color='black', marker='o',
# #          alpha=0.8, label='CC-UU', linestyle='None')


# # axs.set_xlabel(r"AUC 1", size='large')
# # axs.set_ylabel(r"AUC 2", size='large')
# # # axs.set_xlim((0.4, 1.0))
# # # axs.set_ylim((0.4, 1.0))
# # axs.set_xlim((0.35, 1.0))
# # axs.set_ylim((0.35, 1.0))
# # axs.set_xticks(np.arange(0.5, 0.91, 0.2))
# # axs.set_xticklabels(np.arange(0.5, 0.91, 0.2), size='large')
# # axs.set_yticks(np.arange(0.5, 0.91, 0.2))
# # axs.set_yticklabels(np.arange(0.5, 0.91, 0.2), size='large')
# # axs.set_aspect('equal')
# # fig.subplots_adjust(left=0.2, bottom=0.155,
# #                     right=0.99, top=0.999, wspace=0.42, hspace=0.42)

# # legend1 = axs.legend(ellipses, [e.get_label() for e in ellipses], loc='upper right',
# #                      frameon=False, borderpad=0.0, ncol=1, labelspacing=0.25,
# #                      handler_map={Ellipse: HandlerEllipse()})
# # axs.add_artist(legend1) # to keep it on after adding marker legend
# # axs.legend(loc='upper right',
# #            frameon=False, borderpad=0.0, ncol=1, labelspacing=0.25, numpoints=1)

# # if SE_instead_of_CI:
# #     fig.savefig('./figures/pub/auc_errorellipsesSE_withmean.pdf')
# # else:
# #     fig.savefig('./figures/pub/auc_errorellipses_withmean.pdf')

