# import sys
import numpy as np
# import os
import h5py
# import multiprocessing as mp
# from glob import glob
# from collections import Counter
# import cPickle as pickle
from sklearn.metrics import roc_curve
from matplotlib import pyplot as plt
from scipy import stats
from glob import glob
import seaborn as sns
sns.set_style('white')
sns.set_style("ticks")
import matplotlib.gridspec as gridspec
import matplotlib as mpl
import matplotlib.patches as mpatches
import scipy.stats.distributions as dists
mpl.rcParams['pdf.fonttype'] = 42
mpl.rcParams['ps.fonttype'] = 42


GOOD_ELECS_ONLY = False
# GOOD_ELECS_ONLY = True

if GOOD_ELECS_ONLY:
    class_path_suffix = '_goodchans'
else:
    class_path_suffix = ''
    


# pylint: disable=C0103

# plot_name = 'FR1toCatFR1'

run_on_rhino = False
# run_on_rhino = True
if run_on_rhino:
    rhino_mount = ''
else:
    rhino_mount = '/home/ctw/fusemounts/rhino'


# plot_params = {
#     'FR1toCatFR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_FR1toCatFR1_classifiers/' +
#             'RAM_FR1toCatFR1_classifiers_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'  # ,
#         # 'participants': 'participants',
#         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
#         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
#     },
#     'FR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_FR1lolo_classifiers_allfreqs/' +
#             'RAM_FR1lolo_classifiers_allfreqs_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'
#     },
#     'catFR1toFR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_catFR1toFR1_classifiers/' +
#             'RAM_catFR1toFR1_classifiers_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'  # ,
#         # 'participants': 'participants',
#         # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
#         # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
#     },
#     'catFR1': {
#         'classifier_path': (
#             rhino_mount +
#             '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval/RAM_catFR1lolo_classifiers_allfreqs/' +
#             'RAM_catFR1lolo_classifiers_allfreqs_withshuffles/'),
#         'target': 'target2',
#         'events': 'events2'
#     }
# }

sub_pow = {'R1026D', 'R1031M', 'R1035M', 'R1036M', 'R1042M', 'R1050M',
           'R1056M', 'R1060M', 'R1061T', 'R1065J', 'R1066P', 'R1069M',
           'R1074M', 'R1075J', 'R1089P', 'R1102P', 'R1105E', 'R1106M',
           'R1111M', 'R1112M', 'R1114C', 'R1131M', 'R1135E', 'R1147P',
           'R1158T', 'R1163T', 'R1167M', 'R1169P', 'R1170J', 'R1174T',
           'R1176M', 'R1178P', 'R1184M', 'R1189M', 'R1201P', 'R1202M',
           'R1204T', 'R1207J', 'R1215M', 'R1217T', 'R1222M', 'R1228M',
           'R1230J', 'R1231M', 'R1243T', 'R1251M', 'R1260D', 'R1264P',
           'R1274T', 'R1275D', 'R1277J', 'R1291M', 'R1293P', 'R1302M',
           'R1310J', 'R1313J', 'R1315T', 'R1317D', 'R1321M', 'R1324M',
           'R1328E', 'R1330D', 'R1332M', 'R1334T', 'R1337E', 'R1338T',
           'R1342M', 'R1347D', 'R1351M'}

plot_params = {
    'retrieval': {
        'FR1toCatFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_FR1toCatFR1_classifiers/' +
                'RAM_FR1toCatFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'probas': 'probas',
            'events': 'events2',
            'valid_evs_filter': 'valid_evs_filter2'
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'FR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs/' +
                'RAM_FR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target',
            'probas': 'probas',
            'events': 'events',
            'valid_evs_filter': 'valid_evs_filter'
        },
        'catFR1toFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_catFR1toFR1_classifiers/' +
                'RAM_catFR1toFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'probas': 'probas',
            'events': 'events2',
            'valid_evs_filter': 'valid_evs_filter2'
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'catFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_catFR1lolo_classifiers_allfreqs/' +
                'RAM_catFR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target',
            'probas': 'probas',
            'events': 'events',
            'valid_evs_filter': 'valid_evs_filter'
        }
    },
    'encoding': {
        'FR1toCatFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_FR1toCatFR1_classifiers/' +
                'RAM_FR1toCatFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'probas': 'probas',
            'events': 'events2',
            'valid_evs_filter': 'valid_evs_filter2'
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'FR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs/' +
                'RAM_FR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target',
            'probas': 'probas',
            'events': 'events',
            'valid_evs_filter': 'valid_evs_filter'
        },
        'catFR1toFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_catFR1toFR1_classifiers/' +
                'RAM_catFR1toFR1_classifiers_withshuffles/'),
            'target': 'target2',
            'probas': 'probas',
            'events': 'events2', 
            'valid_evs_filter': 'valid_evs_filter2'
            # 'participants': 'participants',
            # 'file_name': './figures/FR1toCatFR1_quantileplot_subj.pdf',
            # 'file_name_normbysess': './figures/FR1toCatFR1_quantileplot_subj_normbysess.pdf'
        },
        'catFR1': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_catFR1lolo_classifiers_allfreqs/' +
                'RAM_catFR1lolo_classifiers_allfreqs_withshuffles/'),
            'target': 'target',
            'probas': 'probas',
            'events': 'events',
            'valid_evs_filter': 'valid_evs_filter'
        },
    }   
}


roc_dat = {}
for encret in ['encoding', 'retrieval']:
    roc_dat[encret] = {}
    for label in ['catFR1', 'FR1', 'FR1toCatFR1', 'catFR1toFR1']:
        
        classifier_path = plot_params[encret][label]['classifier_path']
        classifier_path_hdf5 = classifier_path + 'hdf5_files/'
        c = 0.000719685673001
        classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
                                        '/*_classifier_info.hdf5'))
        roc_dat[encret][label] = {'probas': [],
                                  'targets': []}
        print encret, label
        filecount = 0
        for i, classifier_file in enumerate(classifier_files):
            #print filecount+1, 'of', len(sub_pow)
            subj = classifier_file.split('/')[-1].split('_classifier_info.hdf5')[0]
            if subj not in sub_pow:
                continue
            filecount += 1
            classdat = h5py.File(classifier_file, 'r')
            # if np.shape(np.ravel(classdat[
            #     plot_params[encret][label]['probas']].value[..., 1])) != np.shape(
            #         classdat[
            #             plot_params[encret][label]['target']].value):
            #     print classifier_file, np.shape(np.ravel(classdat[
            #         plot_params[encret][label]['probas']].value[..., 1])), np.shape(
            #             classdat[
            #                 plot_params[encret][label]['target']].value)
            roc_dat[encret][label]['probas'].extend(np.ravel(classdat[
                plot_params[encret][label]['probas']].value[..., 1]))
            roc_dat[encret][label]['targets'].extend(classdat[
                plot_params[encret][label]['target']].value[
                    classdat[plot_params[encret][label][
                        'valid_evs_filter']].value])
        

for encret in roc_dat:
    for label in roc_dat[encret]:
        (roc_dat[encret][label]['fpr'],
         roc_dat[encret][label]['tpr'],
         roc_dat[encret][label]['thresholds']) = roc_curve(
             roc_dat[encret][label]['targets'],
             roc_dat[encret][label]['probas'])


# fig4bdat = h5py.File('./jim/fig4b_data.hdf5')
fig4cdat = h5py.File('./jim/fig4c_data.hdf5')
aucs = fig4cdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)


fig, axs = plt.subplots(nrows=4, ncols=6, sharex=False, sharey=False,
                        figsize=(4.5, 3.5), gridspec_kw={'width_ratios': [1, 10, 2, 5, 10, 2],
                                                         'height_ratios': [10, 1, 10, 1]})

axs[0, 0].set_ylabel('Categorized', labelpad=5, weight='bold')
axs[0, 0].set_frame_on(False)
axs[0, 0].set_xticks([])
axs[0, 0].set_yticks([])

axs[0, 1].plot(roc_dat['encoding']['catFR1']['fpr'], roc_dat['encoding']['catFR1']['tpr'], 'k-', label='decoding')
axs[0, 1].plot(roc_dat['encoding']['catFR1toFR1']['fpr'], roc_dat['encoding']['catFR1toFR1']['tpr'], 'k--', label='cross-decoding')
axs[0, 1].set_xticks([0, 1])
axs[0, 1].set_yticks([0, 1])
axs[0, 1].set_aspect('equal')
axs[0, 1].set_xlabel('FA rate', labelpad=-5)
axs[0, 1].set_ylabel('hit rate', labelpad=-5)
#axs[0, 1].legend(loc='lower right', bbox_to_anchor=(0.5, 0.5))
fig.legend(loc='center', bbox_to_anchor=(0.5, 0.52), ncol=2, frameon=True)
axs[0, 2].errorbar(0, aucs_means[0], yerr=aucs_CI95s[0], fmt='ko', ecolor='k')
axs[0, 2].set_xticks([])
axs[0, 2].set_ylim((-0.04, 0.07))
#axs[0, 2].set_xlim((-0.5, 0.5))
axs[0, 2].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[0, 2].set_xlabel(r'$\Delta$ AUC')
axs[0, 2].set_yticks([0, 0.05])

axs[0, 3].set_visible(False)

axs[0, 4].plot(roc_dat['retrieval']['catFR1']['fpr'], roc_dat['retrieval']['catFR1']['tpr'], 'k-')
axs[0, 4].plot(roc_dat['retrieval']['catFR1toFR1']['fpr'], roc_dat['retrieval']['catFR1toFR1']['tpr'], 'k--')
axs[0, 4].set_xticks([0, 1])
axs[0, 4].set_yticks([0, 1])
axs[0, 4].set_aspect('equal')
axs[0, 4].set_xlabel('FA rate', labelpad=-5)
axs[0, 4].set_ylabel('hit rate', labelpad=-5)
axs[0, 5].set_xticks([])
axs[0, 5].set_ylim((-0.04, 0.07))
axs[0, 5].errorbar(0, aucs_means[2], yerr=aucs_CI95s[2], fmt='ko', ecolor='k')
axs[0, 5].set_xlabel(r'$\Delta$ AUC')
axs[0, 5].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[0, 5].set_yticks([0, 0.05])

axs[2, 0].set_ylabel('Unrelated', labelpad=5, weight='bold')
axs[2, 0].set_frame_on(False)
axs[2, 0].set_xticks([])
axs[2, 0].set_yticks([])

axs[2, 1].plot(roc_dat['encoding']['FR1']['fpr'], roc_dat['encoding']['FR1']['tpr'], 'k-')
axs[2, 1].plot(roc_dat['encoding']['FR1toCatFR1']['fpr'], roc_dat['encoding']['FR1toCatFR1']['tpr'], 'k--')
axs[2, 1].set_xticks([0, 1])
axs[2, 1].set_yticks([0, 1])
axs[2, 1].set_aspect('equal')
axs[2, 1].set_xlabel('FA rate', labelpad=-5)
axs[2, 1].set_ylabel('hit rate', labelpad=-5)
axs[2, 2].errorbar(0, aucs_means[1], yerr=aucs_CI95s[1], fmt='ko', ecolor='k')
axs[2, 2].set_xticks([])
axs[2, 2].set_ylim((-0.04, 0.07))
axs[2, 2].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[2, 2].set_xlabel(r'$\Delta$ AUC')
axs[2, 2].set_yticks([0, 0.05])
axs[2, 3].set_visible(False)


axs[2, 4].plot(roc_dat['retrieval']['FR1']['fpr'], roc_dat['retrieval']['FR1']['tpr'], 'k-')
axs[2, 4].plot(roc_dat['retrieval']['FR1toCatFR1']['fpr'], roc_dat['retrieval']['FR1toCatFR1']['tpr'], 'k--')
axs[2, 4].set_xticks([0, 1])
axs[2, 4].set_yticks([0, 1])
axs[2, 4].set_xlabel('FA rate', labelpad=-5)
axs[2, 4].set_ylabel('hit rate', labelpad=-5)
axs[2, 4].set_aspect('equal')
axs[2, 5].errorbar(0, aucs_means[3], yerr=aucs_CI95s[3], fmt='ko', ecolor='k')
axs[2, 5].set_xticks([])
axs[2, 5].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[2, 5].set_xlabel(r'$\Delta$ AUC')
axs[2, 5].set_ylim((-0.04, 0.07))
axs[2, 5].set_yticks([0, 0.05])

axs[3, 0].set_visible(False)
axs[3, 3].set_visible(False)

axs[1, 0].set_visible(False)
axs[1, 1].set_visible(False)
axs[1, 2].set_visible(False)
axs[1, 3].set_visible(False)
axs[1, 4].set_visible(False)
axs[1, 5].set_visible(False)


gs0 = gridspec.GridSpec(3, 6, width_ratios=[1, 10, 2, 5, 10, 2],
                        height_ratios=[10, 10, 1])
ax_enc = plt.subplot(gs0[2, 1:3])
ax_enc.set_xlabel('Encoding', labelpad=-5, weight='bold')
ax_enc.set_frame_on(False)
ax_enc.set_xticks([])
ax_enc.set_yticks([])

gs0 = gridspec.GridSpec(3, 6, width_ratios=[1, 10, 2, 5, 10, 2],
                        height_ratios=[10, 10, 1])
ax_ret = plt.subplot(gs0[2, 4:])
ax_ret.set_xlabel('Retrieval', labelpad=-5, weight='bold')
ax_ret.set_frame_on(False)
ax_ret.set_xticks([])
ax_ret.set_yticks([])

fig.subplots_adjust(left=0.05, right=0.92, bottom=0.02, top=0.99, wspace=0.2, hspace=0.5)
plt.savefig('./figures/pub/FRcatFR_decoding_xdecoding_rocs.pdf')


plot_params = {
    'encoding':{
        'lag2lag': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_encoding/'+
                'RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_encoding_withshuffles/'),
            'target': 'target',
            'probas': 'probas',
            'events': 'events',
            'valid_evs_filter': 'valid_evs_filter'
        },
        'cat2cat': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' + class_path_suffix +
                '/RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_encoding/'+
                'RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_encoding_withshuffles/'),
            'target': 'target',
            'events': 'events',
            'probas': 'probas',
            'valid_evs_filter': 'valid_evs_filter'
        },
        'lag2cat': {
            'classifier_path': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_encoding/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_encoding_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'probas': 'probas',
            'valid_evs_filter': 'valid_evs_filter2'
        },
        'cat2lag': {
            'classifier_path': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/encoding' +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_encoding/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_encoding_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'probas': 'probas',
            'valid_evs_filter': 'valid_evs_filter2'
        }
    },
    'retrieval':{
        'lag2lag': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_retrieval/'+
                'RAM_FR1lolo_classifiers_allfreqs_withinvsbetweenlagrecall_retrieval_withshuffles/'),
            'target': 'target',
            'probas': 'probas',
            'events': 'events',
            'valid_evs_filter': 'valid_evs_filter'
        },
        'cat2cat': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' + class_path_suffix +
                '/RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_retrieval/'+
                'RAM_catFR1lolo_classifiers_allfreqs_withinvsbetweencatrecall_retrieval_withshuffles/'),
            'target': 'target',
            'events': 'events',
            'probas': 'probas',
            'valid_evs_filter': 'valid_evs_filter'
        },
        'lag2cat': {
            'classifier_path': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_retrieval/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_retrieval_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'probas': 'probas',
            'valid_evs_filter': 'valid_evs_filter2'
        },
        'cat2lag': {
            'classifier_path': (
                rhino_mount +
                '/scratch/cweidema/RAM/RAM_FRcatFR/retrieval' +
                class_path_suffix +
                '/RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_retrieval/'+
                'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_retrieval_withshuffles/'),
            'target': 'target2',
            'events': 'events2',
            'probas': 'probas',
            'valid_evs_filter': 'valid_evs_filter2'
        }
    }
}


clustering_dat = h5py.File('./jim/fig6_final_data.hdf5')



# # aucs = np.array([fig6dat['lag_enc'], fig6dat['lag_rec'], fig6dat['cat_enc'], fig6dat['cat_rec']])[:, 0]
# # aucs_means = np.mean(aucs, 1)
# # aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
# # aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
aucs = np.array([clustering_dat['cat_enc'][:]-clustering_dat['cat2lag_enc'][:],
                 clustering_dat['cat_rec'][:]-clustering_dat['cat2lag_rec'][:],
                 clustering_dat['lag_enc'][:]-clustering_dat['lag2cat_enc'][:],
                 clustering_dat['lag_rec'][:]-clustering_dat['lag2cat_rec'][:]])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

for encret in roc_dat:  # ['encoding', 'retrieval']:
    roc_dat[encret] = {}
    for label in ['lag2lag', 'cat2cat', 'lag2cat', 'cat2lag']:
        
        classifier_path = plot_params[encret][label]['classifier_path']
        classifier_path_hdf5 = classifier_path + 'hdf5_files/'
        c = 0.000719685673001
        classifier_files = np.sort(glob(classifier_path_hdf5 + str(c) +
                                        '/*_classifier_info.hdf5'))
        roc_dat[encret][label] = {'probas': [],
                                  'targets': []}
        print encret, label
        filecount = 0
        for i, classifier_file in enumerate(classifier_files):
            #print filecount+1, 'of', len(sub_pow)
            subj = classifier_file.split('/')[-1].split('_classifier_info.hdf5')[0]
            if subj not in clustering_dat['subjects']:
                continue
            filecount += 1
            classdat = h5py.File(classifier_file, 'r')
            # if np.shape(np.ravel(classdat[
            #     plot_params[encret][label]['probas']].value[..., 1])) != np.shape(
            #         classdat[
            #             plot_params[encret][label]['target']].value):
            #     print classifier_file, np.shape(np.ravel(classdat[
            #         plot_params[encret][label]['probas']].value[..., 1])), np.shape(
            #             classdat[
            #                 plot_params[encret][label]['target']].value)
            roc_dat[encret][label]['probas'].extend(np.ravel(classdat[
                plot_params[encret][label]['probas']].value[..., 1]))
            roc_dat[encret][label]['targets'].extend(classdat[
                plot_params[encret][label]['target']].value[
                    classdat[plot_params[encret][label][
                        'valid_evs_filter']].value])
        

for encret in roc_dat:
    for label in roc_dat[encret]:
        (roc_dat[encret][label]['fpr'],
         roc_dat[encret][label]['tpr'],
         roc_dat[encret][label]['thresholds']) = roc_curve(
             roc_dat[encret][label]['targets'],
             roc_dat[encret][label]['probas'])



fig, axs = plt.subplots(nrows=4, ncols=6, sharex=False, sharey=False,
                        figsize=(4.5, 3.5), gridspec_kw={'width_ratios': [1, 10, 2, 5, 10, 2],
                                                         'height_ratios': [10, 1, 10, 1]})

axs[0, 0].set_ylabel('Categorical', labelpad=5, weight='bold')
axs[0, 0].set_frame_on(False)
axs[0, 0].set_xticks([])
axs[0, 0].set_yticks([])

axs[0, 1].plot(roc_dat['encoding']['cat2cat']['fpr'], roc_dat['encoding']['cat2cat']['tpr'], 'k-', label='decoding')
axs[0, 1].plot(roc_dat['encoding']['cat2lag']['fpr'], roc_dat['encoding']['cat2lag']['tpr'], 'k--', label='cross-decoding')
axs[0, 1].set_xticks([0, 1])
axs[0, 1].set_yticks([0, 1])
axs[0, 1].set_aspect('equal')
axs[0, 1].set_xlabel('FA rate', labelpad=-5)
axs[0, 1].set_ylabel('hit rate', labelpad=-5)
#axs[0, 1].legend(loc='lower right', bbox_to_anchor=(0.5, 0.5))
fig.legend(loc='center', bbox_to_anchor=(0.5, 0.52), ncol=2, frameon=True)
axs[0, 2].errorbar(0, aucs_means[0], yerr=aucs_CI95s[0], fmt='ko', ecolor='k')
axs[0, 2].set_xticks([])
axs[0, 2].set_ylim((-0.08, 0.14))
#axs[0, 2].set_xlim((-0.5, 0.5))
axs[0, 2].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[0, 2].set_xlabel(r'$\Delta$ AUC')
axs[0, 2].set_yticks([0, 0.1])

axs[0, 3].set_visible(False)

axs[0, 4].plot(roc_dat['retrieval']['cat2cat']['fpr'], roc_dat['retrieval']['cat2cat']['tpr'], 'k-')
axs[0, 4].plot(roc_dat['retrieval']['cat2lag']['fpr'], roc_dat['retrieval']['cat2lag']['tpr'], 'k--')
axs[0, 4].set_xticks([0, 1])
axs[0, 4].set_yticks([0, 1])
axs[0, 4].set_aspect('equal')
axs[0, 4].set_xlabel('FA rate', labelpad=-5)
axs[0, 4].set_ylabel('hit rate', labelpad=-5)
axs[0, 5].set_xticks([])
axs[0, 5].set_ylim((-0.08, 0.14))
axs[0, 5].errorbar(0, aucs_means[1], yerr=aucs_CI95s[1], fmt='ko', ecolor='k')
axs[0, 5].set_xlabel(r'$\Delta$ AUC')
axs[0, 5].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[0, 5].set_yticks([0, 0.1])

axs[2, 0].set_ylabel('Temporal', labelpad=5, weight='bold')
axs[2, 0].set_frame_on(False)
axs[2, 0].set_xticks([])
axs[2, 0].set_yticks([])

axs[2, 1].plot(roc_dat['encoding']['lag2lag']['fpr'], roc_dat['encoding']['lag2lag']['tpr'], 'k-')
axs[2, 1].plot(roc_dat['encoding']['lag2cat']['fpr'], roc_dat['encoding']['lag2cat']['tpr'], 'k--')
axs[2, 1].set_xticks([0, 1])
axs[2, 1].set_yticks([0, 1])
axs[2, 1].set_aspect('equal')
axs[2, 1].set_xlabel('FA rate', labelpad=-5)
axs[2, 1].set_ylabel('hit rate', labelpad=-5)
axs[2, 2].errorbar(0, aucs_means[2], yerr=aucs_CI95s[2], fmt='ko', ecolor='k')
axs[2, 2].set_xticks([])
axs[2, 2].set_ylim((-0.08, 0.14))
axs[2, 2].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[2, 2].set_xlabel(r'$\Delta$ AUC')
axs[2, 2].set_yticks([0, 0.1])
axs[2, 3].set_visible(False)


axs[2, 4].plot(roc_dat['retrieval']['lag2lag']['fpr'], roc_dat['retrieval']['lag2lag']['tpr'], 'k-')
axs[2, 4].plot(roc_dat['retrieval']['lag2cat']['fpr'], roc_dat['retrieval']['lag2cat']['tpr'], 'k--')
axs[2, 4].set_xticks([0, 1])
axs[2, 4].set_yticks([0, 1])
axs[2, 4].set_xlabel('FA rate', labelpad=-5)
axs[2, 4].set_ylabel('hit rate', labelpad=-5)
axs[2, 4].set_aspect('equal')
axs[2, 5].errorbar(0, aucs_means[3], yerr=aucs_CI95s[3], fmt='ko', ecolor='k')
axs[2, 5].set_xticks([])
axs[2, 5].tick_params(direction='in', left=True, right=True, labelleft=False, labelright=True)
axs[2, 5].set_xlabel(r'$\Delta$ AUC')
axs[2, 5].set_ylim((-0.08, 0.14))
axs[2, 5].set_yticks([0, 0.1])

axs[3, 0].set_visible(False)
axs[3, 3].set_visible(False)

axs[1, 0].set_visible(False)
axs[1, 1].set_visible(False)
axs[1, 2].set_visible(False)
axs[1, 3].set_visible(False)
axs[1, 4].set_visible(False)
axs[1, 5].set_visible(False)



gs0 = gridspec.GridSpec(3, 6, width_ratios=[1, 10, 2, 5, 10, 2],
                        height_ratios=[10, 10, 1])
ax_enc = plt.subplot(gs0[2, 1:3])
ax_enc.set_xlabel('Encoding', labelpad=-5, weight='bold')
ax_enc.set_frame_on(False)
ax_enc.set_xticks([])
ax_enc.set_yticks([])

gs0 = gridspec.GridSpec(3, 6, width_ratios=[1, 10, 2, 5, 10, 2],
                        height_ratios=[10, 10, 1])
ax_ret = plt.subplot(gs0[2, 4:])
ax_ret.set_xlabel('Retrieval', labelpad=-5, weight='bold')
ax_ret.set_frame_on(False)
ax_ret.set_xticks([])
ax_ret.set_yticks([])

fig.subplots_adjust(left=0.05, right=0.92, bottom=0.02, top=0.99, wspace=0.2, hspace=0.5)
plt.savefig('./figures/pub/FRcatFR_decoding_xdecoding_lagcat_rocs.pdf')









#### ROC plots stop here!

































fig4bdat = h5py.File('./jim/fig4b_data.hdf5')
fig4cdat = h5py.File('./jim/fig4c_data.hdf5')

# decodecolor = 'dimgray'
# xdecodecolor = 'lightgray'

decodecolor = np.array([[0.2, 0.2, 0.2, 0.5]])
xdecodecolor = np.array([[0.6, 0.6, 0.6, 0.5]])



# fig = plt.figure(figsize=(6.92912/2, 3))
# gs = gridspec.GridSpec(nrows=2, ncols=2, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
# aucs = fig4bdat['data']
# aucs_means = np.mean(aucs, 1)
# aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
# aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
# ax1a = plt.subplot(gs[0, 0])
# positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
# parts = ax1a.violinplot(dataset=np.transpose(fig4bdat['data'])[:, :4], showextrema=False,
#                        positions=positions)
# for i, pc in enumerate(parts['bodies']):
#     pc.set_edgecolor('black')
#     pc.set_alpha('0.5')
#     pc.set_linewidth(2)
#     if i%2 == 0:
#         pc.set_facecolor(decodecolor)
#         pc.set_label('decoding')
#     else:
#         pc.set_facecolor(xdecodecolor)
#         pc.set_label('cross-decoding')
# ax1a.errorbar(x=positions, y=aucs_means[:4], yerr=aucs_CI95s[:4], marker='o', ecolor='black',
#              elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax1a.plot([0.5, 5.5], [0.5, 0.5], 'k--')
# ax1a.set_xlim((0.5, 5.5))
# ax1a.set_ylim((0.36, 1.0))
# ax1a.set_yticks(np.arange(0.4, 1.1, 0.1))
# # ax1.set_ylim((0., 0.65))
# # ax1.set_ylabel('p(recall)')#, size='large')
# ax1a.xaxis.set_visible(False)
# sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
# ax1a.yaxis.set_tick_params(pad=3)
# ax1a.set_ylabel('AUC', labelpad=6.5)
# legend_handles = [
#     mpatches.Patch(
#         facecolor=decodecolor, edgecolor='black', alpha=0.5, linewidth=2),
#     mpatches.Patch(
#         facecolor=xdecodecolor, edgecolor='black', alpha=0.5, linewidth=2)]
# legend_labels = ['decoding', 'cross-decoding']
# ax1a.legend(legend_handles, legend_labels, loc='upper left',
#             frameon=False, borderpad=0)

# ax2a = plt.subplot(gs[0, 1])
# positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
# parts = ax2a.violinplot(dataset=np.transpose(fig4bdat['data'])[:, 4:], showextrema=False,
#                        positions=positions)
# for i, pc in enumerate(parts['bodies']):
#     pc.set_edgecolor('black')
#     pc.set_alpha('0.5')
#     pc.set_linewidth(2)
#     if i%2 == 0:
#         pc.set_facecolor(decodecolor)
#         pc.set_label('decoding')
#     else:
#         pc.set_facecolor(xdecodecolor)
#         pc.set_label('cross-decoding')
# ax2a.errorbar(x=positions, y=aucs_means[4:], yerr=aucs_CI95s[4:], marker='o', ecolor='black',
#              elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax2a.plot([0, 5], [0.5, 0.5], 'k--')
# ax2a.set_xlim((0, 5))
# ax2a.set_ylim((0.36, 1.0))
# # ax1.set_yticks(np.arange(0, 0.7, 0.2))
# # ax1.set_ylim((0., 0.65))
# # ax1.set_ylabel('p(recall)')#, size='large')
# ax2a.xaxis.set_visible(False)
# ax2a.yaxis.set_visible(False)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# # ax2a.yaxis.set_tick_params(pad=3)
# # ax1.legend(loc='upper left',
# #            frameon=False, borderpad=0)

# # positions = [1.5, 4, 7, 9.5]
# positions = [1.5, 4]
# aucs = fig4cdat['data']
# aucs_means = np.mean(aucs, 1)
# aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
# aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

# ax1b = plt.subplot(gs[1, 0])
# ax1b.errorbar(x=positions, y=aucs_means[:2], yerr=aucs_CI95s[:2], marker='o', ecolor='black',
#               elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax1b.plot([0.5, 5.5], [0, 0], 'k--')
# ax1b.set_xlim((0.5, 5.5))
# # ax2.set_ylim((-0.5, 0.5))
# # ax2.set_ylim((-0.5, 0.5))
# ax1b.set_ylim((-0.035, 0.065))
# sns.despine(offset=0, ax=ax1b)
# # sns.despine(offset=0, ax=ax1b)
# ax1b.xaxis.set_tick_params(pad=3)
# ax1b.yaxis.set_tick_params(pad=3)
# ax1b.set_ylabel(r'$\Delta$ AUC', labelpad=0)
# ax1b.set_xticks(positions)
# ax1b.set_xticklabels(['C', 'U'])
# ax1b.set_xlabel('Encoding', x=0.45)


# ax2b = plt.subplot(gs[1, 1])
# ax2b.errorbar(x=positions, y=aucs_means[2:], yerr=aucs_CI95s[2:], marker='o', ecolor='black',
#               elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax2b.plot([0, 5], [0, 0], 'k--')
# ax2b.set_xlim((0, 5))
# ax2b.set_ylim((-0.035, 0.065))
# # ax2.set_ylim((-0.5, 0.5))
# # ax2.set_ylim((-0.5, 0.5))
# #ax2.set_ylim((-0.034, 0.065))
# # sns.despine(offset=0, ax=ax2b)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=False, ax=ax2b)
# # sns.despine(offset=0, ax=ax2b)
# ax2b.yaxis.set_visible(False)
# ax2b.xaxis.set_tick_params(pad=3)
# #ax2b.yaxis.set_tick_params(pad=3)
# ax2b.set_xticks(positions)
# ax2b.set_xticklabels(['C', 'U'])
# ax2b.set_xlabel('Retrieval', x=0.55)

# fig.subplots_adjust(left=0.16, bottom=0.15, right=0.99, top=0.98,
#                     wspace=0, hspace=0.1)
# plt.savefig('./figures/pub/FRcatFR_decoding_xdecoding.pdf')












fig = plt.figure(figsize=(6.92912/2, 3))
gs = gridspec.GridSpec(nrows=2, ncols=2, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
aucs = fig4bdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax1a.violinplot(dataset=np.transpose(fig4bdat['data'])[:, :4], showextrema=False,
                       positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax1a.errorbar(x=positions, y=aucs_means[:4], yerr=aucs_CI95s[:4], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.5], [0.5, 0.5], 'k--')
ax1a.set_xlim((0.5, 5.5))
ax1a.set_ylim((0.36, 1.0))
ax1a.set_yticks(np.arange(0.4, 1.1, 0.1))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.xaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
ax1a.yaxis.set_tick_params(pad=3)
ax1a.set_ylabel('AUC', labelpad=6.5)
legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['decoding', 'cross-decoding']
ax1a.legend(legend_handles, legend_labels, loc='upper left',
            frameon=False, borderpad=0)

ax2a = plt.subplot(gs[0, 1])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax2a.violinplot(dataset=np.transpose(fig4bdat['data'])[:, 4:], showextrema=False,
                       positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax2a.errorbar(x=positions, y=aucs_means[4:], yerr=aucs_CI95s[4:], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax2a.plot([0, 5], [0.5, 0.5], 'k--')
ax2a.set_xlim((0, 5))
ax2a.set_ylim((0.36, 1.0))
# ax1.set_yticks(np.arange(0, 0.7, 0.2))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax2a.xaxis.set_visible(False)
ax2a.yaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# ax2a.yaxis.set_tick_params(pad=3)
# ax1.legend(loc='upper left',
#            frameon=False, borderpad=0)

# positions = [1.5, 4, 7, 9.5]
positions = [1.5, 4]
aucs = fig4cdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

ax1b = plt.subplot(gs[1, 0])
ax1b.errorbar(x=positions, y=aucs_means[:2], yerr=aucs_CI95s[:2], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1b.plot([0.5, 5.5], [0, 0], 'k--')
ax1b.set_xlim((0.5, 5.5))
# ax2.set_ylim((-0.5, 0.5))
# ax2.set_ylim((-0.5, 0.5))
ax1b.set_ylim((-0.035, 0.065))
sns.despine(offset=0, ax=ax1b)
# sns.despine(offset=0, ax=ax1b)
ax1b.xaxis.set_tick_params(pad=3)
ax1b.yaxis.set_tick_params(pad=3)
ax1b.set_ylabel(r'$\Delta$ AUC', labelpad=0)
ax1b.set_xticks(positions)
ax1b.set_xticklabels(['categorized', 'unrelated'])
ax1b.set_xlabel('Encoding', x=0.45)


ax2b = plt.subplot(gs[1, 1])
ax2b.errorbar(x=positions, y=aucs_means[2:], yerr=aucs_CI95s[2:], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax2b.plot([0, 5], [0, 0], 'k--')
ax2b.set_xlim((0, 5))
ax2b.set_ylim((-0.035, 0.065))
# ax2.set_ylim((-0.5, 0.5))
# ax2.set_ylim((-0.5, 0.5))
#ax2.set_ylim((-0.034, 0.065))
# sns.despine(offset=0, ax=ax2b)
sns.despine(offset=0, top=True, right=True, left=True, bottom=False, ax=ax2b)
# sns.despine(offset=0, ax=ax2b)
ax2b.yaxis.set_visible(False)
ax2b.xaxis.set_tick_params(pad=3)
#ax2b.yaxis.set_tick_params(pad=3)
ax2b.set_xticks(positions)
ax2b.set_xticklabels(['categorized', 'unrelated'])
ax2b.set_xlabel('Retrieval', x=0.55)

fig.subplots_adjust(left=0.16, bottom=0.15, right=0.99, top=0.98,
                    wspace=0, hspace=0.1)
plt.savefig('./figures/pub/FRcatFR_decoding_xdecoding.pdf')






fig6adat = h5py.File('./jim/fig6a_data.hdf5')
fig6bdat = h5py.File('./jim/fig6b_data.hdf5')


# catcolor = 'blue'
# unrcolor = 'yellow'
#difcolor = 'green'
# decodecolor = 'dimgray'
# xdecodecolor = 'lightgray'

fig = plt.figure(figsize=(6.92912/2, 3))
gs = gridspec.GridSpec(nrows=2, ncols=1, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
aucs = fig6adat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax1a.violinplot(dataset=np.transpose(fig6adat['data'])[:, :], showextrema=False,
                        positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('encoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('retrieval')
ax1a.errorbar(x=positions, y=aucs_means, yerr=aucs_CI95s, marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.0], [0.5, 0.5], 'k--')
ax1a.set_xlim((0.5, 5.0))
ax1a.set_ylim((0.0, 1.0))
# ax1a.set_yticks(np.arange(0.4, 1.1, 0.1))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.xaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
ax1a.set_yticks(np.arange(0, 1.1, 0.1))
ax1a.yaxis.set_tick_params(pad=3)
ax1a.set_ylabel('AUC', labelpad=6.5)

legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['encoding', 'retrieval']

# ax1a.legend(legend_handles, legend_labels, loc='lower right',
#             frameon=False, borderpad=-.5)
ax1a.legend(legend_handles, legend_labels, loc='upper right',
            frameon=False, borderpad=-.5)

# positions = [1.5, 4, 7, 9.5]
# positions = [1.5, 2.75, 4.0]

# order of the conditions in fig6bdat is temporal, categorical,
# interaction, we plot the interaction in the middle so permute the
# position values accordingly:
positions = [1.5, 4.0, 2.75]
aucs = fig6bdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

ax1b = plt.subplot(gs[1, 0])
ax1b.errorbar(x=positions, y=aucs_means, yerr=aucs_CI95s, marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)

ax1b.plot([0.5, 5.5], [0, 0], 'k--')
ax1b.set_xlim((0.5, 5))
ax1b.set_ylim((-0.16, 0.21))
sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1b)
# sns.despine(offset=0, ax=ax1b)
# ax1b.xaxis.set_visible(False)
ax1b.set_yticks(np.arange(-.1, 0.21, 0.1))
# ax1b.yaxis.set_visible(False)
# ax1b.xaxis.set_tick_params(pad=3)
# ax1b.yaxis.set_tick_params(pad=3)
ax1b.set_ylabel(r'$\Delta$ AUC', labelpad=0)

ax1b.set_xticks(positions)
# ax1b.set_xticklabels(['temporal','interaction','categorical'])

# because the position coordinates are out of order interaction needs
# to be last to be plotted in the middel:
ax1b.set_xticklabels(['temporal','categorical','interaction'])

# ax1b.set_xlabel('Temporal', x=0.45)
#
# positions = [4.0]
# ax2b = plt.subplot(gs[1, 1])
# ax2b.errorbar(x=positions, y=aucs_means[1:2], yerr=aucs_CI95s[1:2], marker='o', ecolor='black',
#               elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax2b.plot([3, 5], [0, 0], 'k--')
# ax2b.set_xlim((3, 5))
# ax2b.set_ylim((-0.2, 0.1))
# #ax2.set_ylim((-0.034, 0.065))
# # sns.despine(offset=0, ax=ax2b)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2b)
# # sns.despine(offset=0, ax=ax2b)
# ax2b.yaxis.set_visible(False)
# ax2b.xaxis.set_visible(False)
# #ax2b.yaxis.set_tick_params(pad=3)
# # ax2b.set_xticks(positions)
# # ax2b.set_xticklabels(['C'])
# # ax2b.set_xlabel('Retrieval', x=0.55)

# and the third panel

fig.subplots_adjust(left=0.16, bottom=0.09, right=0.99, top=0.98,
                    wspace=0, hspace=0.15)

plt.savefig('./figures/pub/FRcatFR_clusteringclassifier.pdf')













fig6dat = h5py.File('./jim/fig6_final_data.hdf5')

# catcolor = 'blue'
# unrcolor = 'yellow'
#difcolor = 'green'
# decodecolor = 'dimgray'
# xdecodecolor = 'lightgray'



fig = plt.figure(figsize=(6.92912/2, 3))
gs = gridspec.GridSpec(nrows=2, ncols=1, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
aucs = np.array([fig6dat['lag_enc'], fig6dat['lag_rec'], fig6dat['cat_enc'], fig6dat['cat_rec']])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax1a.violinplot(dataset=np.transpose(aucs), showextrema=False,
                        positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('encoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('retrieval')
ax1a.errorbar(x=positions, y=aucs_means, yerr=aucs_CI95s, marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.0], [0.5, 0.5], 'k--')
ax1a.set_xlim((0.5, 5.0))
ax1a.set_ylim((0.0, 1.0))
# ax1a.set_yticks(np.arange(0.4, 1.1, 0.1))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.xaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
ax1a.set_yticks(np.arange(0, 1.1, 0.1))
ax1a.yaxis.set_tick_params(pad=3)
ax1a.set_ylabel('AUC', labelpad=6.5)

legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['encoding', 'retrieval']

# ax1a.legend(legend_handles, legend_labels, loc='lower right',
#             frameon=False, borderpad=-.5)
ax1a.legend(legend_handles, legend_labels, loc='upper right',
            frameon=False, borderpad=-.5)

# positions = [1.5, 4, 7, 9.5]
# positions = [1.5, 2.75, 4.0]

# order of the conditions in fig6bdat is temporal, categorical,
# interaction, we plot the interaction in the middle so permute the
# position values accordingly:
positions = [1.5, 2.75, 4.0]
#aucs = fig6bdat['data']
aucs = np.array([fig6dat['lag_enc'][:]-fig6dat['lag_rec'][:],
                 ((fig6dat['lag_enc'][:]-fig6dat['lag_rec'][:]) -
                  (fig6dat['cat_enc'][:]-fig6dat['cat_rec'][:])),
                 fig6dat['cat_enc'][:]-fig6dat['cat_rec'][:]])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

ax1b = plt.subplot(gs[1, 0])
ax1b.errorbar(x=positions, y=aucs_means, yerr=aucs_CI95s, marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)

ax1b.plot([0.5, 5.5], [0, 0], 'k--')
ax1b.set_xlim((0.5, 5))
ax1b.set_ylim((-0.16, 0.21))
sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1b)
# sns.despine(offset=0, ax=ax1b)
# ax1b.xaxis.set_visible(False)
ax1b.set_yticks(np.arange(-.1, 0.21, 0.1))
# ax1b.yaxis.set_visible(False)
# ax1b.xaxis.set_tick_params(pad=3)
# ax1b.yaxis.set_tick_params(pad=3)
ax1b.set_ylabel(r'$\Delta$ AUC', labelpad=0)

ax1b.set_xticks(positions)
# ax1b.set_xticklabels(['temporal','interaction','categorical'])

# because the position coordinates are out of order interaction needs
# to be last to be plotted in the middel:
ax1b.set_xticklabels(['temporal', 'interaction', 'categorical'])

# ax1b.set_xlabel('Temporal', x=0.45)
#
# positions = [4.0]
# ax2b = plt.subplot(gs[1, 1])
# ax2b.errorbar(x=positions, y=aucs_means[1:2], yerr=aucs_CI95s[1:2], marker='o', ecolor='black',
#               elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax2b.plot([3, 5], [0, 0], 'k--')
# ax2b.set_xlim((3, 5))
# ax2b.set_ylim((-0.2, 0.1))
# #ax2.set_ylim((-0.034, 0.065))
# # sns.despine(offset=0, ax=ax2b)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2b)
# # sns.despine(offset=0, ax=ax2b)
# ax2b.yaxis.set_visible(False)
# ax2b.xaxis.set_visible(False)
# #ax2b.yaxis.set_tick_params(pad=3)
# # ax2b.set_xticks(positions)
# # ax2b.set_xticklabels(['C'])
# # ax2b.set_xlabel('Retrieval', x=0.55)

# and the third panel

fig.subplots_adjust(left=0.16, bottom=0.09, right=0.99, top=0.98,
                    wspace=0, hspace=0.15)

# plt.savefig('./figures/pub/FRcatFR_clusteringclassifier.pdf')

























# fig6dat = h5py.File('./jim/fig6_final_data.hdf5')

# catcolor = 'blue'
# unrcolor = 'yellow'
#difcolor = 'green'
# decodecolor = 'dimgray'
# xdecodecolor = 'lightgray'

fig = plt.figure(figsize=(6.92912/2, 3))
gs = gridspec.GridSpec(nrows=2, ncols=1, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
aucs = np.array([fig6dat['lag2cat_enc'], fig6dat['lag2cat_rec'],
                 fig6dat['cat2lag_enc'], fig6dat['cat2lag_rec']])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax1a.violinplot(dataset=np.transpose(aucs), showextrema=False,
                        positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('encoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('retrieval')
ax1a.errorbar(x=positions, y=aucs_means, yerr=aucs_CI95s, marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.0], [0.5, 0.5], 'k--')
ax1a.set_xlim((0.5, 5.0))
ax1a.set_ylim((0.0, 1.0))
# ax1a.set_yticks(np.arange(0.4, 1.1, 0.1))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.xaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
ax1a.set_yticks(np.arange(0, 1.1, 0.1))
ax1a.yaxis.set_tick_params(pad=3)
ax1a.set_ylabel('AUC', labelpad=6.5)

legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['encoding', 'retrieval']

# ax1a.legend(legend_handles, legend_labels, loc='lower right',
#             frameon=False, borderpad=-.5)
ax1a.legend(legend_handles, legend_labels, loc='upper right',
            frameon=False, borderpad=-.5)

# positions = [1.5, 4, 7, 9.5]
# positions = [1.5, 2.75, 4.0]

# order of the conditions in fig6bdat is temporal, categorical,
# interaction, we plot the interaction in the middle so permute the
# position values accordingly:
positions = [1.5, 2.75, 4.0]
#aucs = fig6bdat['data']
aucs = np.array([fig6dat['lag2cat_enc'][:]-fig6dat['lag2cat_rec'][:],
                 ((fig6dat['lag2cat_enc'][:]-fig6dat['lag2cat_rec'][:]) -
                  (fig6dat['cat2lag_enc'][:]-fig6dat['cat2lag_rec'][:])),
                 fig6dat['cat2lag_enc'][:]-fig6dat['cat2lag_rec'][:]])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

ax1b = plt.subplot(gs[1, 0])
ax1b.errorbar(x=positions, y=aucs_means, yerr=aucs_CI95s, marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)

ax1b.plot([0.5, 5.5], [0, 0], 'k--')
ax1b.set_xlim((0.5, 5))
ax1b.set_ylim((-0.16, 0.21))
sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1b)
# sns.despine(offset=0, ax=ax1b)
# ax1b.xaxis.set_visible(False)
ax1b.set_yticks(np.arange(-.1, 0.21, 0.1))
# ax1b.yaxis.set_visible(False)
# ax1b.xaxis.set_tick_params(pad=3)
# ax1b.yaxis.set_tick_params(pad=3)
ax1b.set_ylabel(r'$\Delta$ AUC', labelpad=0)

ax1b.set_xticks(positions)
# ax1b.set_xticklabels(['temporal','interaction','categorical'])

# because the position coordinates are out of order interaction needs
# to be last to be plotted in the middel:
ax1b.set_xticklabels(['temporal', 'interaction', 'categorical'])

# ax1b.set_xlabel('Temporal', x=0.45)
#
# positions = [4.0]
# ax2b = plt.subplot(gs[1, 1])
# ax2b.errorbar(x=positions, y=aucs_means[1:2], yerr=aucs_CI95s[1:2], marker='o', ecolor='black',
#               elinewidth=2, mfc='white', mec='black', linestyle='', ms=4)
# ax2b.plot([3, 5], [0, 0], 'k--')
# ax2b.set_xlim((3, 5))
# ax2b.set_ylim((-0.2, 0.1))
# #ax2.set_ylim((-0.034, 0.065))
# # sns.despine(offset=0, ax=ax2b)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2b)
# # sns.despine(offset=0, ax=ax2b)
# ax2b.yaxis.set_visible(False)
# ax2b.xaxis.set_visible(False)
# #ax2b.yaxis.set_tick_params(pad=3)
# # ax2b.set_xticks(positions)
# # ax2b.set_xticklabels(['C'])
# # ax2b.set_xlabel('Retrieval', x=0.55)

# and the third panel

fig.subplots_adjust(left=0.16, bottom=0.09, right=0.99, top=0.98,
                    wspace=0, hspace=0.15)

# plt.savefig('./figures/pub/FRcatFR_clusteringclassifier.pdf')













fig = plt.figure(figsize=(6.92912/2, 3))
gs = gridspec.GridSpec(nrows=2, ncols=2, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
# aucs = fig4bdat['data']
aucs = np.array([fig6dat['cat_enc'], fig6dat['cat2lag_enc'],
                 fig6dat['lag_enc'], fig6dat['lag2cat_enc'],
                 fig6dat['cat_rec'], fig6dat['cat2lag_rec'],
                 fig6dat['lag_rec'], fig6dat['lag2cat_rec']])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax1a.violinplot(dataset=np.transpose(aucs)[:, :4], showextrema=False,
                       positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax1a.errorbar(x=positions, y=aucs_means[:4], yerr=aucs_CI95s[:4], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.5], [0.5, 0.5], 'k--')
ax1a.set_xlim((0.5, 5.5))
# ax1a.set_ylim((0.36, 1.0))
ax1a.set_ylim((0, 1.0))
# ax1a.set_yticks(np.arange(0.4, 1.1, 0.1))
ax1a.set_yticks(np.arange(0, 1.1, 0.2))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.xaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
ax1a.yaxis.set_tick_params(pad=3)
ax1a.set_ylabel('AUC', labelpad=6.5)
legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['decoding', 'cross-decoding']
ax1a.legend(legend_handles, legend_labels, loc='upper left',
            frameon=False, borderpad=0)

ax2a = plt.subplot(gs[0, 1])
positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
parts = ax2a.violinplot(dataset=np.transpose(aucs)[:, 4:], showextrema=False,
                       positions=positions)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax2a.errorbar(x=positions, y=aucs_means[4:], yerr=aucs_CI95s[4:], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax2a.plot([0, 5], [0.5, 0.5], 'k--')
ax2a.set_xlim((0, 5))
# ax2a.set_ylim((0.36, 1.0))
ax2a.set_ylim((0, 1.0))
# ax1.set_yticks(np.arange(0, 0.7, 0.2))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax2a.xaxis.set_visible(False)
ax2a.yaxis.set_visible(False)
sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# ax2a.yaxis.set_tick_params(pad=3)
# ax1.legend(loc='upper left',
#            frameon=False, borderpad=0)

# positions = [1.5, 4, 7, 9.5]
positions = [1.5, 4]
#aucs = fig4cdat['data']
aucs = np.array([fig6dat['cat_enc'][:]-fig6dat['cat2lag_enc'][:],
                 fig6dat['lag_enc'][:]-fig6dat['lag2cat_enc'][:],
                 fig6dat['cat_rec'][:]-fig6dat['cat2lag_rec'][:],
                 fig6dat['lag_rec'][:]-fig6dat['lag2cat_rec'][:]])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)

ax1b = plt.subplot(gs[1, 0])
ax1b.errorbar(x=positions, y=aucs_means[:2], yerr=aucs_CI95s[:2], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1b.plot([0.5, 5.5], [0, 0], 'k--')
ax1b.set_xlim((0.5, 5.5))
# ax2.set_ylim((-0.5, 0.5))
# ax2.set_ylim((-0.5, 0.5))
# ax1b.set_ylim((-0.035, 0.065))
# ax1b.set_ylim((-0.035, 0.065))
ax1b.set_ylim((-0.1, 0.2))
sns.despine(offset=0, ax=ax1b)
# sns.despine(offset=0, ax=ax1b)
ax1b.xaxis.set_tick_params(pad=3)
ax1b.yaxis.set_tick_params(pad=3)
ax1b.set_ylabel(r'$\Delta$ AUC', labelpad=0)
ax1b.set_xticks(positions)
ax1b.set_xticklabels(['categorized', 'unrelated'])
ax1b.set_xlabel('Encoding', x=0.45)


ax2b = plt.subplot(gs[1, 1])
ax2b.errorbar(x=positions, y=aucs_means[2:], yerr=aucs_CI95s[2:], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax2b.plot([0, 5], [0, 0], 'k--')
ax2b.set_xlim((0, 5))
ax2b.set_ylim((-0.1, 0.2))
# ax2.set_ylim((-0.5, 0.5))
# ax2.set_ylim((-0.5, 0.5))
#ax2.set_ylim((-0.034, 0.065))
# sns.despine(offset=0, ax=ax2b)
sns.despine(offset=0, top=True, right=True, left=True, bottom=False, ax=ax2b)
# sns.despine(offset=0, ax=ax2b)
ax2b.yaxis.set_visible(False)
ax2b.xaxis.set_tick_params(pad=3)
#ax2b.yaxis.set_tick_params(pad=3)
ax2b.set_xticks(positions)
ax2b.set_xticklabels(['categorized', 'unrelated'])
ax2b.set_xlabel('Retrieval', x=0.55)

fig.subplots_adjust(left=0.16, bottom=0.15, right=0.99, top=0.98,
                    wspace=0, hspace=0.1)
#plt.savefig('./figures/pub/FRcatFR_decoding_xdecoding_lagcat.pdf')














fig = plt.figure(figsize=(6.92912/2, 3))
# gs = gridspec.GridSpec(nrows=2, ncols=2, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
gs = gridspec.GridSpec(nrows=1, ncols=2)#, width_ratios=[5, 5, 5, 7])
aucs = fig4cdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
# positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
positions = [1.5, 4]
parts = ax1a.violinplot(dataset=np.transpose(fig4cdat['data'])[:, :2], showextrema=False,
                        positions=positions, widths=1)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax1a.errorbar(x=positions, y=aucs_means[:2], yerr=aucs_CI95s[:2], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.5], [0, 0], 'k--')
ax1a.set_xlim((0.5, 5.5))
ax1a.set_ylim((-0.32, 0.32))
ax1a.set_yticks(np.arange(-0.3, 0.31, 0.1))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.set_ylabel(r'$\Delta$ AUC (decoding $-$ cross-decoding)', labelpad=0)
ax1a.set_xticks(positions)
# ax1a.set_xticklabels(['categorized', 'unrelated'])
ax1a.set_xticklabels(['', ''])
ax1a.set_xlabel('Encoding', x=0.45)
ax1a.tick_params(axis='x', direction='in')
# sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
#sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1a, trim=True)
sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1a, trim=True)
# ax1a.yaxis.set_tick_params(pad=3)
legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['categorized', 'unrelated']
ax1a.legend(legend_handles, legend_labels, loc='upper left',
            frameon=False, borderpad=0)#, borderaxespad=0.3)

ax2a = plt.subplot(gs[0, 1])
parts = ax2a.violinplot(dataset=np.transpose(fig4cdat['data'])[:, 2:], showextrema=False,
                        positions=positions, widths=1)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax2a.errorbar(x=positions, y=aucs_means[2:], yerr=aucs_CI95s[2:], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax2a.plot([0, 5], [0, 0], 'k--')
ax2a.set_xlim((0, 5))
# ax2a.set_ylim((0.36, 1.0))
ax2a.set_ylim((-0.32, 0.32))
# ax1.set_yticks(np.arange(0, 0.7, 0.2))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
# ax2a.xaxis.set_visible(False)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# ax2a.yaxis.set_tick_params(pad=3)
# ax1.legend(loc='upper left',
#            frameon=False, borderpad=0)

# positions = [1.5, 4, 7, 9.5]
positions = [1.5, 4]
aucs = fig4cdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# sns.despine(offset=0, ax=ax2b)
ax2a.yaxis.set_visible(False)
ax2a.xaxis.set_tick_params(pad=3)
#ax2b.yaxis.set_tick_params(pad=3)
ax2a.set_xticks(positions)
# ax2a.set_xticklabels(['categorized', 'unrelated'])
ax2a.set_xticklabels(['', ''])
ax2a.tick_params(axis='x', direction='in')
ax2a.set_xlabel('Retrieval', x=0.55)
sns.despine(offset=0, top=True, right=True, left=True, bottom=False, ax=ax2a, trim=True)

fig.subplots_adjust(left=0.17, bottom=0.08, right=0.999, top=0.99,
                    wspace=0, hspace=0.1)
plt.savefig('./figures/pub/FRcatFR_decoding_xdecoding_delta.pdf')









fig = plt.figure(figsize=(6.92912/2, 3))
# gs = gridspec.GridSpec(nrows=2, ncols=2, height_ratios=[3, 1])#, width_ratios=[5, 5, 5, 7])
gs = gridspec.GridSpec(nrows=1, ncols=2)#, width_ratios=[5, 5, 5, 7])
aucs = np.array([fig6dat['cat_enc'][:]-fig6dat['cat2lag_enc'][:],
                 fig6dat['lag_enc'][:]-fig6dat['lag2cat_enc'][:],
                 fig6dat['cat_rec'][:]-fig6dat['cat2lag_rec'][:],
                 fig6dat['lag_rec'][:]-fig6dat['lag2cat_rec'][:]])[:, 0]
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
ax1a = plt.subplot(gs[0, 0])
# positions = [1, 2, 3.5, 4.5] #, 6.5, 7.5, 9, 10]
positions = [1.5, 4]
parts = ax1a.violinplot(dataset=np.transpose(aucs)[:, :2], showextrema=False,
                        positions=positions, widths=1)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax1a.errorbar(x=positions, y=aucs_means[:2], yerr=aucs_CI95s[:2], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax1a.plot([0.5, 5.5], [0, 0], 'k--')
ax1a.set_xlim((0.5, 5.5))
# ax1a.set_ylim((-0.32, 0.32))
ax1a.set_ylim((-0.67, 0.67))
ax1a.set_yticks(np.arange(-0.6, 0.61, 0.1))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
ax1a.set_ylabel(r'$\Delta$ AUC (decoding $-$ cross-decoding)', labelpad=0)
ax1a.set_xticks(positions)
# ax1a.set_xticklabels(['categorized', 'unrelated'])
ax1a.set_xticklabels(['', ''])
ax1a.set_xlabel('Encoding', x=0.45)
ax1a.tick_params(axis='x', direction='in')
# sns.despine(offset=0, top=True, right=True, left=False, bottom=True, ax=ax1a, trim=True)
#sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1a, trim=True)
sns.despine(offset=0, top=True, right=True, left=False, bottom=False, ax=ax1a, trim=True)
# ax1a.yaxis.set_tick_params(pad=3)
legend_handles = [
    mpatches.Patch(
        facecolor=decodecolor[0], edgecolor='black', alpha=0.5, linewidth=2),
    mpatches.Patch(
        facecolor=xdecodecolor[0], edgecolor='black', alpha=0.5, linewidth=2)]
legend_labels = ['categorical', 'temporal']
ax1a.legend(legend_handles, legend_labels, loc='upper left',
            frameon=False, borderpad=0)#, borderaxespad=0.3)

ax2a = plt.subplot(gs[0, 1])
parts = ax2a.violinplot(dataset=np.transpose(aucs)[:, 2:], showextrema=False,
                        positions=positions, widths=1)
for i, pc in enumerate(parts['bodies']):
    pc.set_edgecolor('black')
    pc.set_alpha(0.5)
    pc.set_linewidth(2)
    if i%2 == 0:
        pc.set_facecolor(decodecolor)
        pc.set_label('decoding')
    else:
        pc.set_facecolor(xdecodecolor)
        pc.set_label('cross-decoding')
ax2a.errorbar(x=positions, y=aucs_means[2:], yerr=aucs_CI95s[2:], marker='o', ecolor='black',
              elinewidth=2, mfc='white', mec='black', linestyle='', ms=2)
ax2a.plot([0, 5], [0, 0], 'k--')
ax2a.set_xlim((0, 5))
# ax2a.set_ylim((0.36, 1.0))
#ax2a.set_ylim((-0.32, 0.32))
ax2a.set_ylim((-0.67, 0.67))
# ax1.set_yticks(np.arange(0, 0.7, 0.2))
# ax1.set_ylim((0., 0.65))
# ax1.set_ylabel('p(recall)')#, size='large')
# ax2a.xaxis.set_visible(False)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# ax2a.yaxis.set_tick_params(pad=3)
# ax1.legend(loc='upper left',
#            frameon=False, borderpad=0)

# positions = [1.5, 4, 7, 9.5]
positions = [1.5, 4]
aucs = fig4cdat['data']
aucs_means = np.mean(aucs, 1)
aucs_SEs = np.std(aucs, 1, ddof=1) / np.sqrt(aucs.shape[1])
aucs_CI95s = aucs_SEs * dists.t.ppf(0.975, aucs.shape[1]-1)
# sns.despine(offset=0, top=True, right=True, left=True, bottom=True, ax=ax2a)
# sns.despine(offset=0, ax=ax2b)
ax2a.yaxis.set_visible(False)
ax2a.xaxis.set_tick_params(pad=3)
#ax2b.yaxis.set_tick_params(pad=3)
ax2a.set_xticks(positions)
# ax2a.set_xticklabels(['categorized', 'unrelated'])
ax2a.set_xticklabels(['', ''])
ax2a.tick_params(axis='x', direction='in')
ax2a.set_xlabel('Retrieval', x=0.55)
sns.despine(offset=0, top=True, right=True, left=True, bottom=False, ax=ax2a, trim=True)

fig.subplots_adjust(left=0.17, bottom=0.08, right=0.999, top=0.99,
                    wspace=0, hspace=0.1)
plt.savefig('./figures/pub/FRcatFR_lagcat_decoding_xdecoding_delta.pdf')


