import sys
import numpy as np
import os
import h5py
import multiprocessing as mp
from glob import glob
# from sklearn import cross_validation
from sklearn.model_selection import LeaveOneGroupOut
import cPickle as pickle


# if > 0 use multiprocessing, otherwise regular single-process
# execution:

run_on_rhino = True
# run_on_rhino = False
# experiment = 'FR1'
# xdecoding = 'lag2cat'
xdecoding = 'cat2lag'
encret = 'encoding'  # don't change -- retrieval in different file!

if xdecoding == 'lag2cat':
    exp_from = 'FR1'
    exp_to = 'catFR1'
elif xdecoding == 'cat2lag':
    exp_from = 'catFR1'
    exp_to = 'FR1'
else:
    raise ValueError('Wrong xdecoding: '+ xdecoding)

GOOD_ELECS_ONLY = False
# GOOD_ELECS_ONLY = True

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
else:
    num_mp_procs = 0  # 12
    rhino_mount = '/home/ctw/fusemounts/rhino'


if GOOD_ELECS_ONLY:
    class_path_suffix = '_goodchans'
else:
    class_path_suffix = ''

paths = {
    'power':{
        'retrieval': {
            'catFR1': (rhino_mount +
                       '/scratch/cweidema/RAM/RAM_catFR/' +
                       'RAM_catFR1_power/retrieval/hdf5_files_subj/'),
            'FR1': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FR/' +
                    'RAM_FR1_power/retrieval/hdf5_files_subj/')},
        'encoding': {
            'catFR1': (rhino_mount +
                       '/scratch/cweidema/RAM/RAM_catFR/' +
                       'RAM_catFR1_power/encoding/hdf5_files_subj/'),
            'FR1': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FR/' +
                    'RAM_FR1_power/encoding/hdf5_files_subj/')}},
    'classifier': {
        'lag2cat': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/' + encret +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_'+encret+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenLag2Catrecall_'+encret+'_withshuffles/'),
        'cat2lag': (rhino_mount +
                    '/scratch/cweidema/RAM/RAM_FRcatFR/' + encret +
                    class_path_suffix+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_'+encret+'/'+
                    'RAM_FRcatFR1_classifiers_allfreqs_withinvsbetweenCat2Lagrecall_'+encret+'_withshuffles/')
    }
}




power_files_all = {}
phases = ['encoding', 'retrieval']
for exp in paths['power']['retrieval']:
    power_files_all[exp] = {
        phase: np.sort(glob(os.path.join(
            paths['power'][phase][exp], '*_pow.hdf5'))) for phase in phases}

power_files_bothexp = {}
for exp in power_files_all:
    power_files_bothexp[exp] = {phase: [] for phase in phases}

# for pf_enc, pf_ret in zip(power_files_all['catFR1']['encoding'],
#                           power_files_all['catFR1']['retrieval']):
#     pf_FR_enc = pf_enc.replace('catFR', 'FR')
#     pf_FR_ret = pf_ret.replace('catFR', 'FR')
#     if ((pf_FR_enc in power_files_all['FR1']['encoding']) and
#         (pf_FR_ret in power_files_all['FR1']['retrieval'])):
#         power_files_bothexp['catFR1']['encoding'].append(pf_enc)
#         power_files_bothexp['FR1']['encoding'].append(pf_FR_enc)
#         power_files_bothexp['catFR1']['retrieval'].append(pf_ret)
#         power_files_bothexp['FR1']['retrieval'].append(pf_FR_ret)

for pf_enc in power_files_all['catFR1']['encoding']:
    pf_ret = pf_enc.replace('encoding', 'retrieval')
    if pf_ret not in power_files_all['catFR1']['retrieval']:
        continue
    pf_FR_enc = pf_enc.replace('catFR', 'FR')
    if pf_FR_enc not in power_files_all['FR1']['encoding']:
        continue
    pf_FR_ret = pf_FR_enc.replace('encoding', 'retrieval')
    if pf_FR_ret not in power_files_all['FR1']['retrieval']:
        continue
    power_files_bothexp['catFR1']['encoding'].append(pf_enc)
    power_files_bothexp['FR1']['encoding'].append(pf_FR_enc)
    power_files_bothexp['catFR1']['retrieval'].append(pf_ret)
    power_files_bothexp['FR1']['retrieval'].append(pf_FR_ret)

# for pf_ret in power_files_all['catFR1']['retrieval']:
#     pf_FR_ret = pf_ret.replace('catFR', 'FR')
#     if pf_FR_ret in power_files_all['FR1']['retrieval']:
#         power_files_bothexp['catFR1']['retrieval'].append(pf_ret)
#         power_files_bothexp['FR1']['retrieval'].append(pf_FR_ret)



        
classifier_path = paths['classifier'][xdecoding]
subjpaths = {'from': power_files_bothexp[exp_from][encret],
             'to': power_files_bothexp[exp_to][encret]}






time_bins = [(0.0, 1.6)]
cs = [np.logspace(np.log10(1e-6), np.log10(1e4), 22)[6]]

lr_params = {'penalty': 'l2', 'dual': False, 'tol': 0.0001, 'C': np.nan,
             'fit_intercept': True, 'intercept_scaling': 1,
             'class_weight': 'balanced', 'random_state': None,
             'solver': 'liblinear', 'max_iter': 2000,
             'multi_class': 'ovr', 'verbose': False}


def get_lags(subjpath):
    subjdat_enc = h5py.File(subjpath, 'r')
    subjdat_ret = h5py.File(subjpath.replace('encoding', 'retrieval'), 'r')
    # rec_lag = []
    # recwordfilt = subjdat_ret['events/type'][:]=='REC_WORD'
    # recwordfilt = subjdat_ret['events/rec_base_match'][:] == 1
    # lags = np.empty(subjdat_ret['events/rec_base_match'].shape)
    lags = np.empty(subjdat_enc['events/rec_base_match'].shape)
    lags.fill(np.nan)
    valid_filt = ((subjdat_ret['events/type'][:] == 'REC_WORD') &
                  (subjdat_ret['events/intrusion'][:] == 0) &
                  (subjdat_ret['events/repeat'][:] == 0) &
                  (subjdat_ret['events/output_position'][:] != 0))
    # for valindx in np.where(subjdat_ret['events/rec_base_match'])[0]:
    for valindx in np.where(valid_filt)[0]:
        item1 = subjdat_ret['events/item'][valindx-1]
        item2 = subjdat_ret['events/item'][valindx]
        sessfilt = subjdat_enc['events/session'][:]==subjdat_ret[
            'events/session'][valindx]
        lstfilt = subjdat_enc['events/list'][:]==subjdat_ret[
            'events/list'][valindx]
        spos1 = subjdat_enc['events/serialpos'][
            (subjdat_enc['events/item'][:] == item1) & sessfilt & lstfilt]
        item2_indx = ((subjdat_enc['events/item'][:] == item2) &
                      sessfilt & lstfilt)
        spos2 = subjdat_enc['events/serialpos'][item2_indx]
        # if len(spos1) != 0: # if the prior event was not a valid recal
        if (len(spos1) == 1) and (len(spos2) == 1): # if the prior event was not a valid recal
                            # len(spos1) might be 0
            # if np.all(np.isfinite(spos2)) and np.all(np.isfinite(spos1)):
            lags[item2_indx] = np.int(spos2)-np.int(spos1)
    return lags


# def train_test_lr(lr_params, features, target, train, test, indx=-1):
#     from sklearn.linear_model import LogisticRegression
#     lr = LogisticRegression(**lr_params).fit(
#         features[train], target[train])
#     classifier_info = {
#         'trained_classifier': lr,
#         'train': train,
#         'test': test,
#         'decision_function': lr.decision_function(features[test]),
#         'predict': lr.predict(features[test]),
#         'log_probas': lr.predict_log_proba(features[test]),
#         'probas': lr.predict_proba(features[test]),
#         'score': lr.score(features[test], target[test]),
#         'params': lr.get_params(),
#         'coef': lr.coef_,
#         'intercept': lr.intercept_,
#         'indx': indx}
#     return(classifier_info)


def train_test_lr_xdec(lr_params, features1, target1, features2, target2):
    from sklearn.linear_model import LogisticRegression
    lr = LogisticRegression(**lr_params).fit(
        features1, target1)
    classifier_info = {
        'trained_classifier': lr,
        'decision_function': lr.decision_function(features2),
        'predict': lr.predict(features2),
        'log_probas': lr.predict_log_proba(features2),
        'probas': lr.predict_proba(features2),
        'score': lr.score(features2, target2),
        'params': lr.get_params(),
        'coef': lr.coef_,
        'intercept': lr.intercept_}
    return(classifier_info)


# def train_test_lr_shuffles(lr_params, features, target, train, test,
#                            shuffles=1000, indx=-1):
#     from sklearn.linear_model import LogisticRegression
#     shuffled_classifiers = []
#     shuffled_classifier_info = []
#     for i in range(shuffles):
#         trgt = np.copy(target[train])
#         np.random.shuffle(trgt)
#         lr = LogisticRegression(**lr_params).fit(
#             features[train], trgt)
#         classifier_info = {
#             'trained_classifier': lr,
#             'train': train,
#             'test': test,
#             'decision_function': lr.decision_function(features[test]),
#             'predict': lr.predict(features[test]),
#             'log_probas': lr.predict_log_proba(features[test]),
#             'probas': lr.predict_proba(features[test]),
#             'score': lr.score(features[test], target[test]),
#             'params': lr.get_params(),
#             'coef': lr.coef_,
#             'intercept': lr.intercept_,
#             'train_target': trgt,
#             'indx': indx}
#         shuffled_classifiers.append(lr)
#         shuffled_classifier_info.append(classifier_info)
#     return(shuffled_classifiers, shuffled_classifier_info)


def train_test_lr_xdec_shuffles(lr_params, features1, target1,
                                features2, target2, shuffles=1000):
    from sklearn.linear_model import LogisticRegression
    shuffled_classifiers = []
    shuffled_classifier_info = []
    for i in range(shuffles):
        trgt1 = np.copy(target1)
        np.random.shuffle(trgt1)
        lr = LogisticRegression(**lr_params).fit(
            features1, trgt1)
        classifier_info = {
            'trained_classifier': lr,
            'decision_function': lr.decision_function(features2),
            'predict': lr.predict(features2),
            'log_probas': lr.predict_log_proba(features2),
            'probas': lr.predict_proba(features2),
            'score': lr.score(features2, target2),
            'params': lr.get_params(),
            'coef': lr.coef_,
            'intercept': lr.intercept_,
            'train_target': trgt1}
        shuffled_classifiers.append(lr)
        shuffled_classifier_info.append(classifier_info)
    return(shuffled_classifiers, shuffled_classifier_info)


# def get_valchans(subjdat):
#     subj = subjdat['data'].attrs['subject']
#     elfile = glob(rhino_mount+'/data/eeg/'+subj+
#                   '*/docs/electrode_categories.txt')
#     if len(elfile) != 1:
#         raise ValueError('Invalid elfile for subj '+subj+': '+str(elfile))
#     elfile = elfile[0]
#     with open(elfile, 'r') as ef:
#         lines = [mystr.strip().replace(' ', '').upper()
#                  for mystr in ef.readlines()
#                  if len(mystr.strip().replace(' ', ''))> 0]
#     if 'tal_struct' in subjdat.keys():
#         bppairs = [bp.strip().replace(' ', '').upper().split('-')
#                    for bp in subjdat['tal_struct/code']]
#     elif 'h5info' in subjdat.keys():
#         bppairs = [bp.strip().replace(' ', '').upper().split('-')
#                    for bp in subjdat['h5info/bipolar_info/contact_name']]
#     else:
#         raise ValueError('Missing electrode info for '+str(subjdat))
#     badchans = np.array([np.any([e in lines for e in bp])
#                          for bp in bppairs])
#     return(~badchans)

def get_valchans(subjdat):
    subj = subjdat['data'].attrs['subject']
    elfile = glob(rhino_mount+'/data/eeg/'+subj+
                  '*/docs/*electrode_categories.txt')
    if len(elfile) == 0:
        elfile = glob(rhino_mount+'/scratch/pwanda/electrode_categories/'+
                      subj+'*_electrode_categories.txt')
    if len(elfile) == 0:
        elfile = glob(rhino_mount+'/scratch/pwanda/electrode_categories/' +
                      'electrode_categories_' + subj + '*.txt')
    if len(elfile) == 2:
        subj_with_duplicates = ['R1171M', 'R1191J', 'R1290M', 'R1288P',
                                'R1230J']
        if subj in subj_with_duplicates:
            elfile.pop(0)
    if len(elfile) != 1:
        raise ValueError('Invalid elfile for subj '+subj+': '+str(elfile))
    elfile = elfile[0]
    with open(elfile, 'r') as ef:
        lines = [mystr.strip().replace(' ', '').upper()
                 for mystr in ef.readlines()
                 if len(mystr.strip().replace(' ', ''))> 0]
    if 'tal_struct' in subjdat.keys():
        bppairs = [bp.strip().replace(' ', '').upper().split('-')
                   for bp in subjdat['tal_struct/code']]
    elif 'h5info' in subjdat.keys():
        bppairs = [bp.strip().replace(' ', '').upper().split('-')
                   for bp in subjdat['h5info/bipolar_info/contact_name']]
    else:
        raise ValueError('Missing electrode info for '+str(subjdat))
    badchans = np.array([np.any([e in lines for e in bp])
                         for bp in bppairs])
    return(~badchans)


def get_features(data, targs, time_bins=time_bins, sessfilt=None,
                 freqsfilt=None, channels=None,
                 valid_chan_filter=None):
    if valid_chan_filter is None:
        valid_chan_filter = ~np.all(~np.isfinite(data[0, :, :, 0]), -1)
    else:
        valid_chan_filter &= ~np.all(~np.isfinite(data[0, :, :, 0]), -1)
    # data_valid = data[valid_chan_filter, :, :, :]
    # valid_evs_filter = (np.isfinite(np.min(data[valid_chan_filter, 0, :, 0], 0)) &
    #                     np.isfinite(data.dims[2]['within_cat'][:]))
    valid_evs_filter = (np.isfinite(np.min(data[0, valid_chan_filter, :, 0], 0)) &
                        np.isfinite(targs))
   # data_valid = data_valid[:, :, valid_evs_filter]
    if freqsfilt is None:
        freqsfilt = np.ones(data.shape[0], np.bool)
    if sessfilt is not None:
        valid_evs_filter &= sessfilt   
    timebin_features = np.sum(valid_chan_filter) * np.sum(freqsfilt)
    #                     len(data.dims[1]['freqs']))
    sessdat = data[:, :, valid_evs_filter]
    sessdat = sessdat[:, valid_chan_filter]
    sessdat = sessdat[freqsfilt]
    features = {}
    features['data'] = np.ones((np.sum(valid_evs_filter),
                                timebin_features * len(time_bins))) * np.nan
    features['time_bin_labels'] = []
    features['channels'] = []
    features['freqs'] = []
    if channels is None:
        channels = data.dims[0][0]
    for t, tb in enumerate(time_bins):
        time_filter = ((data.dims[3]['time'].value >= tb[0]) &
                       (data.dims[3]['time'].value < tb[1]))
        features['data'][
            :, (t * timebin_features):((t + 1) * timebin_features)] = np.mean(
            sessdat[:, :, :, time_filter], 3).reshape(
                (np.sum(valid_chan_filter) *
                 np.sum(freqsfilt),
                 np.sum(valid_evs_filter))).T
        features['time_bin_labels'].extend(
            [str(tb[0]) + '-' + str(tb[1])] * timebin_features)
        for chan in channels[valid_chan_filter]:
            features['channels'].extend(
                # [chan] * (len(data.dims[1]['freqs'])))
                [chan] * (np.sum(freqsfilt)))
            features['freqs'].extend(data.dims[1][0][freqsfilt])
    return(features, valid_chan_filter, valid_evs_filter)


def run_classification_xdec(subjpath1, subjpath2, train_test_func, train_test_func_shuffles,
                       classifier_params,
                       classifier_path, time_bins, shuffles=1000):
    classifier_path_pickle = classifier_path + 'pickle_files/'
    classifier_path_hdf5 = classifier_path + 'hdf5_files/'
    if not os.path.exists(classifier_path_pickle +
                          str(classifier_params['C']) + '/'):
        try:
            os.makedirs(classifier_path_pickle+
                        str(classifier_params['C']) + '/')
        except OSError, e:
            print '*** no worries ***', e
    if not os.path.exists(classifier_path_hdf5 +
                          str(classifier_params['C']) + '/'):
        try:
            os.makedirs(classifier_path_hdf5 +
                        str(classifier_params['C']) + '/')
        except OSError, e:
            print '*** no worries ***', e
    subjdat1 = h5py.File(subjpath1, 'r')
    subjdat2 = h5py.File(subjpath2, 'r')
    # # make sure we observe each recall transition on at least two lists:
    # if ((len(np.unique(subjdat['events/list'][subjdat[
    #         'events/within_cat'][:] == 1])) < 2) |
    #     (len(np.unique(subjdat['events/list'][subjdat[
    #         'events/between_cat'][:]== 1])) < 2)):
    #     print 'At least one recall transition type is not present in multiple lists for', subjpath
    #     return
    sess_list_labels1 = np.array(['{:02d}'.format(s)+'-'+'{:02d}'.format(l)
                                 for s,l in zip(
                                         subjdat1['data'].dims[2]['session'][:],
                                         subjdat1['data'].dims[2]['list'][:])])
    sess_list_labels2 = np.array(['{:02d}'.format(s)+'-'+'{:02d}'.format(l)
                                 for s,l in zip(
                                         subjdat2['data'].dims[2]['session'][:],
                                         subjdat2['data'].dims[2]['list'][:])])
    sessfilt1 = (np.array([np.sum((subjdat1['data'].dims[2]['session'][:] == s) &
                                  (subjdat1['data'].dims[2]['list'][:] == l))
                         for s,l in zip(subjdat1['data'].dims[2]['session'][:],
                                        subjdat1['data'].dims[2]['list'][:])])
                == 12)
    sessfilt2 = (np.array([np.sum((subjdat2['data'].dims[2]['session'][:] == s) &
                                  (subjdat2['data'].dims[2]['list'][:] == l))
                         for s,l in zip(subjdat2['data'].dims[2]['session'][:],
                                        subjdat2['data'].dims[2]['list'][:])])
                == 12)
    # remove practice lists:
    sessfilt1 &= subjdat1['data'].dims[2]['session'][:] >= 0
    sessfilt2 &= subjdat2['data'].dims[2]['session'][:] >= 0
    
    if exp_from == 'FR1':
        targs1 = get_lags(subjpath1)
        targs1[np.abs(targs1)==1] = 1.0
        targs1[np.abs(targs1)>1] = 0.0
    elif exp_from == 'catFR1':
        if 'within_cat' not in subjdat1['data'].dims[2]:
            print('No within_cat field: '+subjpath1)
            return
        targs1 = np.empty(subjdat1['data'].dims[2]['within_cat'].shape)
        targs1.fill(np.nan)
        targs1[subjdat1['data'].dims[2]['within_cat'][:] == 1] = 1
        targs1[subjdat1['data'].dims[2]['between_cat'][:] == 1] = 0
    else:
        raise ValueError('Wrong exp_from: ' + exp_from)
    if exp_to == 'FR1':
        targs2 = get_lags(subjpath2)
        targs2[np.abs(targs2)==1] = 1.0
        targs2[np.abs(targs2)>1] = 0.0
    elif exp_to == 'catFR1':
        if 'within_cat' not in subjdat2['data'].dims[2]:
            print('No within_cat field: '+subjpath2)
            return
        # targs2 = subjdat2['data'].dims[2]['within_cat'][:] == 1
        targs2 = np.empty(subjdat2['data'].dims[2]['within_cat'].shape)
        targs2.fill(np.nan)
        targs2[subjdat2['data'].dims[2]['within_cat'][:] == 1] = 1
        targs2[subjdat2['data'].dims[2]['between_cat'][:] == 1] = 0
    else:
        raise ValueError('Wrong exp_to: ' + exp_from)
        
    # make sure we observe each recall transition on at least two lists:
    if ((len(np.unique(sess_list_labels1[targs1 == 1])) < 2) or
        (len(np.unique(sess_list_labels1[targs1 == 0])) < 2) or
        (len(np.unique(sess_list_labels2[targs2 == 1])) < 2) or
        (len(np.unique(sess_list_labels2[targs2 == 0])) < 2)):
        print 'At least one recall transition type is not present in multiple lists for', subjpath1
        return

    if 'session' in subjdat1['data'].attrs:
        session = subjdat1['data'].attrs['session']
        out_path_pickle = (
            classifier_path_pickle + str(classifier_params['C']) +
            '/' + subjdat1['data'].attrs['subject'] +
            '_' + str(subjdat1['data'].attrs['session']) +
            '_classifier_info.pickle')
        out_path_hdf5 = (classifier_path_hdf5 + str(classifier_params['C']) +
                         '/' + subjdat1['data'].attrs['subject'] +
                         '_' + str(subjdat1['data'].attrs['session']) +
                         '_classifier_info.hdf5')
    else:
        out_path_pickle = (
            classifier_path_pickle + str(classifier_params['C']) +
            '/' + subjdat1['data'].attrs['subject'] + 
            '_classifier_info.pickle')
        out_path_hdf5 = (
            classifier_path_hdf5 + str(classifier_params['C']) + '/' +
            subjdat1['data'].attrs['subject'] + 
            '_classifier_info.hdf5')
    if os.path.exists(out_path_pickle) or os.path.exists(out_path_hdf5):
        return
    try:
        out_hdf5 = h5py.File(out_path_hdf5, 'w-', libver='latest')
    except IOError:
        print 'Cannot create', out_path_hdf5
        return
    # logo = LeaveOneGroupOut()
    # withincat_transitions = subjdat['data'].dims[2]['within_cat'][sessfilt] == 1
    res = {}
    res_shuffles = {}
    means1 = {}
    stds1 = {}
    means2 = {}
    stds2 = {}
    if GOOD_ELECS_ONLY:
        valid_chan_filter1 = get_valchans(subjdat1)
        valid_chan_filter2 = get_valchans(subjdat2)
    else:
        valid_chan_filter1 = None
        valid_chan_filter2 = None
    assert(np.all(valid_chan_filter1==valid_chan_filter2))
    chans1 = None
    chans2 = None
    if 'h5info' in subjdat1.keys():
        if 'bipolar_info' in subjdat1['h5info'].keys():
            chans1 = subjdat1['h5info/bipolar_info/contact_name'][:]
    elif 'tal_struct' in subjdat1.keys():
        # if 'code' in subjdat['tal_struct'].keys():           
        chans1 = subjdat1['tal_struct/code'][:]
    if 'h5info' in subjdat2.keys():
        if 'bipolar_info' in subjdat2['h5info'].keys():
            chans2 = subjdat2['h5info/bipolar_info/contact_name'][:]
    elif 'tal_struct' in subjdat2.keys():
        # if 'code' in subjdat['tal_struct'].keys():           
        chans2 = subjdat2['tal_struct/code'][:]
    assert(np.all(chans1==chans2))
    for tb in time_bins:
        means1[tb] = []
        stds1[tb] = []
        means2[tb] = []
        stds2[tb] = []
        res[tb] = []
        res_shuffles[tb] = []
        features1, valid_chan_filter1, valid_evs_filter1 = get_features(
            subjdat1['data'], targs1, time_bins=[tb], sessfilt=sessfilt1,
            # freqsfilt=subjdat['freqs'][:] > 9)
            freqsfilt=subjdat1['freqs'][:] > 0, channels=chans1)
        features2, valid_chan_filter2, valid_evs_filter2 = get_features(
            subjdat2['data'], targs2, time_bins=[tb], sessfilt=sessfilt2,
            # freqsfilt=subjdat['freqs'][:] > 9)
            freqsfilt=subjdat2['freqs'][:] > 0, channels=chans2)
        # for study_list, (train, test) in enumerate(
        #         logo.split(features['data'], lags[valid_evs_filter],
        #                    groups=sess_list_labels[valid_evs_filter])):
        #     print study_list, '/', logo.get_n_splits(
        #         features['data'], lags[valid_evs_filter],
        #         groups=sess_list_labels[valid_evs_filter])
        features_norm1 = features1['data'].copy()
        features_norm2 = features2['data'].copy()
        means1[tb].append([])
        stds1[tb].append([])
        means2[tb].append([])
        stds2[tb].append([])
        for sess in np.unique(
                subjdat1['events/session'][valid_evs_filter1]):
            scalesessfilt = subjdat1['events/session'][
                valid_evs_filter1] == sess
            sessmeans = features_norm1[scalesessfilt].mean(0)
            sessstds = features_norm1[scalesessfilt].std(0)
            means1[tb][-1].append(sessmeans)
            stds1[tb][-1].append(sessstds)
            features_norm1[scalesessfilt] -= sessmeans
            features_norm1[scalesessfilt] /= sessstds
        for sess in np.unique(
                subjdat2['events/session'][valid_evs_filter2]):
            scalesessfilt = subjdat2['events/session'][
                valid_evs_filter2] == sess
            sessmeans = features_norm2[scalesessfilt].mean(0)
            sessstds = features_norm2[scalesessfilt].std(0)
            means2[tb][-1].append(sessmeans)
            stds2[tb][-1].append(sessstds)
            features_norm2[scalesessfilt] -= sessmeans
            features_norm2[scalesessfilt] /= sessstds
        res[tb].append(
            train_test_func(
                classifier_params, features_norm1,
                targs1[valid_evs_filter1], features_norm2,
                targs2[valid_evs_filter2]))
        res_shuffles[tb].append(
            train_test_func_shuffles(
                classifier_params, features_norm1,
                targs1[valid_evs_filter1], features_norm2,
                targs2[valid_evs_filter2], shuffles))
    try:
        pickle.dump([res, res_shuffles], open(out_path_pickle, 'w'), -1)
        # make file read only to avoid accidental loss:
        os.chmod(out_path_pickle, 0o444)
    except IOError:
        print 'Cannot create ', out_path_pickle
    features1_out = out_hdf5.create_dataset(
        'features1', data=features1['data'],
        compression='gzip', compression_opts=9)
    features2_out = out_hdf5.create_dataset(
        'features2', data=features2['data'],
        compression='gzip', compression_opts=9)
    features1_out.dims[0].label = 'events'
    features2_out.dims[0].label = 'events'
    out_hdf5.copy(subjdat1['events'], out_hdf5, name='events1')
    out_hdf5.copy(subjdat2['events'], out_hdf5, name='events2')
    for key in out_hdf5['events1'].keys():
        features1_out.dims.create_scale(out_hdf5['events1/' + key], key)
        features1_out.dims[0].attach_scale(out_hdf5['events1/' + key])
    for key in out_hdf5['events2'].keys():
        features2_out.dims.create_scale(out_hdf5['events2/' + key], key)
        features2_out.dims[0].attach_scale(out_hdf5['events2/' + key])
    # create dimension for features:
    if 'tal_struct' in subjdat1.keys():
        out_hdf5.copy(subjdat1['tal_struct'], out_hdf5, name='tal_struct1')
    if 'tal_struct' in subjdat2.keys():
        out_hdf5.copy(subjdat2['tal_struct'], out_hdf5, name='tal_struct2')
    if 'h5info' in subjdat1.keys():
        out_hdf5.copy(subjdat1['h5info'], out_hdf5, name='h5info1')
    if 'h5info' in subjdat2.keys():
        out_hdf5.copy(subjdat2['h5info'], out_hdf5, name='h5info2')
    targs1 = out_hdf5.create_dataset(
        'target1', data=targs1, compression="gzip", compression_opts=9)
    targs2 = out_hdf5.create_dataset(
        'target2', data=targs2, compression="gzip", compression_opts=9)
    out_hdf5['channels1'] = features1['channels']
    out_hdf5['valid_chan_filter1'] = valid_chan_filter1
    out_hdf5['valid_evs_filter1'] = valid_evs_filter1
    out_hdf5['freqs1'] = features1['freqs']
    features1_out.dims[1].label = 'time_bins1*channels1*frequencies1'
    features1_out.dims.create_scale(out_hdf5['channels1'], 'channels1')
    features1_out.dims[1].attach_scale(out_hdf5['channels1'])
    features1_out.dims.create_scale(out_hdf5['freqs1'], 'freqs1')
    features1_out.dims[1].attach_scale(out_hdf5['freqs1'])
    #
    out_hdf5['channels2'] = features2['channels']
    out_hdf5['valid_chan_filter2'] = valid_chan_filter2
    out_hdf5['valid_evs_filter2'] = valid_evs_filter2
    out_hdf5['freqs2'] = features2['freqs']
    features2_out.dims[1].label = 'time_bins2*channels2*frequencies2'
    features2_out.dims.create_scale(out_hdf5['channels2'], 'channels2')
    features2_out.dims[1].attach_scale(out_hdf5['channels2'])
    features2_out.dims.create_scale(out_hdf5['freqs2'], 'freqs2')
    features2_out.dims[1].attach_scale(out_hdf5['freqs2'])
    #
    out_hdf5['time_bins'] = time_bins
    for param in classifier_params:
        if param == 'random_state':
            features1_out.attrs[param] = str(classifier_params[param])
            features2_out.attrs[param] = str(classifier_params[param])
        else:
            features1_out.attrs[param] = classifier_params[param]
            features2_out.attrs[param] = classifier_params[param]
    for c_info in res[time_bins[0]][0].keys():
        # print c_info
        if c_info == 'trained_classifier':
            continue
        elif c_info == 'params':
            for param in res[time_bins[0]][0][c_info]:
                if param == 'random_state':
                    features1_out.attrs[param] = str(
                        res[time_bins[0]][0][c_info][param])
                    features2_out.attrs[param] = str(
                        res[time_bins[0]][0][c_info][param])
                else:
                    features1_out.attrs[param] = res[
                        time_bins[0]][0][c_info][param]
                    features2_out.attrs[param] = res[
                        time_bins[0]][0][c_info][param]
            continue
        try:
            out_hdf5.create_dataset(
                c_info, data = [[r[c_info] for r in res[tb]] for tb in time_bins],
                compression="gzip", compression_opts=9)
            out_hdf5.create_dataset(
                c_info + '_shuffles',
                data = [[[r[c_info] for r in rtb[1]] for rtb in res_shuffles[tb]]
                        for tb in time_bins],
                compression="gzip", compression_opts=9)
        except TypeError:
            tmp = []
            for tb in time_bins:
                tmp.append(res[tb][0][c_info])
                for r in res[tb][1:]:
                    tmp[-1] = np.r_[tmp[-1], r[c_info]]  
            out_hdf5.create_dataset(
                c_info, data = tmp,
                compression="gzip", compression_opts=9)
            tmp = []
            for tb in time_bins:
                tmp.append([])
                for rtb in res_shuffles[tb]:
                    tmp[-1].append([])
                    for r in rtb[1]:
                        tmp[-1][-1].append(r[c_info])
            tmp2 = np.array([tmp[0][0] for tb in time_bins])
            if len(tmp2.shape) == 4:
                transpose_tuple = (0, 2, 1, 3)
            else:
                transpose_tuple = (0, 2, 1)
            tmp2 = tmp2.transpose(transpose_tuple)
            for i in range(1, len(tmp[0])):
                tmp2 = np.concatenate(
                    (tmp2, np.transpose([tmp[0][i]
                                         for tb in time_bins], transpose_tuple)), 1)
            out_hdf5.create_dataset(
                c_info + '_shuffles',
                data = tmp2,
                compression="gzip", compression_opts=9)
    tmp = []
    for tb in time_bins:
        out_hdf5.create_dataset('means1/'+str(tb), data=means1[tb],
                                compression="gzip", compression_opts=9)
        out_hdf5.create_dataset('stds1/'+str(tb), data=stds1[tb],
                                compression="gzip", compression_opts=9)
        out_hdf5.create_dataset('means2/'+str(tb), data=means2[tb],
                                compression="gzip", compression_opts=9)
        out_hdf5.create_dataset('stds2/'+str(tb), data=stds2[tb],
                                compression="gzip", compression_opts=9)
        tmp.append([])
        for rtb in res_shuffles[tb]:
            tmp[-1].append([])
            for r in rtb[1]:
                tmp[-1][-1].append(r['train_target'])
    tmp2 = np.transpose([tmp[0][0] for tb in time_bins], (0, 2, 1))
    for i in range(1, len(tmp[0])):
        tmp2 = np.concatenate(
            (tmp2, np.transpose([tmp[0][i]
                                 for tb in time_bins], (0, 2, 1))), 1)
    out_hdf5.create_dataset(
        'train_targs_shuffles',
        data = tmp2,
        compression="gzip", compression_opts=9)
    out_hdf5.close()
    # make file read only to avoid accidental loss:
    os.chmod(out_path_hdf5, 0o444)

if num_mp_procs > 0:
    po = mp.Pool(num_mp_procs)
    results = []


# for subjpath in subjpaths_enc:
for sp_from, sp_to in zip(subjpaths['from'], subjpaths['to']):
    for c in cs:
        print(c, sp_from, sp_to)
        lr_params['C'] = c
        if num_mp_procs > 0:
            results.append(po.apply_async(
                run_classification_xdec,
                [sp_from, sp_to, train_test_lr_xdec, train_test_lr_xdec_shuffles,
                 lr_params, classifier_path, time_bins]))
        else:
            run_classification_xdec(sp_from, sp_to, train_test_lr_xdec,
                                    train_test_lr_xdec_shuffles,
                                    lr_params, classifier_path, time_bins)


if num_mp_procs > 0:
    po.close()
    # track results
    for i,r in enumerate(results):
        sys.stdout.write('%d '%i)
        sys.stdout.flush()
        r.get()

