import sys
import numpy as np
import os
import h5py
import multiprocessing as mp
from glob import glob
import cPickle as pickle

# for sp in subjpaths['catFR']:
#     subjpath_FR = sp.replace('catFR', 'FR')
#     if not os.path.exists(subjpath_FR):
#         continue
#     subjdat1 = h5py.File(sp, 'r')
#     subjdat2 = h5py.File(sp.replace('catFR', 'FR'), 'r')
#     print subjdat1['data'].attrs['subject'], np.unique(subjdat1['events']['session']), np.unique(subjdat2['events']['session'])


# if > 0 use multiprocessing, otherwise regular single-process
# execution:

run_on_rhino = True
# run_on_rhino = False

buffer_type = 'normal_buffer'
# classification_type = 'FRtoWithinCatFR'
classification_type = 'FRtoBetweenCatFR'

# GOOD_ELECS_ONLY = False
GOOD_ELECS_ONLY = True

if run_on_rhino:
    num_mp_procs = 0
    rhino_mount = ''
else:
    num_mp_procs = 0  # 12
    rhino_mount = '/home/ctw/fusemounts/rhino'

if GOOD_ELECS_ONLY:
    class_path_suffix = '_goodchans'
else:
    class_path_suffix = ''

paths = {
    'normal_buffer': {
        'pow_path_catFR': (rhino_mount +
                           '/scratch/cweidema/RAM/RAM_catFR/' +
                           'RAM_catFR1_power/retrieval/hdf5_files_subj/'),
        'pow_path_FR': (rhino_mount +
                        '/scratch/cweidema/RAM/RAM_FR/' +
                        'RAM_FR1_power/retrieval/hdf5_files_subj/'),
        'classifier_path': {
            'FRtoWithinCatFR': (rhino_mount +
                                '/scratch/cweidema/RAM/RAM_FRcatFR/' +
                                'retrieval' + class_path_suffix +
                                '/RAM_FR1toWithinCatFR1_classifiers/' +
                                'RAM_FR1toWithinCatFR1_classifiers_withshuffles/'),
            'FRtoBetweenCatFR': (rhino_mount +
                                 '/scratch/cweidema/RAM/RAM_FRcatFR/' +
                                 'retrieval' + class_path_suffix +
                                 '/RAM_FR1toBetweenCatFR1_classifiers/' +
                                 'RAM_FR1toBetweenCatFR1_classifiers_withshuffles/')}
    }
}

# pow_suffix = '.hdf5'

# subjpaths_catFR = np.sort(glob(os.path.join(pow_path_catFR, '*')))
# subjpaths_FR = np.sort(glob(os.path.join(pow_path_catFR, '*')))

if classification_type == 'FRtoWithinCatFR':
    rec_cond = 'within'
elif classification_type == 'FRtoBetweenCatFR':
    rec_cond = 'between'


# subjpaths = {
#     'FR': np.sort(glob(os.path.join(
#         paths[buffer_type]['pow_path_FR'], '*'))),
#     'catFR': np.sort(glob(os.path.join(
#         paths[buffer_type]['pow_path_catFR'], '*')))}

subjpaths = {'from': [], 'to': []}
catFR_paths = np.sort(glob(os.path.join(
        paths[buffer_type]['pow_path_catFR'], '*')))
for catFR_path in catFR_paths:
    fr_path = catFR_path.replace('catFR', 'FR')
    if os.path.exists(fr_path):
        subjpaths['from'].append(fr_path)
        subjpaths['to'].append(catFR_path)

classifier_path_pickle = paths[buffer_type]['classifier_path'][
    classification_type] + 'pickle_files/'
classifier_path_hdf5 = paths[buffer_type]['classifier_path'][
    classification_type] + 'hdf5_files/'

# time_bins = [(0.0,0.2),(0.2,0.4),(0.4,0.6),(0.6,0.8),
#              (0.8,1.0),(1.0,1.2),(1.2,1.4),(1.4,1.6)]
time_bins = [(-0.9, -0.1)]

# c = np.logspace(np.log10(1e-6),np.log10(1e4),22)[6]
cs = [np.logspace(np.log10(1e-6), np.log10(1e4), 22)[6]]
# cs = np.logspace(np.log10(1e-6), np.log10(1e4), 22)

lr_params = {'penalty': 'l2', 'dual': False, 'tol': 0.0001, 'C': np.nan,
             'fit_intercept': True, 'intercept_scaling': 1,
             'class_weight': 'balanced', 'random_state': None,
             'solver': 'liblinear', 'max_iter': 2000,
             'multi_class': 'ovr', 'verbose': False}
# lr_params = {'penalty': 'l2', 'dual': False, 'tol': 0.0001, 'C': np.nan,
#              'fit_intercept': True, 'intercept_scaling': 1,
#              'class_weight': 'auto', 'random_state': None,
#              'solver': 'liblinear', 'max_iter': 2000,
#              'multi_class': 'ovr', 'verbose': 1}
# svc_params = {'C':np.nan, 'kernel':'rbf', 'degree':3, 'gamma': 0.0,
#               'coef0': 0.0, 'shrinking': True, 'probability': False,
#               'tol': 0.001, 'cache_size': 200, 'class_weight': 'auto',
#               'verbose': False, 'max_iter': -1, 'random_state': None}


def train_test_lr(lr_params, features, target, train, test, indx=-1):
    from sklearn.linear_model import LogisticRegression
    lr = LogisticRegression(**lr_params).fit(
        features[train], target[train])
    classifier_info = {
        'trained_classifier': lr,
        'train': train,
        'test': test,
        'decision_function': lr.decision_function(features[test]),
        'predict': lr.predict(features[test]),
        'log_probas': lr.predict_log_proba(features[test]),
        'probas': lr.predict_proba(features[test]),
        'score': lr.score(features[test], target[test]),
        'params': lr.get_params(),
        'coef': lr.coef_,
        'intercept': lr.intercept_,
        'indx': indx}
    return(classifier_info)


def train_test_lr_acrossExps(lr_params, features1,
                             target1, features2, target2):
    from sklearn.linear_model import LogisticRegression
    lr = LogisticRegression(**lr_params).fit(
        features1, target1)
    classifier_info = {
        'trained_classifier': lr,
        'decision_function': lr.decision_function(features2),
        'predict': lr.predict(features2),
        'log_probas': lr.predict_log_proba(features2),
        'probas': lr.predict_proba(features2),
        'score': [lr.score(features2, target2)],
        'params': lr.get_params(),
        'coef': lr.coef_,
        'intercept': lr.intercept_}
    return(classifier_info)


def train_test_lr_acrossExps_shuffles(lr_params, features1, target1,
                                      features2, target2, shuffles=1000):
    from sklearn.linear_model import LogisticRegression
    shuffled_classifiers = []
    shuffled_classifier_info = []
    for i in range(shuffles):
        # moved the below line into the loop after generating the
        # classifier files because train_targs_shuffles in the
        # classifier files did not reflect the target for each shuffle:
        trg1 = np.copy(target1)
        np.random.shuffle(trg1)
        lr = LogisticRegression(**lr_params).fit(
            features1, trg1)
        classifier_info = {
            'trained_classifier': lr,
            'decision_function': lr.decision_function(features2),
            'predict': lr.predict(features2),
            'log_probas': lr.predict_log_proba(features2),
            'probas': lr.predict_proba(features2),
            'score': [lr.score(features2, target2)],
            'params': lr.get_params(),
            'coef': lr.coef_,
            'intercept': lr.intercept_,
            'train_target': trg1}
        shuffled_classifiers.append(lr)
        shuffled_classifier_info.append(classifier_info)
    return(shuffled_classifiers, shuffled_classifier_info)


def train_test_svc(svc_params, features, target, train, test, indx=-1):
    from sklearn.svm import SVC
    svc = SVC(**svc_params).fit(
        features[train], target[train])
    classifier_info = {
        'trained_classifier': svc,
        'train': train,
        'test': test,
        'support': svc.support_,
        'support_vectors': svc.support_vectors_,
        'n_support': svc.n_support_,
        'dual_coef': svc.dual_coef_,
        # 'coef': svc.coef_, # only for linear kernel
        'intercept': svc.intercept_,
        'decision_function': svc.decision_function(features[test]),
        'predict': svc.predict(features[test]),
        'score': svc.score(features[test], target[test]),
        'params': svc.get_params(),
        'indx': indx}
    return(classifier_info)

# def get_valchans(subjdat):
#     subj = subjdat['data'].attrs['subject']
#     elfile = glob(rhino_mount+'/data/eeg/'+subj+
#                   '*/docs/electrode_categories.txt')
#     if len(elfile) != 1:
#         raise ValueError('Invalid elfile for subj '+subj+': '+str(elfile))
#     elfile = elfile[0]
#     with open(elfile, 'r') as ef:
#         lines = [mystr.strip().replace(' ', '').upper()
#                  for mystr in ef.readlines()
#                  if len(mystr.strip().replace(' ', ''))> 0]
#     if 'tal_struct' in subjdat.keys():
#         bppairs = [bp.strip().replace(' ', '').upper().split('-')
#                    for bp in subjdat['tal_struct/code']]
#     elif 'h5info' in subjdat.keys():
#         bppairs = [bp.strip().replace(' ', '').upper().split('-')
#                    for bp in subjdat['h5info/bipolar_info/contact_name']]
#     else:
#         raise ValueError('Missing electrode info for '+str(subjdat))
#     badchans = np.array([np.any([e in lines for e in bp])
#                          for bp in bppairs])
#     return(~badchans)

def get_valchans(subjdat):
    subj = subjdat['data'].attrs['subject']
    elfile = glob(rhino_mount+'/data/eeg/'+subj+
                  '*/docs/*electrode_categories.txt')
    if len(elfile) == 0:
        elfile = glob(rhino_mount+'/scratch/pwanda/electrode_categories/'+
                      subj+'*_electrode_categories.txt')
    if len(elfile) == 0:
        elfile = glob(rhino_mount+'/scratch/pwanda/electrode_categories/' +
                      'electrode_categories_' + subj + '*.txt')
    if len(elfile) == 2:
        subj_with_duplicates = ['R1171M', 'R1191J', 'R1290M', 'R1288P',
                                'R1230J']
        if subj in subj_with_duplicates:
            elfile.pop(0)
    if len(elfile) != 1:
        raise ValueError('Invalid elfile for subj '+subj+': '+str(elfile))
    elfile = elfile[0]
    with open(elfile, 'r') as ef:
        lines = [mystr.strip().replace(' ', '').upper()
                 for mystr in ef.readlines()
                 if len(mystr.strip().replace(' ', ''))> 0]
    if 'tal_struct' in subjdat.keys():
        bppairs = [bp.strip().replace(' ', '').upper().split('-')
                   for bp in subjdat['tal_struct/code']]
    elif 'h5info' in subjdat.keys():
        bppairs = [bp.strip().replace(' ', '').upper().split('-')
                   for bp in subjdat['h5info/bipolar_info/contact_name']]
    else:
        raise ValueError('Missing electrode info for '+str(subjdat))
    badchans = np.array([np.any([e in lines for e in bp])
                         for bp in bppairs])
    return(~badchans)


def get_features(data, time_bins=time_bins, rec_cond='all',
                 sessfilt=None, channels=None,
                 valid_chan_filter=None):
    if valid_chan_filter is None:
        valid_chan_filter = ~np.all(~np.isfinite(data[0, :, :, 0]), -1)
    else:
        valid_chan_filter &= ~np.all(~np.isfinite(data[0, :, :, 0]), -1)
    data_valid = data[:, valid_chan_filter, :, :]
    valid_evs_filter = np.isfinite(np.min(
        data[0, valid_chan_filter, :, 0], 0))
    # only include REC_BASE and matched REC_WORD events:
    valid_evs_filter = valid_evs_filter & (
        (data.dims[2]['type'][:] == 'REC_BASE') |
        (data.dims[2]['rec_base_match'][:] == 1))
    # # make sure to delete complete lists:
    # invalid_sess_list = [(s, l) for s, l in zip(
    #     data.dims[2]['session'][~valid_evs_filter],
    #     data.dims[2]['list'][~valid_evs_filter])]
    # # it's a bit inefficient to loop through all of the elements of
    # # the list but creating a list of unique (session, list) tuples is
    # # a bit tricky in a way that works generally is a bit tricky
    # # (tried with np.unique and set) and in generally the
    # # invalid_sess_list list should be short:
    # for inval_sess, inval_list in invalid_sess_list:
    #     valid_evs_filter[(data.dims[2]['session'][:] == inval_sess) &
    #                      (data.dims[2]['list'][:] == inval_list)] = False
    if rec_cond == 'within':
        valid_evs_filter = valid_evs_filter & (
            (data.dims[2]['type'][:] == 'REC_BASE') |
            (data.dims[2]['within_cat'][:] == 1))
    elif rec_cond == 'between':
        valid_evs_filter = valid_evs_filter & (
            (data.dims[2]['type'][:] == 'REC_BASE') |
            (data.dims[2]['between_cat'][:] == 1))
    if sessfilt is not None:
        valid_evs_filter &= sessfilt   
    data_valid = data_valid[:, :, valid_evs_filter]
    timebin_features = (np.sum(valid_chan_filter) *
                        len(data.dims[1]['freqs']))
    # timebin_features = (np.sum(valid_chan_filter) *
    #                     len(data.dims[1]['freqs']))
    features = {}
    features['data'] = np.ones((np.sum(valid_evs_filter),
                                timebin_features * len(time_bins))) * np.nan
    # features['data'] = np.ones((np.sum(valid_evs_filter),
    #                             timebin_features * len(time_bins))) * np.nan
    features['time_bin_labels'] = []
    features['channels'] = []
    features['freqs'] = []
    if channels is None:
        channels = data.dims[0][0]
    for t, tb in enumerate(time_bins):
        time_filter = ((data.dims[3]['time'].value >= tb[0]) &
                       (data.dims[3]['time'].value < tb[1]))
        # features['data'][
        #     :, (t * timebin_features):((t + 1) * timebin_features)] = np.mean(
        #     data[:, :, :, time_filter], 3).reshape(
        #         (len(data.dims[0]['channels']) *
        #          len(data.dims[1]['freqs']),
        #          len(data.dims[2]['recalled']))).T
        features['data'][
            :, (t * timebin_features):((t + 1) * timebin_features)] = np.mean(
            data_valid[:, :, :, time_filter], 3).reshape(
                (np.sum(valid_chan_filter) *
                 len(data.dims[1]['freqs']),
                 np.sum(valid_evs_filter))).T
        features['time_bin_labels'].extend(
            [str(tb[0]) + '-' + str(tb[1])] * timebin_features)
        # for chan in data.dims[0][0].value:
        for chan in channels[valid_chan_filter]:
            features['channels'].extend(
                [chan] * (len(data.dims[1]['freqs'])))
            features['freqs'].extend(data.dims[1][0].value)
    return(features, valid_chan_filter, valid_evs_filter)


def run_classification_acrossExps(subjpath1, subjpath2, train_test_func,
                                  train_test_func_shuffles, classifier_params,
                                  rec_cond):
    if not os.path.exists(classifier_path_pickle +
                          str(classifier_params['C']) + '/'):
        try:
            os.makedirs(classifier_path_pickle +
                        str(classifier_params['C']) + '/')
        except OSError, e:
            print '*** no worries ***', e
    if not os.path.exists(classifier_path_hdf5 +
                          str(classifier_params['C']) + '/'):
        try:
            os.makedirs(classifier_path_hdf5 +
                        str(classifier_params['C']) + '/')
        except OSError, e:
            print '*** no worries ***', e
    subjdat1 = h5py.File(subjpath1, 'r')
    subjdat2 = h5py.File(subjpath2, 'r')
    # remove practice lists:
    sessfilt1 = subjdat1['data'].dims[2]['session'][:] >= 0
    sessfilt2 = subjdat2['data'].dims[2]['session'][:] >= 0
    if 'session' in subjdat1['data'].attrs:
        out_path_pickle = (
            classifier_path_pickle + str(classifier_params['C']) +
            '/' + subjdat1['data'].attrs['subject'] +
            '_' + str(subjdat1['data'].attrs['session']) +
            '_classifier_info.pickle')
        out_path_hdf5 = (classifier_path_hdf5 + str(classifier_params['C']) +
                         '/' + subjdat1['data'].attrs['subject'] +
                         '_' + str(subjdat1['data'].attrs['session']) +
                         '_classifier_info.hdf5')
    else:
        out_path_pickle = (
            classifier_path_pickle + str(classifier_params['C']) +
            '/' + subjdat1['data'].attrs['subject'] +
            '_classifier_info.pickle')
        out_path_hdf5 = (
            classifier_path_hdf5 + str(classifier_params['C']) + '/' +
            subjdat1['data'].attrs['subject'] +
            '_classifier_info.hdf5')
    if os.path.exists(out_path_pickle) or os.path.exists(out_path_hdf5):
        return
    try:
        out_hdf5 = h5py.File(out_path_hdf5, 'w-', libver='latest')
    except IOError:
        print 'Cannot create', out_path_hdf5
        return
    chans1 = None
    chans2 = None
    if 'h5info' in subjdat1.keys():
        if 'bipolar_info' in subjdat1['h5info'].keys():
            chans1 = subjdat1['h5info/bipolar_info/contact_name'][:]
    elif 'tal_struct' in subjdat1.keys():
        # if 'code' in subjdat['tal_struct'].keys():           
        chans = subjdat1['tal_struct/code'][:]
    if 'h5info' in subjdat2.keys():
        if 'bipolar_info' in subjdat2['h5info'].keys():
            chans2 = subjdat2['h5info/bipolar_info/contact_name'][:]
    elif 'tal_struct' in subjdat2.keys():
        # if 'code' in subjdat['tal_struct'].keys():           
        chans = subjdat2['tal_struct/code'][:]
    assert(np.all(chans1==chans2))
    if GOOD_ELECS_ONLY:
        valid_chan_filter1 = get_valchans(subjdat1)
        valid_chan_filter2 = get_valchans(subjdat2)
    else:
        valid_chan_filter1 = None
        valid_chan_filter2 = None
    features1, valid_chan_filter1, valid_evs_filter1 = get_features(
        subjdat1['data'], rec_cond='all', channels=chans1,
        sessfilt=sessfilt1, valid_chan_filter=valid_chan_filter1)
    # recalled1 = subjdat1['data'].dims[2]['recalled'].value == 1
    recalled1 = subjdat1['data'].dims[2]['type'][
        valid_evs_filter1] == 'REC_WORD'
    if (np.sum(recalled1) < 1) | (
            np.sum(subjdat1['data'].dims[2]['type'][
                valid_evs_filter1] == 'REC_BASE') < 1):
        print 'At least one recall and one recall_base event are required; requirement not met for', subjpath1
        return
    features2, valid_chan_filter2, valid_evs_filter2 = get_features(
        subjdat2['data'], rec_cond=rec_cond, channels=chans2,
        sessfilt=sessfilt2, valid_chan_filter=valid_chan_filter2)
    # recalled2 = subjdat2['data'].dims[2]['recalled'].value == 1
    recalled2 = subjdat2['data'].dims[2]['type'][
        valid_evs_filter2] == 'REC_WORD'
    if (np.sum(recalled2) < 1) | (
            np.sum(subjdat2['data'].dims[2]['type'][
                valid_evs_filter2] == 'REC_BASE') < 1):
        print 'At least one recall and one recall_base event are required; requirement not met for', subjpath1
        return
    # withincat = subjdat2['data'].dims[2]['within_cat'][valid_evs_filter2] == 1
    # betweencat = subjdat2['data'].dims[2]['between_cat'][valid_evs_filter2] == 1
    # sess_list_labels = ['{:02d}'.format(s)+'-'+'{:02d}'.format(l)
    #                     for s,l in zip(
    #                             subjdat['data'].dims[2]['session'].value,
    #                             subjdat['data'].dims[2]['list'].value)]
    # lolo = cross_validation.LeaveOneLabelOut(
    #     sess_list_labels)
    features_norm1 = features1['data'].copy()
    features_norm2 = features2['data'].copy()
    means1 = []
    stds1 = []
    means2 = []
    stds2 = []
    for sess in np.unique(
            subjdat1['events/session'][valid_evs_filter1]):
        scalesessfilt = subjdat1['events/session'][valid_evs_filter1] == sess
        sessmeans = features_norm1[scalesessfilt].mean(0)
        sessstds = features_norm1[scalesessfilt].std(0)
        means1.append(sessmeans)
        stds1.append(sessstds)
        features_norm1[scalesessfilt] -= sessmeans
        features_norm1[scalesessfilt] /= sessstds
    for sess in np.unique(
            subjdat2['events/session'][valid_evs_filter2]):
        scalesessfilt = subjdat2['events/session'][valid_evs_filter2] == sess
        sessmeans = features_norm2[scalesessfilt].mean(0)
        sessstds = features_norm2[scalesessfilt].std(0)
        means2.append(sessmeans)
        stds2.append(sessstds)
        features_norm2[scalesessfilt] -= sessmeans
        features_norm2[scalesessfilt] /= sessstds
    res = train_test_func(classifier_params, features_norm1, recalled1,
                          features_norm2, recalled2)
    shuffled_classifiers, shuffled_classifier_info = train_test_func_shuffles(
        classifier_params, features_norm1, recalled1,
        features_norm2, recalled2)
    try:
        pickle.dump([res, shuffled_classifiers, shuffled_classifier_info],
                    open(out_path_pickle, 'w-'), -1)
        # make file read only to avoid accidental loss:
        os.chmod(out_path_pickle, 0o444)
    except IOError:
        print 'Cannot create ', out_path_pickle
    means1 = out_hdf5.create_dataset(
        'means1', data=means1,
        compression="gzip", compression_opts=9)
    means2 = out_hdf5.create_dataset(
        'means2', data=means1,
        compression="gzip", compression_opts=9)
    std1 = out_hdf5.create_dataset(
        'std1', data=means1,
        compression="gzip", compression_opts=9)
    std2 = out_hdf5.create_dataset(
        'std2', data=means1,
        compression="gzip", compression_opts=9)
    features1_out = out_hdf5.create_dataset(
        'features1', data=features1['data'],
        compression="gzip", compression_opts=9)
    features2_out = out_hdf5.create_dataset(
        'features2', data=features2['data'],
        compression="gzip", compression_opts=9)
    features1_out.dims[0].label = 'events1'
    out_hdf5.copy(subjdat1['events'], out_hdf5, name='events1')
    for key in out_hdf5['events1'].keys():
        features1_out.dims.create_scale(out_hdf5['events1/' + key], key)
        features1_out.dims[0].attach_scale(out_hdf5['events1/' + key])
    features2_out.dims[0].label = 'events2'
    out_hdf5.copy(subjdat2['events'], out_hdf5, name='events2')
    for key in out_hdf5['events2'].keys():
        features2_out.dims.create_scale(out_hdf5['events2/' + key], key)
        features2_out.dims[0].attach_scale(out_hdf5['events2/' + key])
    # create dimension for features:
    if 'tal_struct' in subjdat1.keys():
        out_hdf5.copy(subjdat1['tal_struct'], out_hdf5, name='tal_struct1')
    if 'tal_struct' in subjdat2.keys():
        out_hdf5.copy(subjdat2['tal_struct'], out_hdf5, name='tal_struct2')
    if 'h5info' in subjdat1.keys():
        out_hdf5.copy(subjdat1['h5info'], out_hdf5, name='h5info1')
    if 'h5info' in subjdat2.keys():
        out_hdf5.copy(subjdat2['h5info'], out_hdf5, name='h5info2')
    out_hdf5['channels1'] = features1['channels']
    out_hdf5['valid_chan_filter1'] = valid_chan_filter1
    out_hdf5['valid_evs_filter1'] = valid_evs_filter1    
    out_hdf5['freqs1'] = features1['freqs']
    out_hdf5['time_bins1'] = features1['time_bin_labels']
    features1_out.dims[1].label = 'time_bins1*channels1*frequencies1'
    features1_out.dims.create_scale(out_hdf5['channels1'], 'channels1')
    features1_out.dims[1].attach_scale(out_hdf5['channels1'])
    features1_out.dims.create_scale(out_hdf5['freqs1'], 'freqs1')
    features1_out.dims[1].attach_scale(out_hdf5['freqs1'])
    features1_out.dims.create_scale(out_hdf5['time_bins1'], 'time_bins1')
    features1_out.dims[1].attach_scale(out_hdf5['time_bins1'])
    #
    out_hdf5['channels2'] = features2['channels']
    out_hdf5['valid_chan_filter2'] = valid_chan_filter2
    out_hdf5['valid_evs_filter2'] = valid_evs_filter2  
    out_hdf5['freqs2'] = features2['freqs']
    out_hdf5['time_bins2'] = features2['time_bin_labels']
    features2_out.dims[1].label = 'time_bins2*channels2*frequencies2'
    features2_out.dims.create_scale(out_hdf5['channels2'], 'channels2')
    features2_out.dims[1].attach_scale(out_hdf5['channels2'])
    features2_out.dims.create_scale(out_hdf5['freqs2'], 'freqs2')
    features2_out.dims[1].attach_scale(out_hdf5['freqs2'])
    features2_out.dims.create_scale(out_hdf5['time_bins2'], 'time_bins2')
    features2_out.dims[1].attach_scale(out_hdf5['time_bins2'])
    #
    for param in classifier_params:
        if param == 'random_state':
            features1_out.attrs[param] = str(classifier_params[param])
            features2_out.attrs[param] = str(classifier_params[param])
        else:
            features1_out.attrs[param] = classifier_params[param]
            features2_out.attrs[param] = classifier_params[param]
    recalled1 = out_hdf5.create_dataset(
        'target1', data=recalled1, compression="gzip", compression_opts=9)
    recalled2 = out_hdf5.create_dataset(
        'target2', data=recalled2, compression="gzip", compression_opts=9)
    for c_info in res:
        if c_info == 'trained_classifier':
            continue
        elif c_info == 'params':
            for param in res[c_info]:
                if param == 'random_state':
                    features1_out.attrs[param] = str(res[c_info][param])
                    features2_out.attrs[param] = str(res[c_info][param])
                else:
                    features1_out.attrs[param] = res[c_info][param]
                    features2_out.attrs[param] = res[c_info][param]
            continue
        # try:
        out_hdf5.create_dataset(
            c_info, data=res[c_info],
            compression="gzip", compression_opts=9)
        out_hdf5.create_dataset(
            c_info + '_shuffles', data=[
                sci[c_info] for sci in shuffled_classifier_info],
            compression="gzip", compression_opts=9)
    out_hdf5.create_dataset(
        'train_targs_shuffles', data=[
            sci['train_target'] for sci in shuffled_classifier_info],
        compression="gzip", compression_opts=9)
    out_hdf5.close()
    # make file read only to avoid accidental loss:
    os.chmod(out_path_hdf5, 0o444)

if num_mp_procs > 0:
    po = mp.Pool(num_mp_procs)
    results = []

for sp_from, sp_to in zip(subjpaths['from'], subjpaths['to']):
    for c in cs:
        print(c, sp_from)
        lr_params['C'] = c
        # svc_params['C'] = c
        if num_mp_procs > 0:
            # results.append(po.apply_async(
            #     run_classification_acrossExps, [
            #         subjpath_FR, train_test_lr, lr_params]))
            results.append(po.apply_async(
                run_classification_acrossExps, [
                    sp_from, sp_to, train_test_lr_acrossExps,
                    train_test_lr_acrossExps_shuffles, lr_params, rec_cond]))
            # results.append(po.apply_async(
            #     run_classification_acrossExps, [
            #         subjpath_catFR, subjpath_FR, train_test_lr_acrossExps,
            #         train_test_lr_acrossExps_shuffles, lr_params]))
        else:
            # run_classification(subjpath, train_test_lr, lr_params)
            run_classification_acrossExps(
                sp_from, sp_to, train_test_lr_acrossExps,
                train_test_lr_acrossExps_shuffles, lr_params, rec_cond)
            # run_classification_acrossExps(
            #     subjpath_catFR, subjpath_FR, train_test_lr_acrossExps,
            #     train_test_lr_acrossExps_shuffles, lr_params)

if num_mp_procs > 0:
    po.close()
    # track results
    for i, r in enumerate(results):
        sys.stdout.write('%d ' % i)
        sys.stdout.flush()
        r.get()
