function [st_fr, st_catfr, h_fr, h_catfr, n_fr, n_catfr ] = ...
    catfr_fr_univ(data_dir, fr_power_dir, enc_flag)
%PREP_UNIV run univariate stats for encoding and retrieval success
subs = get_catfr_mscript_subs();

% subject x region x frequency
t_stat = nan(length(subs), 8, 8);

for  k = 1:length(subs)
    
    power = h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/data');
    session =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/session');
    
    if enc_flag
        recalled =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/recalled');
    else
        intrusion =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/intrusion');
        repeat =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/repeat');
        pirt =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/pirt');
        within_cat =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/within_cat');
        between_cat =  h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/between_cat');
        type = h5read([fr_power_dir '/' subs{k} '_pow.hdf5'],'/events/type');
        
        type = deblank(type);
    end
    
    % for retrieval
    if ~enc_flag % we want to do recall
        recalled = nan(size(power,2),1);
        recalled(strcmp(type,'REC_BASE')) = 0;
        recalled(intrusion==0 & repeat == 0 & pirt > 1500 & strcmp(type,'REC_WORD')) = 1;
    end
    
    power(:,isnan(recalled),:,:) = [];
    session(isnan(recalled)) = [];
    recalled(isnan(recalled)) = [];
    
    if length(recalled)<2
        continue
    end
    
    % average over time and zscore
    zpow = zscore_power(power,session);
    
    anatRegion_snap = h5read([fr_power_dir '/' subs{k} '_pow.hdf5'], ...
        '/tal_info/indivSurf/anatRegion_snap');
    locTag = h5read([fr_power_dir '/' subs{k} '_pow.hdf5'], ...
        '/tal_info/locTag');
    
    anatRegion_snap = deblank(anatRegion_snap);
    
    n_fr(k,1) = sum(recalled==1);
    n_fr(k,2) = sum(recalled==0);
    
    hc_flag = is_hc(locTag);
    phg_flag = is_phg(locTag);
    mtl_flag = hc_flag | phg_flag;
    
    ifg_flag = is_ifg(anatRegion_snap);
    mfg_flag = is_mfg(anatRegion_snap);
    sfg_flag = is_sfg(anatRegion_snap);
    
    tc_flag = is_tc(anatRegion_snap);
    ipc_flag = is_ipc(anatRegion_snap);
    spc_flag = is_spc(anatRegion_snap);
    oc_flag = is_oc(anatRegion_snap);
    
    % convert get average power per ROI
    
    subj_roi_mean= nan(size(zpow,1),8,8);
    
    if any(ifg_flag)
        subj_roi_mean(:,1,:) = squeeze(nanmean(zpow(:,:,ifg_flag),3));
    end
    if any(mfg_flag)
        subj_roi_mean(:,2,:) = squeeze(nanmean(zpow(:,:,mfg_flag),3));
    end
    if any(sfg_flag)
        subj_roi_mean(:,3,:) = squeeze(nanmean(zpow(:,:,sfg_flag),3));
    end
    
    if any(mtl_flag)
        subj_roi_mean(:,4,:) = squeeze(nanmean(zpow(:,:,mtl_flag),3));
    end
    if any(tc_flag)
        subj_roi_mean(:,5,:) = squeeze(nanmean(zpow(:,:,tc_flag),3));
    end
    
    if any(ipc_flag)
        subj_roi_mean(:,6,:) = squeeze(nanmean(zpow(:,:,ipc_flag),3));
    end
    if any(spc_flag)
        subj_roi_mean(:,7,:) = squeeze(nanmean(zpow(:,:,spc_flag),3));
    end
    if any(oc_flag)
        subj_roi_mean(:,8,:) = squeeze(nanmean(zpow(:,:,oc_flag),3));
    end
    
    t_stat(k,:,:) = get_t(subj_roi_mean, recalled);
end

fr_tstat = t_stat;
[~,p,~,st_fr] = ttest(t_stat);
pid = fdr(p(:), .05);

if isempty(pid)
    pid = 0;
end

h_fr = p<pid;


for  k = 1:length(subs)
    
    power = h5read([data_dir '/' subs{k} '_pow.hdf5'],'/data');
    
    recalled =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/recalled');
    session =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/session');
    
    if enc_flag
        recalled =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/recalled');
    else
        cintrusion =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/intrusion');
        crepeat =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/repeat');
        cpirt =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/pirt');
        within_cat =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/within_cat');
        between_cat =  h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/between_cat');
        ctype = h5read([data_dir '/' subs{k} '_pow.hdf5'],'/events/type');
        
        ctype = deblank(ctype);
    end
    
    % for retrieval
    if ~enc_flag % we want to do recall
        recalled = nan(size(power,2),1);
        recalled(strcmp(ctype,'REC_BASE')) = 0;
        recalled(cintrusion==0 & crepeat == 0 & cpirt > 1500 & strcmp(ctype,'REC_WORD')) = 1;
    end
    
    power(:,isnan(recalled),:,:) = [];
    session(isnan(recalled)) = [];
    recalled(isnan(recalled)) = [];
    
    if length(recalled)<2
        continue
    end
    
    % average over time and zscore
    zpow = zscore_power(power,session);
    
    anatRegion_snap = h5read([data_dir '/' subs{k} '_pow.hdf5'], ...
        '/tal_info/indivSurf/anatRegion_snap');
    locTag = h5read([data_dir '/' subs{k} '_pow.hdf5'], ...
        '/tal_info/locTag');
    
    anatRegion_snap = deblank(anatRegion_snap);
    
    n_catfr(k,1) = sum(recalled==1);
    n_catfr(k,2) = sum(recalled==0);
    
    hc_flag = is_hc(locTag);
    phg_flag = is_phg(locTag);
    mtl_flag = hc_flag | phg_flag;
    
    ifg_flag = is_ifg(anatRegion_snap);
    mfg_flag = is_mfg(anatRegion_snap);
    sfg_flag = is_sfg(anatRegion_snap);
    
    tc_flag = is_tc(anatRegion_snap);
    ipc_flag = is_ipc(anatRegion_snap);
    spc_flag = is_spc(anatRegion_snap);
    oc_flag = is_oc(anatRegion_snap);
    
    % convert get average power per ROI
    
    subj_roi_mean= nan(size(zpow,1),8,8);
    if any(ifg_flag)
        subj_roi_mean(:,1,:) = squeeze(nanmean(zpow(:,:,ifg_flag),3));
    end
    if any(mfg_flag)
        subj_roi_mean(:,2,:) = squeeze(nanmean(zpow(:,:,mfg_flag),3));
    end
    if any(sfg_flag)
        subj_roi_mean(:,3,:) = squeeze(nanmean(zpow(:,:,sfg_flag),3));
    end
    
    if any(mtl_flag)
        subj_roi_mean(:,4,:) = squeeze(nanmean(zpow(:,:,mtl_flag),3));
    end
    if any(tc_flag)
        subj_roi_mean(:,5,:) = squeeze(nanmean(zpow(:,:,tc_flag),3));
    end
    
    if any(ipc_flag)
        subj_roi_mean(:,6,:) = squeeze(nanmean(zpow(:,:,ipc_flag),3));
    end
    if any(spc_flag)
        subj_roi_mean(:,7,:) = squeeze(nanmean(zpow(:,:,spc_flag),3));
    end
    if any(oc_flag)
        subj_roi_mean(:,8,:) = squeeze(nanmean(zpow(:,:,oc_flag),3));
    end
    
    try
        t_stat(k,:,:) = get_t(subj_roi_mean, recalled);
    end
end

catfr_tstat = t_stat;
[~,p,~,st_catfr] = ttest(t_stat);
pid = fdr(p(:),.05);
if isempty(pid)
    pid = 0;
end

h_catfr = p<pid;

% and the difference between fr and catfr

% the below is not significant, so we do not output it
[~,p,~,st] = ttest(fr_tstat,catfr_tstat);
pid = fdr(p(:),.05);

if isempty(pid)
    pid = 0;
end

end

function t = get_t(power, recalled)

[~,~,~,st] = ttest2(power(recalled==1,:,:), power(recalled==0,:,:));
t = st.tstat;

end

function power = zscore_power(power,session)

% power is time x event x freq x elec

% average over time
power = squeeze(nanmean(power));

us = unique(session);

for s = 1:length(us)
    
    sess_mean = nanmean(power(session == us(s),:,:));
    sess_std = nanstd(power(session == us(s),:,:));
    
    power(session == us(s),:,:) = bsxfun(@minus, power(session == us(s),:,:), sess_mean);
    power(session == us(s),:,:) = bsxfun(@rdivide, power(session == us(s),:,:), sess_std);
    
end

end

function hc_flag = is_hc(locTag)

hc_rois = {'Left CA1','Left CA2','Left CA3','Left DG','Left Sub',...
    'Right CA1','Right CA2','Right CA3','Right DG','Right Sub'};


hc_flag = ismember(locTag,hc_rois);

end

function phg_flag = is_phg(locTag)


phg_rois = {'Left PRC','Right PRC','Right EC','Right PHC','Left EC','Left PHC'};

phg_flag = ismember(locTag,phg_rois);


end

function ifg_flag = is_ifg(anatRegion)

ifg_rois = {'parsopercularis','parsorbitalis','parstriangularis'};

ifg_flag = ismember(anatRegion,ifg_rois);

end

function mfg_flag = is_mfg(anatRegion)

mfg_rois = {'caudalmiddlefrontal','rostralmiddlefrontal'};

mfg_flag = ismember(anatRegion,mfg_rois);

end

function sfg_flag = is_sfg(anatRegion)

sfg_rois = {'superiorfrontal'};

sfg_flag = ismember(anatRegion,sfg_rois);

end

function tc_flag = is_tc(anatRegion)

tc_rois = {'middletemporal','inferiortemporal'};

tc_flag = ismember(anatRegion,tc_rois);

end

function ipc_flag = is_ipc(anatRegion)

ipc_rois = {'inferiorparietal','supramarginal'};

ipc_flag = ismember(anatRegion,ipc_rois);

end

function spc_flag = is_spc(anatRegion)

spc_rois = {'superiorparietal','precuneus'};

spc_flag = ismember(anatRegion,spc_rois);

end

function oc_flag = is_oc(anatRegion)

oc_rois = {'lateraloccipital','lingual','cuneus','pericalcarine'};

oc_flag = ismember(anatRegion,oc_rois);

end
