import seaborn as sns
import matplotlib.pyplot as plt
import h5py

def cm2inch(*tupl):
    inch = 2.54
    if isinstance(tupl[0], tuple):
        return tuple(i/inch for i in tupl[0])
    else:
        return tuple(i/inch for i in tupl)

sns.set_context("paper", rc={"font.size":8,"axes.titlesize":8,"axes.labelsize":5})

rois = ['IFG','MFG','SFG','MTL','TC','IPC','SPC','OC']
freqs = ['3','5','10','17','31','56','100','180']

f, ((ax1, ax2), (ax3, ax4)) = plt.subplots(2, 2, sharex='col', sharey='row', figsize=cm2inch(13.6, 8))

f.subplots_adjust(right=0.7)
cbar_ax = f.add_axes([0.75, 0.15, 0.025, 0.7])

fr_enc = h5py.File("data/fr_enc.hdf5","r")
t = fr_enc['/data/tstat'][:]
h = fr_enc['/data/h'][:]
fr_enc.close()

sns.heatmap(t*h, ax=ax1, cbar=False, vmin=-5.5, vmax=5.5, xticklabels=rois, yticklabels=freqs)
ax1.invert_yaxis()
ax1.set_xticklabels(ax1.get_xticklabels(), rotation=90)
ax1.set_yticklabels(ax1.get_yticklabels(), rotation=0)

fr_rec = h5py.File("data/fr_rec.hdf5","r")
t = fr_rec['/data/tstat'][:]
h = fr_rec['/data/h'][:]
fr_rec.close()

sns.heatmap(t*h, ax=ax2, cbar=False, vmin=-5.5, vmax=5.5, xticklabels=rois, yticklabels=freqs)
ax2.invert_yaxis()
ax2.set_xticklabels(ax2.get_xticklabels(), rotation=90)
ax2.set_yticklabels(ax2.get_yticklabels(), rotation=0)


catfr_enc = h5py.File("data/catfr_enc.hdf5","r")
t = catfr_enc['/data/tstat'][:]
h = catfr_enc['/data/h'][:]
catfr_enc.close()

sns.heatmap(t*h, ax=ax3, cbar=False, vmin=-5.5, vmax=5.5, xticklabels=rois, yticklabels=freqs)
ax3.invert_yaxis()
ax3.set_xticklabels(ax3.get_xticklabels(), rotation=90)
ax3.set_yticklabels(ax3.get_yticklabels(), rotation=0)

catfr_rec = h5py.File("data/catfr_rec.hdf5","r")
t = catfr_rec['/data/tstat'][:]
h = catfr_rec['/data/h'][:]
catfr_rec.close()

sns.heatmap(t*h, ax=ax4, cbar_ax=cbar_ax, vmin=-5.5, vmax=5.5, xticklabels=rois, yticklabels=freqs)
ax4.invert_yaxis()
ax4.set_xticklabels(ax4.get_xticklabels(), rotation=90)
ax4.set_yticklabels(ax4.get_yticklabels(), rotation=0)


f.savefig('figs/univ_results.eps',dpi=500)