import os
import numpy as np
import nibabel as nib
from surfer import Brain

subject_id = "average"
hemi = "lh"
surface = "pial"

brain = Brain(subject_id, hemi, surface, background="white")

aparc_file = os.path.join(os.environ["SUBJECTS_DIR"],
                          subject_id, "label",
                          hemi + ".aparc.annot")

labels, ctab, names = nib.freesurfer.read_annot(aparc_file)

rois = ['parsopercularis','parsorbitalis','parstriangularis',
        'caudalmiddlefrontal','rostralmiddlefrontal',
        'superiorfrontal',
        'middletemporal','inferiortemporal',
        'inferiorparietal','supramarginal',
        'superiorparietal','precuneus',
        'lateraloccipital','lingual','cuneus','pericalcarine']

stat_to_plot = [1, 1, 1,
                2, 2,
                3,
                4, 4,
                5, 5,
                6, 6,
                7, 7, 7, 7] 

roi_data = []
for n in names: # for each roi in the parcellation
    if np.any([np.sum(r==n) for r in rois]): #if this roi is to be displayed/has a stat
        roi_data.append(stat_to_plot[np.where(n==np.array(rois))[0][0]])
    else:
        roi_data.append(0.)


vtx_data = np.array(roi_data)[labels]
brain.add_data(vtx_data, 1.0, 7.0, thresh=0.2, colormap="Paired", alpha=1, colorbar=None)
brain.save_image('figs/left_rois.png') # change to where you want to save

hemi = "rh"
brain = Brain(subject_id, hemi, surface, background="white")

aparc_file = os.path.join(os.environ["SUBJECTS_DIR"],
                          subject_id, "label",
                          hemi + ".aparc.annot")

labels, ctab, names = nib.freesurfer.read_annot(aparc_file)

roi_data = []
for n in names: # for each roi in the parcellation
    if np.any([np.sum(r==n) for r in rois]): #if this roi is to be displayed/has a stat
        roi_data.append(stat_to_plot[np.where(n==np.array(rois))[0][0]])
    else:
        roi_data.append(0.)


vtx_data = np.array(roi_data)[labels]
brain.add_data(vtx_data, 1.0, 7.0, thresh=0.2, colormap="Paired", alpha=1, colorbar=None)
brain.save_image('figs/right_rois.png')